/* decode les donnees tektro  sur stdin et genere PostScript sur stdout
*/
#include <inrimage/image.h>
#include <stdio.h>
#include <ctype.h>
#include <inrimage/tek.h>

extern errno;
extern int debug_;

static char Ucmd[]=
"[-D] [-L] [-C] [-sc n] [-b] [-ce] [-ntext] [-stat]";
static char Udetail[]=
"\ttransforme code tektro (stdin) en PostScript (stdout)\n\
\t-ntext : suppression des textes\n\
\t-L : landscape\n\
\t-C : trace un cadre\n\
\t-b : calcule la bounding box exacte\n\
\t-sc n: facteur d'agrandissement\n\
\\t-ce: centrer dans la page A4\n\
\t-stat : impression du format du dessin sur stdout";

struct paper a4= {8.26 * INCH_PTS, 11.69 * INCH_PTS};
static int cx= 0, cy= 0;
static float scale= 1.;

short xycadre[]= {0,0,TEK_SIZE,TEK_SIZE};  /* x-y min, x-y max */

short *pxy, *path;
int flg_stat= 0, flg_ntext= 0, flg_cadre= 0, landscape=0;
int flg_center= 0, flg_box= 0;
int Xmin, Ymin, Xmax, Ymax;
FILE *f_in;
int is_pipe;
extern int nbvec;

main(argc,argv)
int argc;
char **argv;
{
	char tempname[32];
	char tmpb[128];
	int xymin[2], xymax[2];
	int v1, v2;
	int w, h;
	int xoff, yoff;
	float sc, sc0;
	register offset;
	float dx, dy, rx0, ry0;
	inr_init(argc,argv,"2.1",Ucmd,Udetail);
	landscape = igetopt("-L","",0,"",0,"",0);
	flg_stat = igetopt("-stat","",0,"",0,"",0);
	flg_center = igetopt("-ce","",0,"",0,"",0);
	flg_ntext = igetopt("-ntext","",0,"",0,"",0);
	if(igetopt("-C","",0,"",0,"",0))
		flg_cadre = 1;
	if(igetopt("-b","",0,"",0,"",0))
		flg_box = 1;
	igetopt("-sc","",0,"%f",&scale,"",0);
	if(tstopts())
		iusage_(Ucmd,Udetail);
	f_in = stdin;   /* stdin */
	Xmin = Ymin = 0;
	Xmax = Ymax = TEK_SIZE -1;
	if(flg_box) { /* 1e passe : calculer min et max */
		/* check if stdin is a pipe */
		is_pipe = lseek(0,(long)0,1) < 0 ? 1 : 0;
		if(is_pipe) {
			if(debug_)
				fprintf(stderr,"Creating temp file\n");
			strcpy(tempname,"/tmp/tek2ps.XXXXXX");
			mktemp(tempname);
			if((f_in = fopen(tempname,"w+")) == NULL)
				imerror("tempfile create error %d\n",errno);
			unlink(tempname);
		}
	}
	if(flg_stat)
		is_pipe = 0;
	if(flg_box || (flg_stat && !flg_cadre))
		tk_pass1();
	xycadre[0] = Xmin;
	xycadre[1] = Ymin;
	xycadre[2] = Xmax;
	xycadre[3] = Ymax;
	is_pipe = 0;
	/* calculer scale factor pour passer de tektro a A4 */
	ps_tekscale(a4.width,a4.height,&sc);
	sc *= scale;
	dx = (float)(Xmax - Xmin);  /* en pts tektro */
	dy = (float)(Ymax - Ymin);  /* en pts tektro */
	if(landscape) {
		w = (int)a4.height;
		h = (int)a4.width;
	} else {
		h = (int)a4.height;
		w = (int)a4.width;
	}
	if(flg_center) {
		xoff = ((int)(w / sc) - dx) / 2 - Xmin;
		yoff = ((int)(h / sc) - dy) / 2 - Ymin;
		if(xoff == 0 && yoff == 0)
			flg_center = 0;
		else {
			rx0 = (xoff * sc * INCH_MM) / INCH_PTS; /* en mm */
			ry0 = (yoff * sc * INCH_MM) / INCH_PTS; /* en mm */
		}

	} else {
		xoff = yoff = 0;
	}
	dx *= sc;  /* en pts PostScript */
	dy *= sc;
	if(landscape) {
		xymin[0] = (int)((Ymin + yoff) * sc);
		xymax[1] = (int)(w - (Xmin + xoff) * sc);
		rx0 = dx;
		dx = dy;
		dy = rx0;
		xymin[1] = xymax[1] - (int)dy;
	} else {
		xymin[0] = (int)((Xmin + xoff) * sc);
		xymin[1] = (int)((Ymin + yoff) * sc);
		xymax[1] = xymin[1] + (int)dy;
	}
	xymax[0] = xymin[0] + (int)dx;
	if(!flg_stat) {
		ps_prologue(1,xymin,xymax);
		ps_tekinit(a4.width,a4.height,&sc0);
		if(landscape) {
			printf("0 %d translate -90 rotate\n",(int)(a4.height / sc0));
		}
		if(scale != 1.)
			printf("%f %f scale\n",scale,scale);
		if(flg_center)
			printf("%d %d translate\n",xoff,yoff);
		tekps(a4);
	}
	dx = (dx * INCH_MM) / INCH_PTS;  /* en mm */
	dy = (dy * INCH_MM) / INCH_PTS;
	if(flg_center) {
			sprintf(tmpb,"%fmm x %fmm (at %fmm %fmm)",dx,dy,rx0,ry0);
	} else {
		sprintf(tmpb,"%fmm x %fmm",dx,dy);
	}
	if(flg_stat) {
		printf("%s\n",tmpb);
	} else {
		fprintf(stderr,"%s\n",tmpb);
	}
	
	dx = (float)(Xmax - Xmin) * sc;  /* en pts tektro */
	dy = (float)(Ymax - Ymin) * sc;  /* en pts tektro */
	if(landscape) {
		xymin[0] = (int)((Ymin + yoff) * sc);
		xymax[1] = (int)(w - (Xmin + xoff) * sc);
		rx0 = dx;
		dx = dy;
		dy = rx0;
		xymin[1] = xymax[1] - (int)dy;
	} else {
		xymin[0] = (int)((Xmin + xoff) * sc);
		xymin[1] = (int)((Ymin + yoff) * sc);
		xymax[1] = xymin[1] + (int)dy;
	}
	xymax[0] = xymin[0] + (int)dx;
	if(flg_stat) {
		printf("box (%fmm %fmm) (%fmm %fmm)\n",
		       ((float)xymin[0] * INCH_MM) / INCH_PTS,((float)xymin[1] * INCH_MM) / INCH_PTS,
		       ((float)xymax[0] * INCH_MM) / INCH_PTS,((float)xymax[1] * INCH_MM) / INCH_PTS);
	} else {
		fprintf(stderr,"box (%fmm %fmm) (%fmm %fmm)\n",
			((float)xymin[0] * INCH_MM) / INCH_PTS,((float)xymin[1] * INCH_MM) / INCH_PTS,
			((float)xymax[0] * INCH_MM) / INCH_PTS,((float)xymax[1] * INCH_MM) / INCH_PTS);
	}
	if(!flg_stat) {
		if(nbvec >= 1)
			ps_eop();
		ps_trailer();
	}
	return 0;

}
