#define WITH_DZV
/*  c_apoldzv
	c_apoldzv(A,B,ndim,ct_in,ct_out,nmx,eps,iopfr,iotype,pos_only,smin)
 effectue l'approximation polygonale sur les contours d'un fichier
  de contours "ouvert" par IOUVCT (parametre ct_in)
et range le resultat dans un 2e fichier (parametre ct_out)
 A et B sont 2 tableaux de travail (donnees et resultats pour chaque cont.)
   ndim contient les dimension de A et B
  nmx est le numero maximum de contour
  eps est la marge d'erreur pour l'approximation (en nombre de pas x, y)
   si iopfr=1 les contours sont fermes (on relie 1er et dernier points)
   (en entree chaines DZV, iopfr=1 ==> elimination des contours non fermes )
   iotype : +1 si input type anac (DZV sinon), +0x100 si output anac
 si pos_only > 0 et (iotype & 1), on elimine les contours negatifs
smin : taille min (nb points ou surface selon le type dzv ou anac
*/
#include <stdio.h>
#include <stdlib.h>
#include <inrimage/image.h>
#ifdef WITH_DZV
# include <dzv.h>
#endif
#include <inrimage/contour.h>

extern int errno, debug_;

#define NOTDZVIN 1
#define NOTDZVOUT 0x100

static Fort_int sens= 0, ext= 0, np= 1;
static Fort_int ipar[2]= {0};
static short tete[5]= {0};
static short queue[5]= {0};
static short ferme= 0;
static float moye[2]= {0.};
static short maillon[3]= {0};
static int get_chn_anac(), put_chn_anac();
static void chg_ext();
void polyg_(), polygn_();
#ifdef WITH_DZV
static int get_chn_dzv(), put_chn_dzv();
# define APOLD c_apoldzv
#else
# define get_chn_dzv(x1,x2,x3,x4,x5,x6) 0
# define put_chn_dzv(x1,x2,x3,x4)
# define APOLD c_apold
#endif
static int pos= 0;
static int set_ext= 0;
static short *ptext= NULL;

void APOLD (a,b,ndim,ct_in,ct_out,nmx,eps,iopfr,iotype,pos_only,smin)
short *a, *b;
struct fc_cont *ct_in, *ct_out;
Fort_int *ndim;
int *nmx, *iopfr, *pos_only;
int *iotype;
float *eps;
int *smin;
{
	register int lgra, lgrn, i;
	Fort_int n;
	int num, ndm;
	lgra = lgrn = 0;
	ndm = *ndim * (sizeof (t_aligned)/ sizeof (int4));
	if(*iopfr > 0)
		ferme= 2;
	pos = *pos_only;
	if(*iotype == NOTDZVIN)
		set_ext = 1;
	if(set_ext && pos)
		chg_ext(ct_in,*nmx);
	for(i=0;i<=*nmx;i++) {
		if(*iotype & NOTDZVIN)
			n = get_chn_anac(ct_in,i,&num,a,ndm,*smin);
		else {
			n = get_chn_dzv(ct_in,i,&num,a,ndm,*smin);
			if(*iopfr > 0)
				if(n > 0 && ferme == 0) continue;
		}
		if(n < 0)
			imerror(8,"pas assez de place pour les contours\n");
		else if(n == 0)
			continue;
		lgra += n;   /* calcul ancienne longueur totale */
		if(ferme > 0) {  /* ferme */
			polyg_(a,b,&n,eps);
		} else { /* non ferme */
			polygn_(a,b,&n,eps);
		}
		lgrn += n;
		if(*iotype & NOTDZVOUT) {
			put_chn_anac(ct_out,num,b,n);
		} else {
			put_chn_dzv(ct_out,num,b,n);
		}
	}
	fprintf(stderr,"taille passee de %d a %d\n",lgra,lgrn);
}

/* i = num demande, renvoie numero dans num (= i) */
static int get_chn_anac(ct_in,i,num,a,ndim,smin)
struct fc_cont *ct_in;
int i;
int *num;
int ndim;
short *a;
int smin;
{
	Fort_int n;
	register int s;
	if((n = c_ircttb(ct_in,i,a,ndim)) <= 0) return n;
	*num = i;
	c_irctpa(ct_in,i,&sens,&ext,&n,ipar,1,&np);
	if(debug_)
		fprintf(stderr,"chaine %d (%d), lgr= %d, S= %d\n",i,ext,n,ipar[0]);
	if(pos && sens < 0) return 0;
	s = ipar[0] < 0 ? -ipar[0] : ipar[0];
	if(s < smin) return 0;
	return n;
}

static int put_chn_anac(ct_out,i,b,n)
int i, n;
struct fc_cont *ct_out;
short *b;
{
	if(debug_)
		fprintf(stderr,"ecriture chaine %d, lgr= %d\n",i,n);
	c_wrctev(ct_out,i,ext);
	c_wrctpa(ct_out,i,ipar,np);
	c_wrcttb(ct_out,i,b,n);
}

#ifdef WITH_DZV
/* les DZV */
/* i = num demande, renvoie numero effectif dans num */
static int get_chn_dzv(ct_in,i,num,a,ndim,smin)
int *ct_in, i;
int ndim;
int *num;
register short *a;
int smin;
{
	int n;
	register k;
	if(re_elt(*ct_in,num,&n,tete,queue,&ferme,moye) == 0) return 0;
	de_bug("erreur sur la lecture ") ;
	if(n >= ndim) return -1;
	if(debug_)
		fprintf(stderr,"chaine %d: %d points,fermeture %d,moy, var %f %f\n",*num,n,ferme,moye[0],moye[1]);
	if(n < smin) return 0;
	for(k=0;k<n;k++) {
		/* lecture des maillons */
		if(!re_mai(*ct_in,a)) {
			imerror(1,"Erreur en lecture des maillons\n");
		}
		a += 2;
	}
	return n;
}	

static int put_chn_dzv(ct_out,i,b,n)
int *ct_out, i;
register n;
register short *b;
{
	register k, ist;
	if(debug_)
		fprintf(stderr,"ecriture (DZV) chaine %d, lgr %d\n",i,n);
	if(set_ext) {
		if(ptext != NULL)
			ext = *(ptext+i);
		ipar[1] = ipar[0];  /* surface */
		ipar[0] = ext;
		ist = wr_elt( *ct_out,i,n,tete,queue,&ferme,ipar);
	} else {
		ist = wr_elt( *ct_out,i,n,tete,queue,&ferme,moye);
	}
		if(!ist) imerror(1,"wr_elt mal passe\n") ;
	for(k=0;k<n;k++) {
		maillon[0] = *b++;
		maillon[1] = *b++;
		if(!wr_mai(*ct_out,maillon))
			imerror(1,"pb ecriture maillon\n");
	}
	return 0;
}
#endif /* end WITH_DZV */

/* recalcule les numeros d'enveloppe en eliminant les contours neg. */
static void chg_ext(ct_in,nmax)
int nmax;
struct fc_cont *ct_in;
{
	register int i, j;
	register short *ps;
	int n;
	if(nmax > 0x7fff) {
		fprintf(stderr,"ATTENTION! trop de contours pour changer les numeros d'enveloppe\n");
		free(ptext);
		ptext = NULL;
		return;
	}
	if((ptext = (short *)calloc(nmax,sizeof(short))) == NULL)
		imerror(8,"alloc pb pour table des enveloppes\n");
	ps = ptext;
	for(i=0;i<nmax;i++,ps++) {
		c_irctpa(ct_in,i,&sens,&ext,&n,ipar,1,&np);
		if(sens == 0) continue;
		if(sens < 0)
			*ps = -ext;
		else
			*ps = ext;
	}
	/* eliminer les enveloppes negatives */
	ps = ptext;
	for(i=0;i<nmax;i++,ps++) {
		if((j = *ps) == 0) continue;
		if(j < 0) j = -j;
		if(*(ptext+j) < 0)
			*ps = - *(ptext+j);
	}
}

