/* 
	iouvct, fermct, ctflsh
	c_iouvct, c_fermct, c_ctflsh

	FUNCTION IOUVCT(NOM,AGE,ACCES)
ouverture d'un fichier de contours de nom NOM
AGE = 1 => il faut recreer le fichier
    = 0 => on cree le fichier si n'existe pas et acces en write
ACCES : caracteres r, w , rw ou m, en maj ou min.
	ATTENTION : W et M sur un nouveau fichier imposent
		le nouveau type (iext, lgr, nbpar de type int4 ) 

renvoie 1 pointeur vers une structure associee au contour
*/

#include <inrimage/image.h>
#include <inrimage/contour.h>
#include <stdio.h>
extern int debug_;

static struct fc_cont *tbct[MAXCT];
static struct fc_cont *tbfree[MAXCT];
static int nbfree= 0;
extern char *malloc();

static Fort_int er9= 9, er8= 8;

struct fc_cont *c_iouvct();
void c_ctflsh(), wrindx();

/**
 * @brief Version Fortran de la fonction c_iouvct()
 * IOUVCT(nom,age,acces)
 */
struct fc_cont *iouvct_(nom,age,acces)
char *nom, *acces;
Fort_int *age;
{
	return c_iouvct(nom,(int)*age,acces);
}

/**
 * @brief Ouverture d'un fichier de contours
 * @param nom fichier contour
 * @param age si 1 le fichier est recr, si 0
 *            le fichier est cr et ouvert en criture 
 *            s'il n'existe pas 
 * @param acces mode d'acces, combinaisons de lettre minuscule
 *              ou majuscules suivantes:
 *            - r : lecture,
 *            - w : criture,
 *            - rw : lecture et criture,
 *            - m : modification.
 * @return pointeur vers un descripteur de fichier contours.
 * 
 * @warning les modes W et M sur un nouveau fichier imposent
 * le nouveau type (iext, lgr, nbpar de type int4).
 * @sa c_fermnct() 
 */

struct fc_cont *c_iouvct(nom,age,acces)
char *nom, *acces;
int age;
{
	int fd, ist, inew, flag;
	int mode, ict, i;
	char *p1;
	struct ct_b0 bk0;
	register struct fc_cont *nfct;
	for(ict=0;ict<MAXCT;ict++)
		if(tbct[ict] == 0) break;
	if(ict >= MAXCT) ermesf_(&er8);	/* trop de fichiers ouverts */
	flag = inew = 0;
	switch (*acces) {
	case 'r' :
	case 'R' :
		ist = CT_R;
		mode = 0;
		acces++;
		if(*acces == 'w' || *acces == 'W') {
			ist |= CT_W;
			mode = 2;
			if(*acces == 'W')
				flag = 1;
		}
		break;
	case 'W' :
		flag = 1;
	case 'w' :
		mode = 2;
		ist = CT_W;
		break;
	case 'M' :
		flag = 1;
	case 'm' :
		ist = CT_R | CT_W | CT_MOD;
		mode = 2;
		break;
	default :
		if(debug_) 
			fprintf(stderr,"iouvct : acces incorrect\n");
		ermesf_(&er9);
	}
	if(age == 0) {
		if((fd = open(nom,mode)) > 0) goto do_ini;
		if(mode == 0) {
err:			if(debug_)
				fprintf(stderr,"iouvct: open err on %s\n",nom);
			ermesf_(&er9);
		}
	}
	if((fd = creat(nom,0644)) <= 0) goto err;
	inew = 1;	/* flag nouveau fichier */
	close(fd);
	if((fd = open(nom,mode)) <= 0) goto err;
do_ini:	if(nbfree) {
		nbfree--;
		nfct = tbfree[nbfree];
	} else {
		nfct = (struct fc_cont *) malloc(sizeof (struct fc_cont));
		if(nfct == NULL)
err8:			ermesf_(&er8);
	}
	if(inew) {	/* initialiser tout  et creer 1er sect. index et datas */
		z_buf(nfct,(sizeof (struct fc_cont))/4);
		nfct->c_nbsi = 1;
		nfct->c_nbs = 2;
		nfct->s_ind = nfct->c_sin = nfct->c_si1 = 1;
		nfct->c_dat1 = nfct->c_fdat = 2*NBYTE + 2;
		nfct->s_dat = 2;
		if(flag) {
			nfct->c_magic = CT_MAGIC;
			nfct->fc_typ = 1;
			nfct->fc_posp = 2;
			nfct->fc_posp2 = 4;
		} else {
			nfct->fc_posp = 1;
			nfct->fc_posp2 = 2;
		}
		z_buf(&bk0,(sizeof (struct ct_b0))/4);
		bk0.b0_s0 = nfct->fc_s0;
		xwrit(fd,&bk0,0);
		xwrit(fd,&nfct->fc_indx,1);
		xwrit(fd,nfct->fc_dat,2);
		nfct->fc_nbwr = 3;
		nfct->fc_swap_needed = 0;
	} else {	/* lire les 1ers blocs */
		z_buf(nfct,((char *) nfct->fc_name - ((char *)nfct))/4);
		xread(fd,&bk0,0);
		nfct->fc_swap_needed = 0;
		if(bk0.b0_s0.ct_magic == CT_MAGIC_SWAP)	{		
			nfct->fc_swap_needed = 1;
			if(ist & CT_W)
				imerror(9,"Ecriture + SWAP incompatibles\n");
			swap4buf(&bk0,sizeof(struct ct_s0)/4);
		}

		xread(fd,&nfct->fc_indx,bk0.b0_s0.ct_si1);
		xread(fd,nfct->fc_dat,(bk0.b0_s0.ct_dat1 >> 8));
		if(nfct->fc_swap_needed) {
			swap4buf(&nfct->fc_indx,NBINT4);
			swap4buf(nfct->fc_dat,1);  /* on swap juste le ptr */
		}
		nfct->fc_s0 = bk0.b0_s0;
		nfct->s_ind = nfct->c_si1;
		nfct->s_dat = nfct->c_dat1 >> 8;
		nfct->fc_nbwr = nfct->c_nbs + 1;
		if(nfct->c_magic == CT_MAGIC) {
			nfct->fc_typ = 1;
			nfct->fc_posp = 2;
			nfct->fc_posp2 = 4;
		} else {
			nfct->fc_posp = 1;
			nfct->fc_posp2 = 2;
		}
	}
	tbct[ict] = nfct;
	nfct->fc_numx = -1;
	nfct->fc_num = -1;
	nfct->fc_numd = -1;
	nfct->fc_numf = ict;
	nfct->fc_fd = fd;
	nfct->fc_access = ist;
	p1 = nom;
	for(i=0;i<79;i++)
		if((nfct->fc_name[i] = *p1++) == 0) break;
	return nfct;
}


/**
 * @brief Version FORTRAN de c_fermct()
 * FERMCT(nfct)
 */
fermct_(nfct)
struct fc_cont **nfct;
{
	c_fermct(*nfct);
}

/**
 * @brief Fermeture d'un fichier de contours.
 * @param nfc descripteur de fichier de contours
 * @sa c_iouvct()
 */
void c_fermct(nfc)
struct fc_cont *nfc;
{
int i;
	if((i = chk_fc(nfc)) < 0) return;
	c_ctflsh(nfc);	/* terminer les i/o */
	tbfree[nbfree] = nfc;
	tbct[i] = 0;
	close(nfc->fc_fd);
}

/* test de validit de nfc :
retourne -1 si invalide, numro sinon
*/

int chk_fc(nfc)
struct fc_cont *nfc;
{
int i;
	if((i = nfc->fc_numf) < 0) return -1;
	if(i > MAXCT) return -1;
	if(tbct[i] != nfc) return -1;
	return i;
}

/*
vidage buffers */

ctflsh_(nfct)
struct fc_cont **nfct;
{
	c_ctflsh(*nfct);
}

void c_ctflsh(nfc)
register struct fc_cont *nfc;
{
	struct ct_b0 bk0;
	register fd;
	if(chk_fc(nfc) < 0) return;
	
	fd = nfc->fc_fd;
	wrindx(nfc);
	if(nfc->flg_d) {
		xwrit(fd,nfc->fc_dat,nfc->s_dat);
		nfc->flg_d = 0;
		if(nfc->s_dat == nfc->fc_nbwr)
			nfc->fc_nbwr++;
	}
	if(nfc->flg_s0) {
		xread(fd,&bk0,0L);
		bk0.b0_s0 = nfc->fc_s0;
		xwrit(fd,&bk0,0L);
		nfc->flg_s0 = 0;
	}
}
