/*  
ecritures sur un fichier de contours
 (subroutines)
	wctcom(nfct,icom)		c_wctcom
	wctdim(nfct,ixy0,idimxy)	c_wctdim
	wrctpa(nfct,num,par,npar)	c_wrctpa
	wrctev(nfct,num,iext)		c_wrctev
	wrcttb(nfct,num,icont,lgr)	c_wrcttb
	wrctls(nfct,num,liste,ilist,lgbl,iop)	c_wrctls
	ctdel(nfct,num)			c_ctdel
*/

#include <inrimage/image.h>
#include <inrimage/contour.h>
#include <stdio.h>

#define LGR_T 16

extern int debug_, errno;

static Fort_int er9= 9;

void w_list();
int go_indx();


/**
 * @brief Version Fortran de c_wctcom().
 * subroutine wctcom(nfct,icom)
 */

wctcom_(nfct,icom)
struct fc_cont **nfct;
char *icom;
{
	c_wctcom(*nfct,icom);
}

/**
 * @brief criture d'un commentaire dans un fichier contours.
 * @param nfc pointeur descripteur fichier contours
 * @param icom commentaire  crire (tampon d'au plus 128 caractes).
 */

void c_wctcom(nfc,icom)
struct fc_cont *nfc;
char *icom;
{
	register char *p1, *p2;
	register i;
	struct ct_b0 bk0;
	if(chk_fc(nfc) < 0) ermesf_(&er9);
	if(!(nfc->fc_access & CT_W)) ermesf_(&er9);
	xread(nfc->fc_fd,&bk0,0L);
	p2 = bk0.s_com;
	p1 = icom;
	for(i=0;i<128;i++)
		if((*p2++ = *p1++) == '\0') break;
	xwrit(nfc->fc_fd,&bk0,0L);
}

/** 
 * @brief Version Fortran de x_wctdim()
 *  subroutine wctdim(nfct,ixy0,idimxy)
 */

wctdim_(nfct,ixy0,idimxy)
struct fc_cont **nfct;
Fort_int *ixy0, *idimxy;
{
	c_wctdim(*nfct,ixy0,idimxy);
}

/**
 * @brief crire ou rcrire en commentaire dans le fichier
 *        de contours la taille de la fentre analyse.
 * @param nfc descripteur fichier de contours
 * @param ixy0 position de la fentre (tableau de 2 entiers)
 * @param idimxy dimension de la fentre (tableau de 2 entiers)
 * @sa c_wctcom()
 */

void c_wctdim(nfc,ixy0,idimxy)
struct fc_cont *nfc;
int4 *ixy0, *idimxy;
{
	char icom[128];
	sprintf(icom,"-ix %ld -iy %ld -dx %ld -dy %ld",ixy0[0],ixy0[1],idimxy[0],idimxy[1]);
	c_wctcom(nfc,icom);
}
/*	subroutine wrctev(nfct,num,iext)
ecrire ou reecrire le num d'enveloppe IEXT du contour numero num
du fichier associe a NFCT.
*/

wrctev_(nfct,num,iext)
struct fc_cont **nfct;
int4 *num, *iext;
{
	c_wrctev(*nfct,(int)*num,(int)*iext);
}

/**
 * @brief criture du numro d'enveloppe d'un contour.
 * @param nfc fichier de contours
 * @param n numro du contour
 * @param iext numro d'enveloppe du contour.
 */
void c_wrctev(nfc,n,iext)
struct fc_cont *nfc;
int n, iext;
{
register union hdrdata *ph;
register ist;
	if(chk_fc(nfc) < 0) ermesf_(&er9);
	if(!(nfc->fc_access & CT_W)) ermesf_(&er9);
	go_indx(nfc,n,1);		/* aller sur sect. index */
	if((ist = go_dat(nfc,0,1)) == 0) {		/* aller vers debut data contour courant */
		nfc->c_nb++;	/* incr. nb cont. */
		if(n > nfc->c_nmax) nfc->c_nmax = n;
	}
	if(nfc->fc_typ) {
		gomot(nfc,1);
		nfc->fc_dat[nfc->i_dat] = iext;
	} else {
		ph = (union hdrdata *) &nfc->fc_dat[nfc->i_dat];
		ph->d1.d_env = iext;
	}
	nfc->flg_d = 1;
}

/* ecriture des parametres */
wrctpa_(nfct,num,par,npar)
struct fc_cont **nfct;
Fort_int *num, *par, *npar;
{
	c_wrctpa(*nfct,(int)*num,par,(int)*npar);
}

/**
 * @brief criture des paramtres d'un contour.
 * @param nfc fichier de contours
 * @param n numro du contour
 * @param par tableau de paramtres
 * @param np taille du tableau de paramtres
 *
 * Les paramtres constituent un ensemble d'informations 
 * arbitraires que l'on peut associer  un contour. En pratique, il semble
 * que l'on doive en ajouter qu'une seule : la surface du contour (en nombre de pixels).
 * C'est cette information qui est affiche sous la colonne S par la commande
 * <tt>visc</tt>. Plus important, cette surface est signe et ce signe sert  indiquer
 * le type de contour (positif ou ngatif). Cette information est importante
 * car elle est utilise par la commande <tt>fillc</tt> pour recontruire des images 
 *  partir de fichier de contours: les contours ngatifs correspondent  des
 * trous (donc des rgions noires) et les positifs  des objets (donc des rgions
 * blanches).
 */

void c_wrctpa(nfc,n,par,np)
register struct fc_cont *nfc;
int n, np;
int4 *par;
{
int ist;
register np0;
union hdrdata hdrd;
struct hdrdn hdrdn;
	if(chk_fc(nfc) < 0) ermesf_(&er9);
	if(!(nfc->fc_access & CT_W)) ermesf_(&er9);
	go_indx(nfc,n,1);		/* aller sur sect. index */
	/* aller vers debut param. contour courant */
	if((ist = go_dat(nfc,1,1)) < 0)	{ 
		nfc->c_nb++;	/* incr. nb cont. */
		if(n > nfc->c_nmax) nfc->c_nmax = n;
	}
	if(ist > 0) { /* param. deja presents */
		if(!(nfc->fc_access & CT_MOD)) ermesf_(&er9);
		if(nfc->fc_typ) 
			np0 = nfc->fc_dat[nfc->i_dat];
		else	{
			hdrd.d_l = nfc->fc_dat[nfc->i_dat];
			np0 = hdrd.d2.d_nbpar;
		}
		if(np != np0) ermesf_(&er9);
		gomot(nfc,nfc->fc_posp2);
		putmot(nfc,par,np);
		return;
	}
	nfc->fc_numx = n;
	if(nfc->fc_typ) {
		hdrdn.dn_nbpar = np;
		hdrdn.dn_store = 0;
		putmot(nfc,&hdrdn.dn_nbpar,2);
	} else {
		hdrd.d_l = 0;
		hdrd.d2.d_nbpar = np;
		putmot(nfc,&hdrd,1);
	}
	if(nfc->c_npmax < np)	nfc->c_npmax = np;
	putmot(nfc,par,np);
}
	
/* ecriture d'un tableau de contour */
wrcttb_(nfct,num,icont,lgr)
struct fc_cont **nfct;
Fort_int *num, *icont, *lgr;
{
	c_wrcttb(*nfct,(int)*num,icont,(int)*lgr);
}

/**
 * @brief criture des lments d'un contour.
 * @param nfc fichier de contours
 * @param n numro de contour
 * @param icont tableau des donnes  crire
 * @param lgr nombre d'lments  crire
 *
 * Chaque lment du tableau <i>icont</i> dcrit la colonne
 * (x) dans les deux octets de poid faible et la ligne
 * (y) dans les deux octets de poid fort.
 * @sa c_ircttb()
 */
void c_wrcttb(nfc,n,icont,lgr)
struct fc_cont *nfc;
int n, lgr;
int4 *icont;
{
	union hdrdata hdrd[2];
	struct hdrdn hdrdn;
	register union hdrdata *ph;
	int4 *pl, nm;
	register int lg0, lg2, lgp;
	if(chk_fc(nfc) < 0) ermesf_(&er9);
	if(!(nfc->fc_access & CT_W)) ermesf_(&er9);
	go_indx(nfc,n,1);	/* aller sur bloc index (creation possible) */
	if((lg0 = go_dat(nfc,2,1)) < 0)	/* aller vers debut datas*/
		ermesf_(&er9);
	if(lg0 == 1) {		/* deja present */
		if(!(nfc->fc_access & CT_MOD))
			ermesf_(&er9);	/* err si non modifiable */
		gomot(nfc,0);
		if(nfc->fc_typ) {
			pl = (int4 *) &hdrdn;
			nm = 4;
			gettab(nfc,pl,4);
			lgp = hdrdn.dn_nbpar;
			if((lg0 = lg2 = hdrdn.dn_store) == 0)
				lg0 = hdrdn.dn_lgr;
			hdrdn.dn_lgr = lgr;
		} else {
			pl = (int4 *) hdrd;
			nm = 2;
			gettab(nfc,pl,2);
			if((lg0 = lg2 = hdrd[1].d2.d_store) == 0) 
				lg0 = hdrd[0].d1.d_lgr;
			hdrd[0].d1.d_lgr = lgr;
			lgp = hdrd[1].d2.d_nbpar;
		}
		gomot(nfc,0);
		if(lgr > lg0)
			ermesf_(&er9);	/* not yet implemented */
		if((lgr < lg0) && (lg2 <= 0)) { /* marquer si taille inf. */
			if(nfc->fc_typ)
				hdrdn.dn_store = lg0;
			else
				hdrd[1].d2.d_store = lg0;
		}
		putmot(nfc,pl,nm);
		gomot(nfc,nfc->fc_posp2 + lgp);
		putmot(nfc,icont,lgr);
		nfc->ct_loc = 0;
		return;
	}
	/* datas non encore ecrites */
	if(nfc->ct_loc != 3) ermesf_(&er9);
	if(nfc->fc_numx != n) ermesf_(&er9);
	putmot(nfc,icont,lgr);
	gomot(nfc,0);
	if(nfc->fc_typ)
		nfc->fc_dat[nfc->i_dat] = lgr;
	else {
		ph = (union hdrdata *)&nfc->fc_dat[nfc->i_dat];
		ph->d1.d_lgr = lgr;
	}
	nfc->ct_lgr = lgr;
	nfc->ct_loc = 1;
	nfc->fc_flag = 2;
	nfc->flg_d = 1;
	return;
}


/*
ecriture liste de points
	wrctls(nfct,num,liste,ilist,lgbl,iop)	c_wrctls
* liste = tableau contenant les listes
* ilist = indice de mot ou de bloc (>=1) dans liste (selon iop)
* lgbl = nb mots data par bloc de liste
* iop = 0 s'il faut ignorer les donnees nulles
	le 1er 1/2 mot contient la lgr; les blocs sont chaines par 2e 1/2 mot
  -1 : idem, mais les indices sont des indices de blocs (taille lgbl+1)
*/

wrctls_(nfct,num,liste,ilist,lgbl,iop)
struct fc_cont **nfct;
Fort_int *num, *liste, *ilist, *lgbl, *iop;
{
	c_wrctls(*nfct,(int)*num,liste,(int)*ilist,(int)*lgbl,(int)*iop);
}

void c_wrctls(nfc,n,list,i0,lgbl,iop)
register struct fc_cont *nfc;
int4 *list;
int n, i0, lgbl, iop;
{
	union hdrdata hdrd[2];
	struct hdrdn hdrdn;
	register union hdrdata *ph;
	register int lg0, lg2, lgr, k;
	if((chk_fc(nfc) < 0) || (i0 < 1)) ermesf_(&er9);
	if(!(nfc->fc_access & CT_W)) ermesf_(&er9);
	go_indx(nfc,n,1);	/* aller sur bloc index (creation possible) */
	if((lg0 = go_dat(nfc,2,1)) < 0)	/* aller vers debut datas*/
		ermesf_(&er9);
/*	if((nfc->ct_lgr = lgr = (*(list + i0 -1) >> 16)) <= 0) ermesf_(&er9); */
	k = iop < 0 ? ((i0 -1) * (lgbl+1)) : i0 - 1;
	if((nfc->ct_lgr = lgr = *(unsigned short *)(list + k)) <= 0) ermesf_(&er9);
	if(lg0 == 1) {		/* deja present */
		if(!(nfc->fc_access & CT_MOD))
			ermesf_(&er9);	/* err si non modifiable */
		gomot(nfc,0);
		if(nfc->fc_typ) {
		  gettab(nfc,&hdrdn,4);
		  gomot(nfc,0);
		  lg0 = hdrdn.dn_lgr;
		  if((lg2 = hdrdn.dn_store) > 0) lg0 = lg2;
		  if(lgr > lg0)
		    ermesf_(&er9);	/* not yet implemented */
		  if(lg2 <= 0) hdrdn.dn_store = lg0;
		  hdrdn.dn_lgr = lgr;
		  lg2 = hdrdn.dn_nbpar;
		  putmot(nfc,&hdrdn,4);
		  gomot(nfc,4+lg2);
		} else {
		  gettab(nfc,hdrd,2);
		  gomot(nfc,0);
		  lg0 = hdrd[0].d1.d_lgr;
		  if((lg2 = hdrd[1].d2.d_store) > 0) lg0 = lg2;
		  if(lgr > lg0)
		    ermesf_(&er9);	/* not yet implemented */
		  if(lg2 <= 0) hdrd[1].d2.d_store = lg0;
		  hdrd[0].d1.d_lgr = lgr;
		  lg2 = hdrd[1].d2.d_nbpar;
		  putmot(nfc,hdrd,2);
		  gomot(nfc,2+lg2);
		}
		w_list(nfc,list,i0-1,lgbl,iop);
		nfc->ct_loc = 0;
		return;
	}
	if(nfc->ct_loc != 3) ermesf_(&er9);
	w_list(nfc,list,i0-1,lgbl,iop);
	gomot(nfc,0);
	ph = (union hdrdata *)&nfc->fc_dat[nfc->i_dat];
	if(nfc->fc_typ)
	  ph->d_l = lgr;
	else
	  ph->d1.d_lgr = lgr;
	nfc->ct_lgr = lgr;
	nfc->ct_loc = 1;
	nfc->fc_flag = 2;
	nfc->flg_d = 1;
	return;
}

/* ecriture d'une liste en eliminant les points nuls si iop = 0 */
void w_list(nfc,list,i0,lgbl,iop)
struct fc_cont *nfc;
register int4 *list;
int i0, iop;
int lgbl;
{
	register int4 *pl, *pl1;
	register unsigned short i, lgr, n;

	int4 tmpb[16];
	if(iop < 0) i0 *= lgbl + 1;
	pl = list + i0;
	lgr = nfc->ct_lgr;
	n = 0;
	if(iop <= 0) {
		while (1) {
			i = *((unsigned short *)pl + 1) -1;
			if(iop < 0) i *= lgbl + 1;
			pl++;
/*			i = (*pl++ & 0x0ffff) -1; */
			pl1 = list + i;
			i = lgbl;
			while (i--) {
				if(*pl) {
					tmpb[n++] = *pl;
					if(n >= LGR_T) {
						putmot(nfc,tmpb,LGR_T);
						n = 0;
					}
					if(--lgr <= 0) goto l_end;
				}
				pl++;
			}
			pl = pl1;
		}
l_end:		if(n > 0)
			putmot(nfc,tmpb,n);
		return;
	}
	/* liste sans elimination des 0 */
	while (1) {
		i = (*pl++ & 0x0ffff) -1;
		pl1 = list + i;
		i = lgbl;
		while (i--) {
			tmpb[n++] = *pl;
			if(n >= LGR_T) {
				putmot(nfc,tmpb,LGR_T);
				n = 0;
			}
			if(--lgr <= 0) goto l_end;
			pl++;
		}
		pl = pl1;
	}
}

/* delete contour :
on met 0 dans ptr de datas, sans recuperer la place
*/
ctdel_(nfct,num)
struct fc_cont **nfct;
Fort_int *num;
{
	c_ctdel(*nfct,(int)*num);
}

/**
 * @brief Suppression d'un contour
 * @param nfc fichier de contours
 * @param  n numro du contour  supprimer
 */
void c_ctdel(nfc,n)
register struct fc_cont *nfc;
register int n;
{
register int ist;
	if(chk_fc(nfc) < 0) ermesf_(&er9);
	if(!(nfc->fc_access & CT_MOD)) ermesf_(&er9);
	go_indx(nfc,n,0);		/* aller sur sect. index */
	if((ist = go_dat(nfc,0,0)) <= 0) 		/* aller vers debut data contour courant */
		return;	/* le contour n'existe pas */
	*nfc->pt_datn = 0;
	nfc->flg_i = 1;
	nfc->fc_numd = nfc->ct_loc = 0;
	nfc->c_nb--;
	if(n >= nfc->c_nmax) nfc->c_nmax--;
	nfc->flg_s0 = 1;
}
