/**
 * Par dfaut, cris une image au format donne par
 * l'extension du fichier de sortie : pbm,pgm,ppm,pam.
 * mais on peut forcer un format de sortie avec -pbm,-pgm,-ppm,-pam
 * si le format de l'image d'entree le permet.
 *
 * On peut choisir entre raw (defaut) ou plain -plain
 * sauf pour la sortie pam (option ignore)
 *
 * Codage en plan non support : conversion du premier plan
 * Codage flottant non support : fin du programme
 * Codage packee non support : conversion en 1 ou 2 octet
 * Codage vectorielle : si 3 -> forcage ppm, sinon forcage pam
 * 
 * @TODO
 * extraction colormap au format netpbm
 * conversion image colormap vers PPM
 * commentaires
 * (c) Dominique Brziat 2006
 */

#include <stdio.h>
#include <string.h>
#include <inrimage/image.h>
#ifdef HAVE_NETPBM_SUBDIR
#include <netpbm/pam.h>
#else
#include <pam.h>
#endif
extern int debug_;

char version[] = "1.0.0";
char command[] = "[input][pam-file][format options][-plain]";
char details[] = 
  "Convert Inrimage file in Netpbm file. By default, image is converted\n"
  "depending on outfile name extension :\n"
  "  .pbm : convert to PBM (bitmap)\n"
  "  .pgm : convert to PGM (graymap)\n"
  "  .ppm : convert to PPM (RGB pixmap)\n"
  "  others extension : convert to PAM (newest Netpbm format).\n"
  "However, the format can be forced using following options:\n"
  "  -pbm, -pgm, -ppm, -pam : convert to PBM, PGM, PPM, PAM\n"
  "The output format is raw by defaut and may be forced to\n"
  "plain format (ASCII) using -plain switch.\n"
  "\n"
  "Restrictions:\n"
  " * Multi frame images (NDIMZ>1) are not supported by Netpbm : only the\n"
  "   first frame is converted.\n"
  " * Conversion to PPM format is only possible with RGB image.\n"
  " * Vector images (NDIMV>1) are supported only by PAM format.\n"
  " * Float images are not supported and must be converted to fix\n"
  "   format before Netpbm conversion\n"
  " * Conversion to Netpbm may lossy in some case; for example: \n"
  "   convert an 1-byte image in PBM, an 4-byte image in PGM, ...\n"
;

void pam_init( struct pam *outpam, struct image *nf, int format) {
#ifdef PAM_STRUCT_SIZE
  outpam->size = PAM_STRUCT_SIZE(tuple_type);
#else
  outpam->size = sizeof(struct pam);
#endif
  outpam->len = (outpam->size<sizeof(struct pam)) ?
    outpam->size : sizeof(struct pam);
  outpam->format = format;
  outpam->width = nf->NDIMX;
  outpam->height = nf->NDIMY;
  outpam->depth = nf->NDIMV;
  outpam->maxval = (1 << (nf->BSIZE<0 ? -nf->BSIZE : nf->BSIZE*8))-1;

  if( outpam->maxval > 1 && 
      ( format == PBM_FORMAT || format == RPBM_FORMAT)) {
    fprintf( stderr, "warning: conversion to pbm (dynamic reduction)\n");
    outpam->maxval = 1;
  }

  if( outpam->maxval > 65535 && 
      ( format == PGM_FORMAT || format == RPGM_FORMAT)) {
    fprintf( stderr, "warning: conversion to pgm (dynamic reduction)\n");
    outpam->maxval = 1;
  }

  if( format == PAM_FORMAT) {
    if( outpam->depth == 3)
      strcpy( outpam->tuple_type, "PAM_PPM_TUPLETYPE");
    else if( outpam->maxval == 1)
      strcpy( outpam->tuple_type, "PAM_PBM_TUPLETYPE");
    else
      strcpy( outpam->tuple_type, "PAM_PGM_TUPLETYPE");
  }
}


int main( int argc, char **argv) {
  char name[256], *p;
  struct image *nf;
  Fort_int lfmt[9];
#if PAM_FLOAT_ACCESS
  tuplen *tbuf;
#else
  tuple *tbuf;
  float *fbuf;
  int col, coef;
#endif
  struct pam outpam;
  int row, format, plain;

  inr_init( argc, argv, version, command, details);
  pnm_init( &argc, argv);

  infileopt( name);
  nf = image_( name, "e", " ", (NF_fmt*)lfmt);

  if( TYPE == REELLE)
    imerror( 5, "unsupported format (real)");

  format = PAM_FORMAT; /* defaut */
  plain = igetopt0( "-plain");

  /* analyse nom de sortie */
  outfileopt( name);
  p = strrchr( name, '.');
  if( p) {
    if( strstr( p, ".pbm") == p)
      format = plain ? PBM_FORMAT : RPBM_FORMAT;
    else if( strstr( p, ".pgm") == p)
      format = plain ? PGM_FORMAT : RPGM_FORMAT;
    else if( strstr( p, ".ppm") == p)
      format = plain ? PPM_FORMAT : RPPM_FORMAT;
  }

  /* forcage du format */
  if( igetopt0( "-pbm"))
    format = plain ? PBM_FORMAT : RPBM_FORMAT;
  else if( igetopt0( "-pgm"))
    format = plain ? PGM_FORMAT : RPGM_FORMAT;
  else if( igetopt0( "-ppm"))
    format = plain ? PPM_FORMAT : RPPM_FORMAT;
  else if( igetopt0( "-pam"))
    format = PAM_FORMAT;

  /* controle du format */
  if( NDIMV == 3 && format != PAM_FORMAT)
    format = PPM_FORMAT;

  /* dabord du PAM */
  if( debug_) fprintf( stderr, "trace: format=%d plain=%d\n", format, plain);
  pam_init( &outpam, nf, format);
 
  outpam.file = ( *name == '>') ? stdout : fopen( name, "w");

  /* ecriture entete */
  pnm_writepaminit(&outpam);

#if PAM_FLOAT_ACCESS
  tbuf = pnm_allocpamrown( &outpam);
  for( row=0; row<NDIMY; row ++) {
    c_lecflt( nf, 1, (void*)*tbuf);    
    pnm_writepamrown( &outpam, tbuf);     
  }
  pnm_freepamrown( tbuf);
#else
  tbuf = pnm_allocpamrow( &outpam);
  fbuf = (float*)i_malloc( sizeof(float)*DIMX);
  if( TYPE != REELLE)
    coef = (1 <<(BSIZE<0?-BSIZE:8*BSIZE)) - 1;
  else
    coef = 1;
  for( row=0; row<NDIMY; row ++) {
    c_lecflt( nf, 1, (void*)fbuf);
    for( col=0; col<DIMX; col ++)
      (*tbuf)[col] = coef * fbuf[col];
    pnm_writepamrow( &outpam, tbuf);     
  }
  i_Free( (void*)&fbuf);
  pnm_freepamrow( tbuf);
#endif
  if( NDIMZ > 1)
    fprintf( stderr, "multiframe image not supported : write only first frame.\n");
  fclose( outpam.file);
  fermnf_( &nf);

  
  return 0;
}
