/**
 * Netpgm : netpbm, netpbm-devel, netpbm-progs
 * man libpm (netpbm utilities)
 * man libpgm libpbm libppm libpnm
 *
 * pbm : portable bitmap
 * pgm : portable greymap
 * ppm : portable pixmap
 * pnm : portable anymap (inclus pbm,pgm et ppm)
 * pam : portable arbitrary map, une surcouche de pnm qui masque tout (ce qu'il faut utiliser)
 *
 * (c) Dominique Brziat
 */

/*
 * Acces normaliss depuis 04.01.30 / Release 10.20
 * 
 */


#include <inrimage/image.h>
#include <stdio.h>
#ifdef HAVE_NETPBM_SUBDIR
#include <netpbm/pam.h>
#else
#include <pam.h>
#endif

char version[] = "1.0.0";
char command[] = "[pam-file][outfile|-i]";
char details[] = 
  "Converts a Netpbm image (pbm,pgm,ppm,pam) to Inrimage format.\n"
  "  -i : do not convert but print informations found in input header.";

/*
 * read header file 
 */

void pam_read_header( struct pam *inpam, FILE *fp) {
#ifdef PAM_STRUCT_SIZE
#ifdef PAM_HAVE_COMMENT_P
  static const char *comments;

  inpam->comment_p = &comments;
  pnm_readpaminit( fp, inpam, PAM_STRUCT_SIZE(comment_p));
#else
  pnm_readpaminit( fp, inpam, PAM_STRUCT_SIZE(tuple_type));
#endif
#else /* PAM_STRUCT_SIZE */
  pnm_readpaminit( fp, inpam, sizeof(struct pam));
#endif  
}

/* 
 * Print a header file.
 */

void pam_print_info( struct pam *inpam, int *plain) {  
  *plain = 0;
  switch(inpam->format) {
  case RPBM_FORMAT:
    fprintf( stderr, "  format: PBM (raw)\n");  
    break;
  case PBM_FORMAT:
    fprintf( stderr, "  format: PBM\n");  
    *plain = 1;
    break;
  case RPGM_FORMAT:
    fprintf( stderr, "  format: PGM (raw)\n");  
    break;
  case PGM_FORMAT:
    fprintf( stderr, "  format: PGM\n");  
    *plain = 1;
    break;
  case RPPM_FORMAT:
    fprintf( stderr, "  format: PPM (raw)\n");  
    break;
  case PPM_FORMAT:
    fprintf( stderr, "  format: PPM\n");
    *plain = 1;
    break;
  case PAM_FORMAT:
    fprintf( stderr, "  format: PAM\n");  
    break;
  default:
    fprintf( stderr, "  format: unknown (%d)\n", inpam->format);
  }
  
  fprintf( stderr, "  size: %dx%d\n", inpam->width, inpam->height);
  fprintf( stderr, "  depth: %d\n", inpam->depth);
  fprintf( stderr, "  maxval: %d\n", inpam->maxval);
  fprintf( stderr, "  bytes/sample: %d\n", inpam->bytes_per_sample);
#ifdef PAM_HAVE_COMMENT_P
  fprintf( stderr, "  comments: %d\n", inpam->comment_p);
#endif
  fprintf( stderr, "  tuple type : %s\n", inpam->tuple_type);
}

int main( int argc, char *argv[]) {
  char name[256];
  FILE *fp;
  struct pam inpam;
  int eof;

  inr_init( argc, argv, version, command, details);
  pnm_init( &argc, argv);
 
  if( igetopt0( "-i")) {
    int plain;

    while( infileopt( name)) {
      fp = (*name == '<') ? stdin : fopen(name,"rb");

      eof = -1;
      fprintf( stderr, "%s:\n", name);
      do {
	if( eof != -1)
	  fprintf( stderr, "next image in %s:\n", name);
	eof = 1;
	pam_read_header( &inpam, fp);	
	pam_print_info( &inpam, &plain);
	if( !plain) {
	  fseek( fp, inpam.width*inpam.height*inpam.depth*inpam.bytes_per_sample, SEEK_CUR);
	  pnm_nextimage( fp, &eof);
	}
      } while( !eof);
      
      if( fp != stdin) fclose( fp);
    }
  
  } else {
    struct image *nf;
    int row, eof, col;
    Fort_int lfmt[9];
#ifdef PAM_FLOAT_ACCESS
    tuplen *buf;
#else
    tuple  *buf;
    float *fbuf;
#endif
    infileopt( name);
    fp = (*name == '<') ? stdin : fopen(name,"rb");
    
    pam_read_header( &inpam, fp);

    TYPE = FIXE;
    NDIMZ = NDIMV = 1;
    switch(inpam.format) {
    case RPBM_FORMAT:
    case PBM_FORMAT:
      BSIZE = -1; TYPE = PACKEE;
      break;
    case RPGM_FORMAT:
    case PGM_FORMAT:
      BSIZE = inpam.bytes_per_sample; 
      break;
    case RPPM_FORMAT:
    case PPM_FORMAT:
      BSIZE = inpam.bytes_per_sample; 
      NDIMV = 3;
      break;
    case PAM_FORMAT:
      BSIZE = inpam.bytes_per_sample;
      NDIMV = inpam.depth;
      break;
    default:
      imerror( 1, "fatal: format %d unsupporter", inpam.format);
      return 1;
    }

    NDIMX = inpam.width;
    NDIMY = inpam.height;

    DIMX = NDIMX * NDIMV;
    DIMY = NDIMY * NDIMZ;
    EXP  = 0;
    
    outfileopt( name);
    nf = image_( name, "c", " ", (NF_fmt*)lfmt);

#ifdef PAM_HAVE_COMMENT_P
    /* TODO : ecriture des commentaires dans l'entete inrimage */
#endif

#ifdef PAM_FLOAT_ACCESS
    buf = pnm_allocpamrown( &inpam);    
    for( row=0; row<inpam.height; row++) {
      pnm_readpamrown( &inpam, buf);
      c_ecrflt( nf, 1, (void*)*buf);
    }
    pnm_freepamrown( buf);
#else
    buf = pnm_allocpamrow( &inpam);
    fbuf = i_malloc( DIMX * sizeof(float));
    
    for( row=0; row<inpam.height; row++) {
      pnm_readpamrow( &inpam, buf);
      for( col=0; col<DIMX; col++)
	fbuf[col] = (float)(*buf)[col] / inpam.maxval;
      c_ecrflt( nf, 1, (void*) fbuf);
    }
    pnm_freepamrow( buf);
    i_Free( (void*)&fbuf);
#endif
    fermnf_( &nf);


    /* TODO gerer les images multiples */
    pnm_nextimage( fp, &eof);
    if( !eof) fprintf( stderr, "warning: I found an another image ...   \n");
    fclose( fp);

  }

  return 0;
}
