/**
 * A png wrapper for Inrimage using libpng
 * (c) Dominique Brziat 2006
 */

#include <inrimage/image.h>
#include <png.h>
#include <stdio.h>
#include <libgen.h>

extern int debug_;

/**
 * \page inr2png inr2png
 *
 * \section name NAME
 * A que
 * \section synopsys SYNOPSIS
 *    png2inr png-file output [-i][-8]
 *
 * \section desc DESCRIPTION
 * Coucou
 * \section sa SEE ALSO
 *  
 */

/* Based on :
 * /usr/share/doc/libpng-1.2.8/libpng.txt
 */

/* options possibles : 
 *    - transformer les 16 bits en 8 bits (gr par libpng, libpng.txt:765) 
 *      mais Inrimage, xvis peut l'afficher, donc option.
 *    - ne pas tenir compte du canal alpha (gr par libpng)
 *    - inverser ordre des canaux
 */

char version[] = "1.0.0";
char cmd[] = "[png-file][-i|output][options]";
char details[] = "Convert a PNG image in INRIMAGE format."
  "Options are:\n"
  "       -i : just display info on png file\n"
  "       -8 : convert pixel/component values to 8 bits. Works\n"
  "            for any bitcoding (1,2,4,16).\n"
  " -noalpha : Remove the alpha channel (if exist)\n"
  //  "       -C : todo\n"

  "  -hdr=%d : to increase the header file size if needed\n"
  "       -D : print Inrimage and program traces\n";

extern int debug_;

int main( int argc, char **argv) {
  char name[256];
  int  rowbytes, y, bit;
  unsigned char *rowbuf;
  int exit_value = 0;

  /* interface */
  int opt_8;
  int opt_noalpha;
  int opt_C;
  int show_header;

  /* inrimage side */
  struct image *nf;
  Fort_int lfmt[9];
  
  /* pnd side */
  int colortype;
  FILE *fp;
  unsigned char sig[8]; /* magic number */
  png_structp png;
  png_infop info;
  
  inr_init( argc, argv, version, cmd, details);

  show_header = igetopt0( "-i");
  opt_8 = igetopt0( "-8");
  opt_noalpha = igetopt0( "-noalpha");
  opt_C = igetopt0( "-C"); /* forcer en mode RGB - pour gray ou indexed (libpng.txt:884) */ 
  
  infileopt( name);

  do {
    fp = *name!='<' ? fopen( name, "rb") : stdin;
    if( !fp) {
      fprintf( stderr, "png2inr:error: can not open file %s\n", name);
      exit_value=1;
      continue;
    }
    
    /* check that is regular png file */
    fread( sig, 1, 8, fp);
    if(!png_check_sig(sig, 8)){
      fprintf( stderr, "png2inr:error:not a valid PNG file (bad signature)\n");
      fclose( fp);
      exit_value=1;
      continue;
    }
  
    /* create required structures for reading png */
    png = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);      
    if( !png) {
      fprintf( stderr, "Could not create PNG structures (out of memory ?)\n");
      fclose( fp);
      exit_value=1;
      continue;    
    }
    info = png_create_info_struct(png);
    if( !info) {
      png_destroy_read_struct( &png, NULL, NULL);
      fprintf( stderr, "Could not create PNG structures (out of memory ?)\n");
      fclose(fp);
      exit_value=1;
      continue;
    }
  
    /* reading header png file */
    png_init_io(png, fp);
    png_set_sig_bytes( png, 8);
    png_read_info(png, info);

    bit = png_get_bit_depth( png, info);

    /* mettre les switch ici avant affichage info (pour vrif) */ 
    if ( opt_8) {
    
      if( bit < 8)
	// cela ne va pas. Il faudrait aussi modifier le TYPE
	png_set_packing( png);
      else if( bit == 16)
	png_set_strip_16( png);
    }

    /* @TODO: not tested with PNG-file created on little-endian machine */
    if( bit == 16 && ismsbfirst()==0) {
      png_set_swap( png);
      if( debug_) fprintf( stderr, "2-byte image reads on big-endian host machine: image is swapped.\n");
    }

    /* test one day
       if( igetopt0("-test"))
       png_set_packswap( png);
    */
    
    if( opt_noalpha &&  png_get_color_type( png, info) & PNG_COLOR_MASK_ALPHA)
      png_set_strip_alpha( png);

    //  mettre a jour les info selon les choix fait
    //  avec les switch de libpng
    png_read_update_info(png, info);
  

    /* show only information from png file */
    if( show_header) {
      int num, i;
      png_text *ptext=NULL;

      printf( "%s\n", basename(name));
      printf( "      width = %d\n", png_get_image_width( png, info));
      printf( "     height = %d\n", png_get_image_height( png, info));
      printf( "    rowsize = %d\n", png_get_rowbytes( png, info));
      printf( " color type = ");
      switch( png_get_color_type( png, info)) {
      case PNG_COLOR_TYPE_GRAY: printf( "gray\n");break;
      case PNG_COLOR_TYPE_GRAY_ALPHA: printf( "gray + alpha\n");break;
      case PNG_COLOR_TYPE_PALETTE: printf( "indexed\n");break;
      case PNG_COLOR_TYPE_RGB: printf( "rgb\n");break;
      case PNG_COLOR_TYPE_RGB_ALPHA: printf( "rgb + alpha\n");break;
      default: printf( "color type inconnu\n");
      }
      printf( "  bit/value = %d\n", png_get_bit_depth(png,info));
      printf( "value/pixel = %d\n", png_get_channels(png,info));
      printf( "  interlace = %s\n", png_get_interlace_type( png, info)==PNG_INTERLACE_NONE?"none":"adam7");
      png_get_text( png, info, &ptext, &num);
      for( i=0; i<num; i++)
	printf( "    comment = %s %s\n", ptext[i].key, ptext[i].text);
      if( ptext) free(ptext);
 
      fclose( fp);
      png_destroy_read_struct( &png, NULL, NULL);
      //    return 0;
    } 
  } while( show_header && infileopt(name));
  if( show_header || exit_value) return exit_value;

  /* build Inrimage format */
  NDIMX = png_get_image_width( png, info);
  NDIMY = png_get_image_height( png, info);
  NDIMZ = 1;
  NDIMV = png_get_channels( png, info);
  DIMX = NDIMX * NDIMV;
  DIMY = NDIMY * NDIMZ;
  switch( png_get_bit_depth( png, info)) {
  case 1: BSIZE = -1; break;
  case 2: BSIZE = -2; break;
  case 4: BSIZE = -4; break;
  case 8: BSIZE =  1; break;
  case 16: BSIZE = 2; break;
  }
  EXP = 200;
  TYPE = BSIZE<0?PACKEE:FIXE;
  colortype =  png_get_color_type( png, info);

  /* force a large header file for indexed image */
  if( colortype == PNG_COLOR_TYPE_PALETTE) {
    int hdr=8;
    igetopt0x("-hdr=","%d",&hdr);
    if( debug_) fprintf( stderr, "Setting header size to %d bytes\n", 256*hdr);
    set_hdr_min( hdr);
  }

  /* open inrimage output */
  outfileopt( name);
  nf = image_( name, "c", "", (void*)lfmt);

  /* write colormap in header file */
  if( colortype == PNG_COLOR_TYPE_PALETTE) {
    png_color *palette;
    int num_palette;
    unsigned char *r, *g, *b;

    png_get_PLTE( png, info, &palette, &num_palette);
    if( debug_) fprintf( stderr, "found %d colors in colormap\n", num_palette);
    if( num_palette) {
      int i;
      r = (unsigned char *)i_malloc( num_palette);
      g = (unsigned char *)i_malloc( num_palette);
      b = (unsigned char *)i_malloc( num_palette);
      for( i=0; i<num_palette; i++) {
	r[i] = palette[i].red;
	g[i] = palette[i].green;
	b[i] = palette[i].blue;
      }
      iwr_ctrgb( nf, 0, num_palette, (char*)r, (char*)g, (char*)b);
    }
    i_Free( (void*)&r);
    i_Free( (void*)&g);
    i_Free( (void*)&b);
  }

  /* case of comments */
  {
    int num, i;
    png_text *ptext;

    png_get_text( png, info, &ptext, &num);
    for( i=0; i<num; i++) {
      char line[256];
      sprintf( line, "%s:%s\n", ptext[i].key, ptext[i].text);      
      c_iwrcom( nf, line);
    }
  }

  rowbytes =  png_get_rowbytes( png, info);
  if( png_get_interlace_type( png, info)==PNG_INTERLACE_ADAM7)  {
    /* if interlace, we have to full-load image */
    png_bytep *row_pointers;
    if( debug_) fprintf( stderr, "Input is interlace, load whole image.\n");

    rowbuf = (unsigned char *)i_malloc( rowbytes * NDIMY);
    row_pointers = (png_bytep *) i_malloc( NDIMY * sizeof(png_bytep));
    for( y=0; y<NDIMY; y ++)
      row_pointers[y] = rowbuf + y * rowbytes;
    png_read_image( png, row_pointers);
    c_ecr( nf, NDIMY, rowbuf);
  } else {
    /* alloc buffer for one row */
    rowbuf = (unsigned char *)i_malloc( rowbytes );
    for ( y = 0; y < NDIMY; y ++) {
      png_read_row( png, rowbuf, NULL);
      c_ecr( nf, 1, rowbuf);
    }
  }

  fermnf_(&nf);
  i_Free( (void*)&rowbuf);
  fclose( fp);
  png_destroy_read_struct( &png, NULL, NULL);
  //png_destroy_info_struct( &info, NULL);
  return exit_value;
}

