/**
 * (c) 2007 Dominique Brziat
 */

#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <inrimage/gvis.h>
#include "structs.h"
#include "labels.h"

/* dfinitions venant de tools.c */
extern GdkColor gdkcolors[];
void par_str( char *str, struct inrgvis *data, int ix, int iy);

/* fixer les couleurs des contextes graphiques de GDK */
#define SET_COLOR(color, widget)                              \
  gdk_gc_set_rgb_fg_color                                     \
  (widget->style->fg_gc[GTK_WIDGET_STATE (widget)], &gdkcolors[color])

/* liste des widgets gvis */
static GList *list_gvis = NULL; 

/*********************************************
 * CALLBACKS
 *********************************************/

static gboolean
on_gvis_image_expose_event        ( GtkWidget       *widget,
				    GdkEventExpose  *event,
				    gpointer         datatable)
{
  /* On rcupre notre structure de donne */
  struct inrgvis *data = TABLE_GET( datatable, GVIS_INRGVIS);
  struct gvis    *api  = TABLE_GET( datatable, GVIS_API);
  GdkPixmap    *pixmap = TABLE_GET( datatable, GVIS_PIXMAP);
  GdkPixbuf    *pixbuf;

  if( data) {
    Fort_int *lfmt = data->lfmt;
    int dimx, dimy, ix, iy;
    size_t offset;

    
    if( api->flags & OPT_XVIS_SCROLL){
      ix = iy = 0;
      dimx = DIMX;
      dimy = DIMY;
      offset = 0;
    } else {
#if 1
      offset = (data->iz-1) * DIMX * NDIMY;
      ix = iy = 0;
      dimx = NDIMX;
      dimy = NDIMY;
#else
      /* OK :-> Ca marche bien, mais il faut grer tous les cas .. */
      
      /* choix du plan ... (data->iz)
       * choix de la ligne (event->area.y)
       * choix de la colonne (event->area.x) */
      offset = (data->iz-1) * DIMX * NDIMY + event->area.y * DIMX + event->area.x * NDIMV;
      ix = event->area.x;
      iy = event->area.y;
      dimx = MIN(event->area.width,NDIMX);
      dimy = MIN(event->area.height,NDIMY);
#endif
    }

    if( data->reorderv)
      offset += (data->iv-1) * NDIMX;

    if( api->flags & DRAW_PIXMAP) {
      switch( data->display) {
      case DISP_GRAY:
	gdk_draw_gray_image( pixmap,
			     widget->style->fg_gc[GTK_STATE_NORMAL],
			     ix, iy, dimx, dimy,
			     GDK_RGB_DITHER_NORMAL, 
			     data->buf + offset, DIMX);
	break;
      case DISP_INDEXED:
	gdk_draw_indexed_image( pixmap,
				widget->style->fg_gc[GTK_STATE_NORMAL],
				ix, iy, dimx, dimy,
				GDK_RGB_DITHER_NORMAL, 
				data->buf + offset, DIMX,
				/* @FIXME Utiliser GdkRgbCmap */
				(void*)&data->tcol);
	break;
      case DISP_RGB:
	gdk_draw_rgb_image( pixmap,
			    widget->style->fg_gc[GTK_STATE_NORMAL],
			    ix, iy, NDIMX, dimy,			  
			    GDK_RGB_DITHER_NORMAL, 
			    data->buf + offset, DIMX);
	break;
      case DISP_RGBA:
	pixbuf = gdk_pixbuf_new_from_data( data->buf + offset, GDK_COLORSPACE_RGB, TRUE,
					   8, NDIMX, NDIMY, DIMX, NULL, NULL);
	// SET_COLOR( BLUE, widget);
	/* arrire plan */
	gdk_draw_rectangle( pixmap, widget->style->fg_gc[GTK_STATE_NORMAL],
			   TRUE, 0, 0, NDIMX, NDIMY);
	gdk_draw_pixbuf( pixmap,
			 widget->style->fg_gc[GTK_STATE_NORMAL],
			 pixbuf, 0, 0, 0, 0, -1, -1,
			 GDK_RGB_DITHER_NORMAL, 0, 0);
	g_object_unref( G_OBJECT(pixbuf));
	break;
      }
      api->flags &= ~DRAW_PIXMAP;
    }
    gdk_draw_drawable(widget->window,
		      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		      pixmap,
		      event->area.x, event->area.y,
		      event->area.x, event->area.y,
		      event->area.width, event->area.height);
    if( data->x) {
      SET_COLOR( api->roi_color, widget);
      gdk_draw_rectangle( widget->window, 
			  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			  FALSE, 
			  data->ix-1, data->iy-1, data->x-1, data->y-1);
    }
  }
  return FALSE;
}

static void
on_gvis_ndimz_value_changed     (GtkRange        *range,
				 gpointer         data)
{
  gvis_set_frame( data, gtk_range_get_value( range));
}

static void
on_gvis_ndimv_value_changed     (GtkRange        *range,
				 gpointer         data)
{
  gvis_set_component( data, gtk_range_get_value( range));
}

static gboolean
on_gvis_image_motion_notify_event ( GtkWidget       *widget,
				    GdkEventMotion  *event,
				    gpointer         datatable)
{
  struct inrgvis *data = TABLE_GET( datatable, GVIS_INRGVIS);  
  if( data) {
    GtkWidget *statusbar = TABLE_GET( datatable, GVIS_STATUSBAR);
    char str[60];
    par_str( str, data, event -> x + 1, event -> y + 1);
    gtk_statusbar_pop  ( GTK_STATUSBAR(statusbar), 1);
    gtk_statusbar_push ( GTK_STATUSBAR(statusbar), 1, str);
  }
  return FALSE;
}

static gboolean
on_gvis_image_button_press_event ( GtkWidget       *widget,
				   GdkEventButton  *event,
				   gpointer         datatable)
{
  struct inrgvis *data = TABLE_GET( datatable, GVIS_INRGVIS);  
  if( data) {
    char str[60];
    par_str( str, data, event->x+1, event->y+1);
    fprintf( stdout, "%s\n", str);
  }
  return FALSE;
}

/* callback for destroying all gvis window */

static void cb_destroy_toplevel( gpointer data){
  gtk_widget_destroy( gtk_widget_get_toplevel( TABLE_GET(data,GVIS_ROOT)));
}

static gboolean
on_gvis_key_press_event       (GtkWidget       *widget,
			       GdkEventKey     *event,
			       gpointer        datatable) {
  gvisapi *data = datatable;
  
  switch( event->keyval) {
  case 'r':
    gtk_widget_queue_draw( TABLE_GET( datatable, GVIS_IMAGE));
    break;
    
  case 'q':
    /* @FIXME faire unattach - puis destruction des image puis des widgets */
    if( event->state & GDK_CONTROL_MASK)
      g_list_foreach( list_gvis, (GFunc) cb_destroy_toplevel, NULL);
    else
      gtk_widget_destroy( gtk_widget_get_toplevel(widget));
    break;

  case 'z':
    gtk_button_clicked( GTK_BUTTON(TABLE_GET(datatable,GVIS_LOOPZ)));
    break;
    
  case 'v':
    gtk_button_clicked( GTK_BUTTON(TABLE_GET(datatable,GVIS_LOOPV)));
    break;

    /* @FIXME : interdire la permutation des images en mode -i */
  case ' ':
  case 'n':
    if( event->state & GDK_MOD1_MASK)
      gvis_next_component( data);
    else if( event->state & GDK_CONTROL_MASK)
      gvis_next_frame( data);
    else 
      gtk_button_clicked( GTK_BUTTON(TABLE_GET(datatable,GVIS_NEXTI)));
    break;

  case 'p':
    if( event->state & GDK_MOD1_MASK)
      gvis_prev_component( data);
    else if( event->state & GDK_CONTROL_MASK)
      gvis_prev_frame( data);
    else
      gtk_button_clicked( GTK_BUTTON(TABLE_GET(datatable,GVIS_PREVI)));
    break;

    
  }
  return FALSE;
}

static void on_gvis_loopz_clicked (GtkButton *button, gpointer   data) {
  gvis_set_zloop_pause( (gvisapi*) data);
}

static void on_gvis_loopv_clicked (GtkButton *button, gpointer   data) {
  gvis_set_vloop_pause( (gvisapi*) data);
}

static void on_gvis_inrgvis_prev (GtkButton *button, gpointer data) {
  inrgvis* image = gvis_inrgvis( data);
  gvis_unattach( data);
  gvis_attach( data, inrgvis_prev( image));
  gvis_window_adjust( data, GVIS_ADJUST_TITLE);
}

static void on_gvis_inrgvis_next (GtkButton *button, gpointer data) {
  inrgvis* image = gvis_inrgvis( data);
  gvis_unattach( data);
  gvis_attach( data, inrgvis_next( image));
  gvis_window_adjust( data, GVIS_ADJUST_TITLE);
}


static void
on_gvis_destroy               (GtkObject       *object,
			       gpointer         datatable)
{
  gvis_delete( (gvisapi*)datatable);
  /** @FIXME */
  if( list_gvis == NULL) 
    gtk_main_quit();
}

/***************************************
 * Widgets / Interface
 ***************************************/

/* fonction utilisatrice de gtk_gvis_new() :
 * rcupre une image du stock et la redimensionne */
static GdkPixbuf * get_scaled_icon( GtkWidget *widget, gchar *stock, gint w, gint h) {
  GdkPixbuf *pixbuf, *scaled; 
  
  pixbuf = gtk_widget_render_icon ( widget, stock, (GtkIconSize)-1 , NULL);
  scaled = gdk_pixbuf_scale_simple ( pixbuf, w, h, GDK_INTERP_BILINEAR);
  g_object_unref (pixbuf);
  return scaled;
}

/* remarque sur l'api
 * les fonctions gtk_gvis_*() utilise un gtkwidget comme premier argument
 * les fonction  gvis_*() utilise un gvisapi comme premier argument
 */

static void set_tip( GtkWidget *widget, char *tip) {
  GtkTooltips* tooltip;
  tooltip = gtk_tooltips_new();
  gtk_tooltips_set_tip( tooltip, widget, tip, NULL);
  gtk_tooltips_enable( tooltip);
}

/**
 * @brief Crer un widget gvis.
 * @param mode masque sur les valeurs suivantes :
 *   - <b>GVIS_WIDGET_TOOLBAR</b> affiche une barre d'outil
 *   - <b>GVIS_WIDGET_STATUSBAR</b> affiche une barre de status (coordonnes 
 *                            et valeur du pixel visit par la souris).
 *
 * Cette fonction cre un widget gvis prt  tre insrer dans une
 * hirarchie GTK ou plus simplement dans une fentre.
 */

GtkWidget* gtk_gvis_new( int mode) {
  GtkWidget *gvis_root, *parent, *child, *subchild, *vbox, *widget;
  GHashTable *datatable;
  struct gvis *api;
  GtkRequisition req;
  
  
  /* cration d'une table de hachage qui va contenir
   * les donnes associes et les adresses des widgets importants */
  datatable = g_hash_table_new (g_str_hash, g_str_equal);
  
  gvis_root = vbox = gtk_vbox_new (FALSE, 0);
  g_signal_connect ((gpointer) gvis_root, "destroy",
		    G_CALLBACK (on_gvis_destroy), datatable);
  g_hash_table_insert( datatable, GVIS_ROOT, gvis_root);

  /* L'objet racine d'un widget gvis a comme donne un GVIS_TABLE */
  g_object_set_data( G_OBJECT(gvis_root), GVIS_TABLE, datatable);

  parent = gtk_toolbar_new (); 
  if( mode & GVIS_WIDGET_TOOLBAR) gtk_widget_show (parent);
  gtk_box_pack_start (GTK_BOX (vbox), parent, FALSE, FALSE, 0);
  gtk_toolbar_set_style (GTK_TOOLBAR (parent), GTK_TOOLBAR_BOTH);
  gtk_toolbar_set_show_arrow  (GTK_TOOLBAR (parent), TRUE);
  g_hash_table_insert( datatable, GVIS_TOOLBAR, parent);
  
  child = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (child);
  gtk_container_add (GTK_CONTAINER (parent), child);

  subchild = gtk_button_new();
  gtk_widget_show (subchild);
  gtk_button_set_focus_on_click( GTK_BUTTON(subchild), FALSE);
  gtk_button_set_relief( GTK_BUTTON(subchild), GTK_RELIEF_NONE);
  set_tip( subchild, "image precedente");
  gtk_container_add (GTK_CONTAINER (child), subchild);
  g_signal_connect ((gpointer) subchild, "clicked",
		    G_CALLBACK (on_gvis_inrgvis_prev), datatable);
  g_hash_table_insert( datatable, GVIS_PREVI, subchild);

  widget = gtk_image_new_from_stock( "gtk-go-back", GTK_ICON_SIZE_MENU);
  gtk_widget_show (widget);
  gtk_container_add (GTK_CONTAINER (subchild), widget);

  child = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (child);
  gtk_container_add (GTK_CONTAINER (parent), child);

  subchild = gtk_button_new();
  gtk_widget_show (subchild);
  gtk_button_set_focus_on_click( GTK_BUTTON(subchild), FALSE);
  gtk_button_set_relief( GTK_BUTTON(subchild), GTK_RELIEF_NONE);
  set_tip( subchild, "image suivante");
  gtk_container_add (GTK_CONTAINER (child), subchild);
  g_signal_connect ((gpointer) subchild, "clicked",
		    G_CALLBACK (on_gvis_inrgvis_next), datatable);
  g_hash_table_insert( datatable, GVIS_NEXTI, subchild);

  widget = gtk_image_new_from_stock( "gtk-go-forward", GTK_ICON_SIZE_MENU);
  gtk_widget_show (widget);
  gtk_container_add (GTK_CONTAINER (subchild), widget);


  child = (GtkWidget*) gtk_tool_item_new ();
  gtk_widget_show (child);
  gtk_container_add (GTK_CONTAINER (parent), child);

  subchild = gtk_entry_new ();
  gtk_entry_set_has_frame( GTK_ENTRY(subchild), FALSE);
  gtk_widget_set_sensitive( subchild, FALSE);
  gtk_widget_show (subchild);
  gtk_entry_set_editable( GTK_ENTRY(subchild), FALSE);
  gtk_container_add (GTK_CONTAINER (child), subchild);
  g_hash_table_insert( datatable, GVIS_PAR, subchild);

  /* boite horizontale contenant le scroller de composante et 
   * son bouton de cycle */
  child = gtk_hbox_new (FALSE, 0);
  // gtk_widget_show( child);
  gtk_box_pack_start (GTK_BOX (vbox), child, FALSE, TRUE, 0);
  g_hash_table_insert( datatable, GVIS_BASEV, child);
  /* le scroller */
  subchild = gtk_hscrollbar_new (GTK_ADJUSTMENT (gtk_adjustment_new (1, 1, 1, 1, 1, 1)));  
  gtk_widget_show (subchild);
  gtk_box_pack_start (GTK_BOX (child), subchild, TRUE, TRUE, 0);
  g_hash_table_insert( datatable, GVIS_NDIMV, subchild);
  g_signal_connect ((gpointer) subchild, "value_changed",
                    G_CALLBACK (on_gvis_ndimv_value_changed), datatable);
  /* le bouton (qui contient une icone) */
  gtk_widget_size_request ( subchild, &req);
  subchild = gtk_toggle_button_new();
  //  gtk_button_set_relief( GTK_BUTTON(subchild), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click( GTK_BUTTON(subchild), FALSE);
  gtk_widget_show( subchild);
  set_tip( subchild, "Animer les composantes");
  gtk_widget_set_size_request( subchild,  req.height, req.height);
  gtk_box_pack_end (GTK_BOX (child), subchild, FALSE, TRUE, 0);
  g_hash_table_insert( datatable, GVIS_LOOPV, subchild);
  g_signal_connect ((gpointer) subchild, "clicked",
		    G_CALLBACK (on_gvis_loopv_clicked), datatable);
  widget = gtk_image_new_from_pixbuf(get_scaled_icon( child, GTK_STOCK_MEDIA_PLAY, 
						      req.height-1, req.height-1));
  gtk_widget_show (widget);
  gtk_container_add (GTK_CONTAINER (subchild), widget);


  /* boite horizontale contenant le scroller de plan et l'image */
  child = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (child);
  gtk_box_pack_start (GTK_BOX (vbox), child, TRUE, TRUE, 0);
  parent = child;

  /* scroller de plan contient le scroller et un bouton de cycle */
  child = gtk_vbox_new(FALSE, 0);
  // gtk_widget_show( child);
  gtk_box_pack_start (GTK_BOX (parent), child, FALSE, TRUE, 0);
  g_hash_table_insert( datatable, GVIS_BASEZ, child);

  /* le scroller */
  subchild = gtk_vscrollbar_new (GTK_ADJUSTMENT (gtk_adjustment_new (1, 1, 1, 1, 1, 1))); 
  gtk_widget_show (subchild);
  gtk_box_pack_start (GTK_BOX (child), subchild, TRUE, TRUE, 0);
  g_hash_table_insert( datatable, GVIS_NDIMZ, subchild);
  g_signal_connect ((gpointer) subchild, "value_changed",
                    G_CALLBACK (on_gvis_ndimz_value_changed), datatable);

  /* le bouton (qui contient une icone) */
  gtk_widget_size_request ( subchild, &req);
  subchild = gtk_toggle_button_new();
  //  gtk_button_set_relief( GTK_BUTTON(subchild), GTK_RELIEF_NONE);
  gtk_button_set_focus_on_click( GTK_BUTTON(subchild), FALSE);
  gtk_widget_show( subchild);
  set_tip( subchild, "Animer les plans");
  gtk_widget_set_size_request( subchild,  req.width, req.width);
  gtk_box_pack_end (GTK_BOX (child), subchild, FALSE, TRUE, 0);
  g_hash_table_insert( datatable, GVIS_LOOPZ, subchild);
  g_signal_connect ((gpointer) subchild, "clicked",
		    G_CALLBACK (on_gvis_loopz_clicked), datatable);
  widget = gtk_image_new_from_pixbuf( get_scaled_icon( child, GTK_STOCK_MEDIA_PLAY, 
						       req.width-1, req.width-1));
  gtk_widget_show (widget);
  gtk_container_add (GTK_CONTAINER (subchild), widget);


  /* zone image */

  child = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show ( child);
  gtk_box_pack_start (GTK_BOX (parent), child, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (child), 
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  parent = child;

  child = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (child);
  gtk_container_add (GTK_CONTAINER (parent), child);
  parent = child;

  child = gtk_drawing_area_new ();
  gtk_widget_show (child);
  gtk_container_add (GTK_CONTAINER (parent), child);
  g_hash_table_insert( datatable, GVIS_IMAGE, child);
  g_signal_connect( (gpointer) child, "expose_event",
                    G_CALLBACK (on_gvis_image_expose_event), datatable);  
  g_signal_connect ((gpointer) child, "motion_notify_event",
                    G_CALLBACK (on_gvis_image_motion_notify_event), datatable);
  g_signal_connect ((gpointer) child, "button_press_event",
                    G_CALLBACK (on_gvis_image_button_press_event), datatable);  
  gtk_widget_set_events( child, GDK_POINTER_MOTION_MASK | GDK_BUTTON1_MOTION_MASK | 
			 GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | 
			 GDK_LEAVE_NOTIFY_MASK );


  child = gtk_statusbar_new ();
  if( mode & GVIS_WIDGET_STATUSBAR) gtk_widget_show (child);
  gtk_box_pack_start (GTK_BOX (vbox), child, FALSE, TRUE, 0);
  g_hash_table_insert( datatable, GVIS_STATUSBAR, child);

  api = (struct gvis *) malloc( sizeof( struct gvis));
  api -> flags = 0;
  api -> roi_color = RED;
  api -> zloop_time = api->vloop_time = 10;
  g_hash_table_insert( datatable, GVIS_API, api);

  list_gvis = g_list_append( list_gvis, datatable);
  return gvis_root;
}

/** 
 *  @brief Retourne le descripteur GVIS d'un widget GVIS.
 *  @param widget widget cre par gtk_gvis_new().
 *  @return descripteur GVIS
 *
 *  Cette fonction retourne la structure de donnes associ  un widget
 *  gvis (cr par gtk_gvis_new()). Cette structure sert
 *  d'interface aux fonctions gvis_() de l'API.
 *  @sa gtk_gvis_new()
 */

gvisapi *gtk_gvis_api ( GtkWidget *widget) {
  return g_object_get_data( G_OBJECT( widget), GVIS_TABLE);
}

/**
 *  @brief Retourne la donne image d'un descripteur GVIS
 *  @param gvis descripteur GVIS
 *  @return descripteur image.
 *  
 */
inrgvis* gvis_inrgvis ( gvisapi *gvis) {
  return TABLE_GET(gvis, GVIS_INRGVIS);
}

/**
 * @brief Force un rafraichissement du widget GVIS.
 * @param gvis descripteur GVIS.
 * Le tampon est relu avant que le widget soit rafraichi.
 */
void gvis_refresh( gvisapi *gvis) {
  struct gvis *api = TABLE_GET( gvis, GVIS_API);
  api->flags |= DRAW_PIXMAP;
  gtk_widget_queue_draw( TABLE_GET( gvis, GVIS_IMAGE));
}

/** @brief Calcul la taille d'un widget GVIS.
 *  @param gvis API gvis
 *  @param w,h  taille du widget racine
 *  Cette fonction prend en compte la prcence ventuel
 *  de barre d'outils, de barre de dfilement de plan,
 *  etc.
 */
void gvis_get_size( gvisapi *gvis, int *w, int *h) {
  GtkWidget *widget;
  GtkRequisition req;

  widget = TABLE_GET( gvis, GVIS_IMAGE);
  gtk_widget_size_request( widget, &req);
  *w = req.width;
  *h = req.height;

  widget = TABLE_GET( gvis, GVIS_TOOLBAR);
  if( GTK_WIDGET_VISIBLE(widget)) {
    gtk_widget_size_request( widget, &req);
    *h += req.height;
  }
  widget = TABLE_GET( gvis, GVIS_STATUSBAR);
  if( GTK_WIDGET_VISIBLE(widget)) {
    gtk_widget_size_request( widget, &req);
    *h += req.height;
  }
  widget = TABLE_GET( gvis, GVIS_BASEZ);
  if( GTK_WIDGET_VISIBLE(widget)) {
    gtk_widget_size_request( widget, &req);
    *w += req.width;
  }
  widget = TABLE_GET( gvis, GVIS_BASEV);
  if( GTK_WIDGET_VISIBLE(widget)) {
    gtk_widget_size_request( widget, &req);
    *h += req.height;
  }
}


/**
 * Change les proprits de la fentre hte (actuellement titre et 
 * dimension). Cette fonction est utilise si une simple fentre 
 * contient seulement un widget gvis.
 * @param data
 * @param flags lment  mettre  jour:
 *   - <b>GVIS_ADJUST_TITLE</b>
 *   - <b>GVIS_ADJUST_SIZE</b>
 */


void gvis_window_adjust( gvisapi *data, int flags) {  
  GtkRequisition req;
  GtkWidget *widget;
  struct inrgvis *image = TABLE_GET(data,GVIS_INRGVIS);
  int width, height;
  char title[128], *p;

  widget = gtk_widget_get_toplevel( TABLE_GET( data, GVIS_ROOT));
  
  if( flags & GVIS_ADJUST_SIZE) {
    gvis_get_size( data, &width, &height);
 
    /* controle taille cran */
    if( width > gdk_screen_width()) width = gdk_screen_width();
    if( height > gdk_screen_height()) height = gdk_screen_height();
      
    gtk_window_resize(GTK_WINDOW(widget), width, height);
  }

  if( flags & GVIS_ADJUST_TITLE) {
    /* nouveau titre pour la fentre */
    p = strrchr( image->name, '/');
    sprintf( title, "gvis %s (%s)", VERSION, p ? p + 1: image->name);
    gtk_window_set_title ( GTK_WINDOW(widget), title);  
  }
}

/** 
 * @brief Lier une image  un descripteur GVIS.
 * @param window fentre cre par gvis_new(),
 * @param data structure de donne d'une image au charge par inrgvis_load().
 */

void gvis_attach( gvisapi *data, inrgvis* idata) {
  struct gvis *api  = TABLE_GET( data, GVIS_API);
  struct inrgvis *image = idata;
  GtkWidget *widget;
  GtkAdjustment *adj;
  char title[128], *p;
  GtkRequisition req;

  g_hash_table_insert( data, GVIS_INRGVIS, image);

  /* toolbar */
  sprintf( title, "-x %d -y %d", image->NDIMX, image->NDIMY);
  if( image->NDIMV>1) sprintf( title, "%s %s %d", title, 
			       image->rgb3z?"-z":"-v", image->NDIMV);
  if( image->NDIMZ>1) sprintf( title, "%s -z %d", title, image->NDIMZ);
  switch( image->TYPE){
  case REELLE: sprintf( title, "%s -r", title); break;
  case FIXE: sprintf( title, "%s -f", title); break;
  case PACKEE: sprintf( title, "%s -p", title); break;
  }
  if( image->BSIZE>0)
    sprintf( title, "%s -o %d", title, image->BSIZE);
  else
    sprintf( title, "%s -b %d", title, -image->BSIZE);
  if( image->EXP < 0)
    sprintf( title, "%s -s", title);
  gtk_entry_set_text( GTK_ENTRY(TABLE_GET( data, GVIS_PAR)), title);
  gtk_entry_set_width_chars( GTK_ENTRY(TABLE_GET( data, GVIS_PAR)), strlen(title));
  
  /* drawing area */
  widget = TABLE_GET( data, GVIS_IMAGE);
  gtk_widget_set_size_request( widget, image->NDIMX, image->NDIMY);
  gtk_widget_queue_draw( widget);

  /* scrollers Z et V */
  widget = TABLE_GET( data, GVIS_NDIMZ);
  if( image->NDIMZ > 1) {
    adj = gtk_range_get_adjustment( GTK_RANGE(widget));    
    adj -> lower = 1;
    adj -> upper = image->NDIMZ+1;
    adj -> step_increment = 1;
    adj -> page_increment = image->NDIMZ;
    gtk_widget_show( TABLE_GET( data, GVIS_BASEZ));
  } else
    gtk_widget_hide( TABLE_GET( data, GVIS_BASEZ));
  
  widget = TABLE_GET( data, GVIS_NDIMV);
  if( image->NDIMV > 1 && image->reorderv) {
    adj = gtk_range_get_adjustment( GTK_RANGE(widget));    
    adj -> lower = 1;
    adj -> upper = image->NDIMV+1;
    adj -> step_increment = 1;
    adj -> page_increment = image->NDIMV;
    gtk_widget_show( TABLE_GET( data, GVIS_BASEV));   
  } else
    gtk_widget_hide( TABLE_GET( data, GVIS_BASEV));

  /* utilisation d'un pixmap pour amliorer la vitesse d'affichage */
  /* @TODO En faire une option, car si l'image est trs grande, on risque
   * de manquer de mmoire */
  widget = TABLE_GET( data, GVIS_IMAGE);
  g_hash_table_insert( data, GVIS_PIXMAP, 
		       gdk_pixmap_new( widget->window, image->NDIMX, image->NDIMY, -1));
  api->flags |= DRAW_PIXMAP;
  /* la suite de la gestion du pixmap est dans le callback d'exposition */
}

/**
 * @brief dtache une image d'une fentre gvis.
 * @param window widget racine d'une fentre gvis. 
 */

void gvis_unattach( gvisapi *datatable) {
  GdkPixmap *pixmap;

  pixmap = TABLE_GET( datatable, GVIS_PIXMAP);
  g_object_unref( G_OBJECT(pixmap)); 
  g_hash_table_remove ( datatable, GVIS_INRGVIS);
  /** @FIXME : librer le pixmap */
  g_hash_table_remove ( datatable, GVIS_PIXMAP);
}

/**
 * Positionne  l'cran un plan particulier.
 * @param window widget racine d'une fentre gvis. 
 * @param iz index du plan  afficher.
 */

void gvis_set_frame( gvisapi* data, int iz) {
  struct inrgvis *image = TABLE_GET( data, GVIS_INRGVIS);  
  if ( image) {
    struct gvis *api = TABLE_GET( data, GVIS_API);
    GtkWidget *statusbar = TABLE_GET( data, GVIS_STATUSBAR);
    char str[60];

    image->iz = iz;
    gtk_range_set_value( GTK_RANGE(TABLE_GET( data, GVIS_NDIMZ)),image->iz);
    api->flags |= DRAW_PIXMAP;
    gtk_widget_queue_draw( TABLE_GET( data, GVIS_IMAGE));
    par_str( str, image, 0, 0);
    gtk_statusbar_pop  ( GTK_STATUSBAR(statusbar), 1);
    gtk_statusbar_push ( GTK_STATUSBAR(statusbar), 1, str);
  }
}

/** 
 * @brief Affiche le plan suivante.
 * @param window widget racine d'une fentre gvis. 
 */
void gvis_next_frame( gvisapi *data) {
  struct inrgvis *image = TABLE_GET( data, GVIS_INRGVIS);
  gvis_set_frame( data, image -> iz % image->NDIMZ + 1);
}

/** 
 * @brief Affiche le plan prcdent.
 * @param window widget racine d'une fentre gvis. 
 */
void gvis_prev_frame( gvisapi *data) {
  struct inrgvis *image = TABLE_GET( data, GVIS_INRGVIS);
  if( image -> iz == 1)
    gvis_set_frame( data, image -> NDIMZ );
  else
    gvis_set_frame( data, image -> iz - 1);
}

/* callback (timer) */
static 
gboolean timer_zloop( gpointer data ) {
  struct gvis *api = TABLE_GET(data, GVIS_API);

  if( !g_list_find( list_gvis, data))
    return FALSE;
  gvis_next_frame( data);
  return api->flags & ZLOOP_IS_RUNNING;
}

/** 
 * @brief Fixe le temps d'animation des plans d'une image.
 * @param window widget racine d'une fentre gvis. 
 * @param timer vitesse d'animation (msec) ou -1 pour le dsactiver.
 */
void gvis_set_zloop_time( gvisapi *data, int time) {
  struct gvis *api = TABLE_GET(data,GVIS_API);
  api->zloop_time = time;
}

/** 
 * @param window widget racine d'une fentre gvis. 
 * @brief Arrte ou demare une animation 
 */
void gvis_set_zloop_pause( gvisapi *data) {
  struct gvis *api = TABLE_GET(data,GVIS_API);
  api->flags ^= ZLOOP_IS_RUNNING;
  if( api->flags & ZLOOP_IS_RUNNING)
    g_timeout_add( api->zloop_time, timer_zloop, data);
}

/**
 * @brief Positionne  l'cran une composante particulire.
 * @param window widget racine d'une fentre gvis. 
 */

void gvis_set_component( gvisapi *data, int iv) {
  struct inrgvis *image = TABLE_GET( data, GVIS_INRGVIS);  
  if ( image) {
    struct gvis *api = TABLE_GET( data, GVIS_API);
    GtkWidget *statusbar = TABLE_GET( data, GVIS_STATUSBAR);
    char str[60];

    image->iv = iv;
    gtk_range_set_value( GTK_RANGE(TABLE_GET( data, GVIS_NDIMV)),image->iv);
    api->flags |= DRAW_PIXMAP;
    gtk_widget_queue_draw( TABLE_GET( data, GVIS_IMAGE));
    par_str( str, image, 0, 0);
    gtk_statusbar_pop  ( GTK_STATUSBAR(statusbar), 1);
    gtk_statusbar_push ( GTK_STATUSBAR(statusbar), 1, str);
  }
}

/** 
 * @brief Affiche la composante suivante.
 * @param window widget racine d'une fentre gvis. 
 */
void gvis_next_component( gvisapi *data) {
  struct inrgvis *image = TABLE_GET( data, GVIS_INRGVIS);
  gvis_set_component( data, image -> iv % image->NDIMV + 1);
}


/** 
 * @brief  Affiche la composante prcdente.
 * @param window widget racine d'une fentre gvis. 
 */
void gvis_prev_component( gvisapi *data) {
  struct inrgvis *image = TABLE_GET( data, GVIS_INRGVIS);
  if( image -> iv == 1)
    gvis_set_component( data, image -> NDIMV );
  else
    gvis_set_component( data, image -> iv - 1);
}

static 
gboolean timer_vloop( gpointer data ) {
  struct gvis *api = TABLE_GET( data, GVIS_API);

  if( !g_list_find( list_gvis, data))
    return FALSE;
  gvis_next_component( data);
  return api->flags & VLOOP_IS_RUNNING;
}

/** 
 * @brief Fixe le temps d'animation des composantes d'une image.
 * @param window widget racine d'une fentre gvis. 
 * @param timer vitesse d'animation (msec) ou -1 pour le dsactiver.
 */
void gvis_set_vloop_time( gvisapi *data, int time) {
  struct gvis *api = TABLE_GET(data,GVIS_API);
  api->vloop_time = time; 
}

/** 
 * @brief Arrte ou dmarre une animation de composante.
 * @param window widget racine d'une fentre gvis. 
 */
void gvis_set_vloop_pause( gvisapi *data) {
  struct gvis *api = TABLE_GET(data,GVIS_API);
  api->flags ^= VLOOP_IS_RUNNING;
  if( api->flags & VLOOP_IS_RUNNING)
    g_timeout_add( api->vloop_time, timer_vloop, data);
}

/**
 * @brief Dtruit une fentre gvis.
 * @param window widget racine d'une fentre gvis. 
 * @return widget racine gvis.
 */ 

void gvis_delete(gvisapi *data) {
  struct inrgvis *image = g_hash_table_lookup( (GHashTable *)data, GVIS_INRGVIS);
  
  list_gvis = g_list_remove( list_gvis, data); 
  free( g_hash_table_lookup( data, GVIS_API));  
  if( image)  inrgvis_free( image);
}

/**********************************
 * API LIB (debut)
 */

/**
 * @brief Crer une fentre gvis  partir d'un tampon image  afficher.
 * @param Masque sur les valeurs suivantes :
 *   - <b>GVIS_BUFFER_FLOAT</b> le tampon est flottant normalis (malgr le codage de lfmt). C'est le cas des images charges par la fonction lecflt().
 *   - <b>GVIS_DISPLAY_XVIS</b> affichage compatible xvis.
 *   - <b>GVIS_DISPLAY_NORGB</b> les images RGB(A) sont traites comme des images vectorielles.
 *   - <b>GVIS_WIDGET_TOOLBAR</b> affiche une barre de controle.
 *   - <b>GVIS_WIDGET_STATUSBAR</b> affiche une barre d'tat (valeurs de pixel visits). 
 *
 * Cette fonction cre une fentre GVIS qui contient l'image dcrite
 * par le tampon <i>buf</i> et le tableau de format <i>lfmt</i>.
 * Il faut ensuite appeler la fonction gvis_event() pour grer les
 * vnements GTK jusqu' la fermeture de la fentre.
 *
 * @sa gvis_event().
 */

gvisapi *gvis_show( void *buf, Fort_int lfmt[], int mode) {
  gvisapi *gvis;
  GtkWidget *widget, *window;
  Fort_int gfmt[9];

  static int init = 1;
  
  /* Initialisation GTK */
  if( init) {
    int argc = 0; 
    char *argv[] = {"libgvis",NULL};
    gtk_disable_setlocale (); 
    gtk_init( &argc, (char***)&argv);
    init = 0;
  }

  /* Cration de la fentre GVIS */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_show( window); 
    
  widget = gtk_gvis_new( mode);
  gtk_widget_show( widget);
  gvis = gtk_gvis_api( widget);
  gtk_container_add (GTK_CONTAINER (window), widget);
  
  /* cration donnes et attache */
  memcpy( gfmt, lfmt, sizeof(Fort_int)*9);
  if( mode & GVIS_BUFFER_FLOAT) {
    gfmt[I_BSIZE] = sizeof(float);
    gfmt[I_TYPE] = REELLE;
  }
  gvis_attach( gvis, inrgvis_rbuf( buf, gfmt, mode));
  gvis_window_adjust( gvis, GVIS_ADJUST_TITLE+GVIS_ADJUST_SIZE);
  gvis_keymap( gvis);

  return gvis;
}

/** @brief Gre les venements des fentres gvis 
 */

void gvis_event( void) {
  gtk_main ();  
}

/** temporaire : j'arrive pas a capturer
 * les venements clavier dans un widget hors GtkWindow */
void gvis_keymap( gvisapi *gvis) {
  GtkWidget *widget = TABLE_GET( gvis, GVIS_ROOT);
  g_signal_connect ((gpointer) gtk_widget_get_toplevel(widget), "key_press_event",
  		    G_CALLBACK (on_gvis_key_press_event), gvis);
}


/* test avec simple fork */
#include <unistd.h>

void gvis_fork( void *buf, Fort_int lfmt[]) {
  if( fork() == 0) {
    gvis_show( buf, lfmt, 0);
    gvis_event();
    exit(0);
  }
}

/* test avec thread + essai de communication IPC */
#include <pthread.h>
struct thdata {
  void *buf; 
  Fort_int *lfmt;
  gvisapi *gvis;
};

static void*gvis_thread_func( void *ptr) {
  struct thdata * data = ptr;
  data->gvis = gvis_show( data->buf, data->lfmt, 0);
  gvis_event();
}

gvisapi* gvis_thread( void *buf, Fort_int lfmt[]) {
  int th;
  pthread_t thread;  
  struct thdata data = {buf, lfmt, NULL};
  pthread_create( &thread, NULL, gvis_thread_func, &data);
  while( data.gvis != NULL);  
  return data.gvis;
}




/* eof */

