/*
 * INRDATA - Routine de chargement d'image Inrimage (tous codages)
 *           et adaptation  l'cran (format admissible par GTK).
 * (c) 2006 D.Brziat
 */

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <glib.h>
#include <inrimage/gvis.h>
#include "structs.h"

#include <inrimage/error.h>

/** todo : virer les ACCES_LINE */

static float THRESHOLD = 10;

static GList *list_data = NULL;    /* liste des images charges */

/*
 * Conversion tampon flottant normalis au format GTK (1 octet)
 * On utilise un algorithme maison qui se distingue :
 *  - avec les codages fixes diffrents de 1 octet.
 *  - avec les codages rels non normaliss
 * La transformation est in-place.
 */

static void conv_1o(  float *src, uchar *dst, size_t size, Fort_int *lfmt) {
  float *psrc = src;
  if( TYPE == FIXE) {
    int dyn = 1 << (BSIZE < 0 ? -BSIZE : 8 * BSIZE) - 1;
    while( size --) 
      *dst++ = (uchar) (*src++ * dyn) ;
  } else {    
    int count, not_norma = 0;
           
    for( count = 0; count < size; count ++, psrc++)
      if( *psrc > THRESHOLD) {
	not_norma = 1;
	break;
      }
    if( not_norma)
      while( size --) {
	if( *src < 0)
	  *dst = 0;
	else if( *src < 255)
	  *dst = (uchar) *src;
	else
	  *dst = 255;
	src ++; 
	dst ++;
      }
    else
      while( size --) {
	if( *src < 0)
	  *dst = 0;
	 else if( *src<1)
	   *dst = (uchar)(255 * *src);
	 else
	   *dst = 255;
	src ++; 
	dst ++;
      }
  }
}

/*
 * Rorganise un tampon 1 octet  3 plans en un tampon  3 composantes.
 */

static void z3tov3( uchar *dst, uchar *src, size_t nelem) {
  uchar *gsrc = src + nelem;
  uchar *bsrc = src + 2*nelem;

  while( nelem --) {
    *dst++ = *src++;
    *dst++ = *gsrc++;
    *dst++ = *bsrc++;      
  }
}

/*
 * Rorganise un tampon 1 octet  dimz plans en un tampon  dimz composantes.
 */

static void ztov( uchar *dst, uchar *src, size_t nelem, int dimz) {
  size_t count;
  int z;
  for( count=0; count < nelem; count ++) {
    for( z=0;z<dimz;z++) 
      *dst++ = *(src + z*nelem);
    src ++;
  }
}

/*
 * Rorganise une ligne d'un tampon vectoriel en ... 
 * a1 b1 a2 b2 a3 b3 -> a1 a2 a3 b1 b2 b3
 */

static void vtox( uchar *dest, uchar *src, int dimx, int dimv) {
  int x,v;

  for( x=0;x<dimx;x++) { // while(dimx--)
    for( v=0;v<dimv;v++) {
      *(dest+v*dimx) = *src; 
      src ++;
    }
    dest ++;
  }
}

/*
 * Identique  c_lecflt() sauf que les codages rels double
 * prcision sont supports.
 */

void c_lecdbl( struct image *nf, int line, float *buf) {
  if( nf->TYPE == REELLE && nf->BSIZE == sizeof(double)) {
    double *dbuf, *pdb;
    size_t size;

    /* malloc standard pour controler les erreurs */
    dbuf = (double *) malloc( sizeof(double) * line * nf->DIMX);
    if( dbuf) {
      c_lect ( nf, line, (void*)dbuf);
      for( pdb = dbuf, size = nf->DIMX * line; size; size --)
	*buf ++ = (float) *pdb ++;
      free( dbuf);
    } else {
      int y;
      /* pas assez de memoire : acces par ligne */
      dbuf = (double *) i_malloc( sizeof(double) * nf->DIMX);
      for( y=0; y<line; y++) {
	c_lect( nf, 1, (void*)dbuf);
	for( pdb = dbuf + nf->DIMX*y, size = nf->DIMX; size; size --)
	  *buf ++ = (float) *pdb ++;
      }	
      i_Free( (void*)&dbuf);
    }
  } else
    c_lecflt( nf, line, (void*)buf);
}

/** 
 * @brief Chargement universelle d'une image INRIMAGE.
 *
 * @param nom nom de l'image  charger.
 * @param flags masque de valeur sur : 
 *   - <b>GVIS_DISPLAY_XVIS</b> : mthode d'affichage conforme  xvis,
 *         sinon on utilise une mthode alternative.
 *   - <b>GVIS_DISPLAY_NORGB</b> : traiter les images RGB (ou RGBA)
 *         comme des images vectorielles.
 * @param progress callback pour progression ou <b>NULL</b>.
 * @return structure de donne associe  cette image.
 *
 * L'image est charge entirement en mmoire et convertie au format cran (GTK).
 * Comme cette fonction peut crire des donnes temporaires sur le disque,
 * il faut appeler la fonction inrgvis_clean() pour librer toutes les ressources.
 *
 * Affichage xvis (<b>GVIS_DISPLAY_XVIS</b>).
 * - Les images  codage fixe sont converties directement sur 1 octet. Si le codage est
 *   plus grand qu'un octet, de l'information est donc perdue.
 * - Les images  codage flottant sont multiplies par 255 (dynamique 1 octet) et 
 *   converties sur un octet. Cette mthode donne de bon rsultat sur des images
 *   normalises. Sur des images qui ont des dynamiques plus grande, cela correspond
 *    un seuillage binaire  1.
 * Ces conversions sont celles fates par xvis.
 *
 * Affichage alternatif (~<b>GVIS_DISPLAY_XVIS</b>).
 * - Les images  codage fixe sont divises par la taille de leur codage puis
 *   considres comme des images  codage flottant. Ceci permet de prendre en 
 *   compte toute la dynamique des images (le cas 1 octet est identique dans les
 *   deux modes d'affichage).
 * - Les images  codage flottant sont traites de la faon qu'avec <i>xvis</i>=0
 *   si on ne trouve pas de pixels ayant une valeur dpassant un certain seuil (S>1).
 *   Si ce seuil est dpass, l'image est convertie directement sur 1 octet. En 
 *   consquence, les images qui ont une dynamique sur 256 units (ou proche) sont
 *   mieux traites. Les images ayant une dynamique beaucoup plus grande ne seront
 *   pas mieux visualises et n'chapperont pas  une normalisation (mais c'est une
 *   opration plus lourde, la priorit est la vitesse d'affichage).
 *   Le seuil S est fix empiriquement  10.
 *
 * @sa inrgvis_free()
 */

inrgvis* inrgvis_load( char *nom, int flags, void (*progress)( char fmt[], ...) ) {
  void *buf;
  struct image *nf;
  struct inrgvis *data = (struct inrgvis *)i_malloc( sizeof (struct inrgvis)); 
  Fort_int *lfmt = data->lfmt;  
  int err;
    
  data -> display = 0;
  data -> buf = NULL;
  *data->name = '\0';
  data->rgb3z = FALSE;
  data->sln = FALSE;

  /* Les commandes et les fonctions Inrimages dtestent
   * les blancs dans les noms de fichiers image. On va
   * crer un lien symbolique avec un nom admissible pour
   * Inrimage vers cette image et on travaille avec le
   * lien symbolique.
   */
  
  if( strchr( nom, ' ')) {
    char snom[256], *p, command[256];
    
    /* nom provisoire pour faire un lien symbolique dans /tmp */
    p = strrchr( nom, '/');
    sprintf( snom, "/tmp/%s", p ? p+1 : nom);
    for( p = snom; *p; p++)
      if( *p == ' ') *p = '_';
    
    /* On fait le lien */
    sprintf( command, "ln -sf '%s' %s", nom, snom);
    system( command);
    
    /* nouveau nom de travail */
    nom = strcpy( data->name, snom);
    
    /* on a fait un lien symbolique */
    data -> sln = TRUE;
    
  } else
    strcpy( data->name, nom);

  /*
   * Dtournement des erreurs Inrimage
   */

  /* @TODO faire l'inventaire des erreurs Inrimage */
  // silent_err(); 

  /* dfinition d'un point de reprise */
  c_erreur( err);
  if( err != 0) return NULL;

  /* Vrification si c'est bien une image traitable par Inrimage 
   * @TODO  cause du hack prcdent, on pourrait s'en passer */
  //  if( is_inrimage(nom,2) == 0) return NULL;

  nf = image_( nom, "e", "", (void*)lfmt);

  if( flags & GVIS_DISPLAY_NORGB)
    data -> reorderv = (NDIMV>1);
  else
    /* les images vectorielles sont rorganises (sauf RGB et RGBA) */
    data -> reorderv = ( NDIMV == 2 || NDIMV > 4) ;
    
  data -> display = DISP_GRAY;

  /* on vrifie si l'image a une table de couleurs */
  if( TYPE != REELLE ) { 
    unsigned char r[256],g[256],b[256];
    int start, end;
    
    if( ird_ctrgb( nf, &start, &end, (char*)r, (char*)g, (char*)b, 256) > 0) {      
      int i;

      for( i=0; i< 256; i++) {
	if( i>=start && i<=end) {
	  data->tcol[i][0] = b[i];
	  data->tcol[i][1] = g[i];
	  data->tcol[i][2] = r[i];	 
	  data->tcol[i][3] = 200;
	} else
	  data->tcol[i][0]=data->tcol[i][1]=data->tcol[i][2]=0;
      }
      
      /* Les tables de couleurs de taille diffrente 
       * de 256 doivent tre largies sur 256 . Elles correspondent
       * a des codages diffrent de 1 octet. Le chargement flottant
       * tale les valeurs sur [0,1] puis sur 8 bits.
       */
      if( BSIZE < 0 && BSIZE > -8) {
	int i, j, max = (1 << -BSIZE) - 1;
	for( i=max; i>=0; i--) {
	  /* valeur correspondante sur 8 bits */
	  j = i*255/max;
	  /* rordonnement de la table des couleurs */
	  data->tcol[j][0] = data->tcol[i][0];
	  data->tcol[j][1] = data->tcol[i][1];
	  data->tcol[j][2] = data->tcol[i][2];
	}
	data -> display = DISP_INDEXED;
      } else if( BSIZE >= 1)
	data -> display = DISP_INDEXED;
      /* 
       * Les tables de plus de 256 ne sont pas gres.
       */
    }
  }

  if( data->buf) i_Free( (void*) &data->buf);
  data -> buf = (uchar *) i_malloc(sizeof(uchar)*DIMX*DIMY);

  /* Chargement des images codes sur 1 octet : accs direct */
  if( BSIZE == 1 && TYPE == FIXE) { 
    int z;

    if ( data->reorderv) {
      int y;
      uchar *line ;

      line = (uchar *) i_malloc( DIMX*NDIMY);
      for( z=0; z<NDIMZ; z++) {
	if( progress && NDIMZ>1) (*progress)( "Chargement plan %d/%d", z+1,NDIMZ);
	c_lect( nf, NDIMY, (void*) line);
 	for( y=0; y<NDIMY; y++)
	  vtox( data->buf + (y + z*NDIMY)*DIMX, line + y*DIMX, NDIMX, NDIMV);	
      }
      i_Free( (void*)&line);
    } else
      for(z=0; z<NDIMZ; z++) {
	if( progress && NDIMZ>1) (*progress)( "Chargement plan %d/%d", z+1,NDIMZ);
	c_lect( nf, NDIMY, (void*)data->buf + z*DIMX*NDIMY);
      }
  } else { 
    /* Les autres codages sont traits ici */
    uchar *cline;
    float *line;
    int y, z;
    Fort_int icodi[] = { sizeof(float), REELLE};
    Fort_int icodo[] = { 1, FIXE};
    Fort_int iexpi= EXP, iexpo=200;

    line  = (float *) i_malloc(sizeof(float)*DIMX*NDIMY);
    cline = (uchar *)line;
  
    for( z=0; z<NDIMZ; z++) {
      if( progress && NDIMZ>1) (*progress)( "Chargement plan %d/%d", z+1, NDIMZ);
      c_lecdbl( nf, NDIMY, line);
      if( flags & GVIS_DISPLAY_XVIS)
	c_cnvtbg( (void*)line, (void*)cline, DIMX*NDIMY, icodi, icodo, iexpi, iexpo);	
      else
	conv_1o( line, cline, DIMX*NDIMY, lfmt);

      /* rorganisation des composantes */
      if( data->reorderv) 
	for( y=0; y<NDIMY; y++) /* la routine vtox() ne fonctionne que par ligne */
	  vtox( data->buf + y*DIMX + z*DIMX*NDIMY, cline + y*DIMX, NDIMX, NDIMV);
      else
	memcpy( data->buf + z*DIMX*NDIMY, cline, DIMX*NDIMY*sizeof(uchar));
    }
    i_Free( (void*)&line);
  }

  /* Cas des images RBG codes par plans */
  if( (NDIMZ == 3 || NDIMZ == 4) && NDIMV == 1 && !(flags & GVIS_DISPLAY_NORGB)) {
    /* image RGB organise par plan -> on la rordonne en NDIMV=3 */
    uchar *buf = (uchar *)i_malloc( sizeof(uchar)*DIMX*DIMY);
#if 0
    /* cette fonction a disparu de la libinrimage ! */
    c_tbpenv( buf, data->buf, NDIMX*NDIMY, 8, NDIMZ, 1);
#else
    ztov( buf, data->buf, NDIMX*NDIMY, NDIMZ);
    /* indique que NDIMZ code NDIMV */
    data->rgb3z = TRUE;
#endif
    NDIMV=NDIMZ;
    NDIMZ=1;
    DIMX=NDIMV*NDIMX;
    DIMY=NDIMZ*NDIMY;
    i_Free( (void*) &data->buf);
    data->buf = buf;
    data -> display = NDIMV==3 ? DISP_RGB : DISP_RGBA;
  } else if( NDIMV == 3 && data->reorderv == 0) 
    data -> display = DISP_RGB;    
  else if( NDIMV == 4 && data->reorderv == 0)
    data -> display = DISP_RGBA;
  fermnf_(  &nf);

  data -> x = data -> y = 0;
  data -> ix = data -> iy = 1;
  data -> iv = data -> iz = 1;

  list_data = g_list_append( list_data, data);
  
  return data;
}

/**
 * @brief Libre une structure inrgvis.
 * @param structure de donne  librer.
 * Cette fonction doit tre appele car inrgvis_load()
 * peut crire sur le disque.
 */

void inrgvis_free( inrgvis* image) {
  struct inrgvis* data = image;
  list_data = g_list_remove( list_data, data);
  if( data->sln) unlink( data->name);
  if( data->buf) i_Free( (void*) &data->buf);
  i_Free( (void*)&data);
}

/**
 * @brief Retourne la donne suivante, dans la liste des donnes.
 * @param donne concerne.
 */
inrgvis* inrgvis_next( inrgvis* image) {
  struct inrgvis* data = image;
  GList *elem = g_list_find( list_data, data);
  if( g_list_next( elem))
    return g_list_next( elem)->data;
  else
    return g_list_first( elem)->data;
}

/**
 * @brief Retourne la donne prcdente, dans la liste des donnes.
 * @param donne concerne.
 */
inrgvis* inrgvis_prev( inrgvis* image) {
  struct inrgvis* data = image;
  GList *elem = g_list_find( list_data, data);
  if( g_list_previous( elem))
    return g_list_previous( elem)->data;
  else
    return g_list_last( elem)->data;
}

/** @brief Seuil de conversion codage flottant.
 *  @param thresh valeur de seuil.
 *
 * Pour les conversion d'image en codage flottant, cette
 * fonction fixe le seuil au dela duquel l'image est 
 * ramen entre 0 et 1. Cette fonction affecte globalement
 * les routines concernes.
 */
void inrgvis_set_threshold( float thresh) {
  THRESHOLD = thresh;
}

/** @brief Relit un tampon.
 *  @param image
 *  @param buf
 *  @param xvis voir inrgvis_load().
 */

inrgvis* inrgvis_rrbuf( inrgvis* image, void* buf, int flags) {
  struct inrgvis* data = image;
  Fort_int *lfmt = data->lfmt;
  int z;
  
  if( TYPE == FIXE && BSIZE==1 ) {
    /* pas de conversion */
    if( data->reorderv) {
      int y;
      for( y=0; y<DIMY; y++)
	vtox( data->buf + y*DIMX, (uchar*)buf + y*DIMX, NDIMX, NDIMV);
    } else
      memcpy( data->buf, buf, DIMX*DIMY);
  } else {
    Fort_int icodi[] = { BSIZE, TYPE}, iexpi= EXP;    
    Fort_int icodo[2], iexpo;
    
    /* @TODO traitement par ligne pour conomiser la mmoire */

    /* conversion au format 1 octet */
    if( flags & GVIS_DISPLAY_XVIS) {
      icodo[0] = 1;
      icodo[1] = FIXE;
      iexpo = 200;
      
      if( TYPE == PACKEE) {
	void *ubuf;
	/* cnvtbg sait grer les formats compacts MAIS il dcompacte
	 * dans le buffer d'entre. Comme on ne sait rien sur la
	 * taille du tampon buf, on doit dcompacter dans un tampon
	 * provisoire. */
	icodi[1] = FIXE;
	ubuf = i_malloc( DIMX*DIMY*(7-BSIZE)/8);
	c_unpkbt( buf, ubuf , DIMX*DIMY , -BSIZE );
	c_cnvtbg( ubuf, data->buf, DIMX*DIMY, icodi, icodo, iexpi, iexpo);
	i_Free( (void*)&ubuf);
      } else
	c_cnvtbg( buf, data->buf, DIMX*DIMY, icodi, icodo, iexpi, iexpo);      
    } else if( TYPE==REELLE && BSIZE==sizeof(float)) {
      conv_1o( buf, data->buf, DIMX*DIMY, lfmt);
    } else {
      float *fbuf = (float *)i_malloc( sizeof(float)*DIMX*DIMY);
      icodo[0] = sizeof(float);
      icodo[1] = REELLE;
      iexpo = 0;

      if( TYPE == PACKEE) {
	c_unpkbt( buf, fbuf , DIMX*DIMY , -BSIZE ) ;
	icodi[1] = FIXE;
	c_cnvtbg( fbuf, fbuf, DIMX*DIMY, icodi, icodo, iexpi, iexpo);
      } else
	c_cnvtbg( buf, fbuf, DIMX*DIMY, icodi, icodo, iexpi, iexpo);
      conv_1o( fbuf, data->buf, DIMX*DIMY, lfmt);
      i_Free( (void *)&fbuf);
    }

    /* rorganisation des composantes (si besoin) */
    if( data->reorderv) {
      uchar *cline  = (uchar *) i_malloc(sizeof(uchar)*DIMX);    
      int y;

      for( y=0; y<DIMY; y++) {
	/* la routine vtox() ne fonctionne que par ligne */	
	vtox( cline, data->buf + y*DIMX , NDIMX, NDIMV);
	memcpy( data->buf + y*DIMX, cline, DIMX*sizeof(uchar));
      }
      i_Free( (void*)&cline);
    }
  }

  /* rorganisation des plans (si besoin) */
  if( (NDIMZ == 3 || NDIMZ == 3) && NDIMV == 1 && !(flags & GVIS_DISPLAY_NORGB)) {
    uchar *buf = (uchar *)i_malloc( sizeof(uchar)*DIMX*DIMY);
    ztov( buf, data->buf, NDIMX*NDIMY, NDIMZ);
    i_Free( (void*) &data->buf);
    data->buf = buf;
    NDIMV=NDIMZ;
    NDIMZ=1;
    DIMX=NDIMV*NDIMX;
    DIMY=NDIMZ*NDIMY;
    data->rgb3z = TRUE;
    data -> display = NDIMV==3 ? DISP_RGB : DISP_RGBA;
  } else if( NDIMV == 3 && data->reorderv == 0) 
    data -> display = DISP_RGB; 
  else if( NDIMV == 4 && data->reorderv == 0) 
    data -> display = DISP_RGBA; 
  
  list_data = g_list_append( list_data, data);
  return data;
}

/** @brief Lecture d'un tampon et conversion au format GTK+
 * @param buf adresse du tampon  lire,
 * @param lfmt tableau de format du tampon,
 * @param flags voir parametre <i>flags</i> de inrgvis_load(),
 * @return structure d'image admissible par GVIS.
 * 
 * Lit un tampon, dcrit par lfmt et cr une structure de donne pour gvis 
 * 
 * @todo lire directement le cas 1 octet sans composante (ni RGB-plan)
 */

inrgvis* inrgvis_rbuf( void* buf, Fort_int lfmt[], int flags) {
  struct inrgvis *data = (struct inrgvis*)i_malloc( sizeof (struct inrgvis)); 
  
  data -> display = DISP_GRAY;
  *data->name = '\0';
  data->rgb3z = FALSE;
  data->sln = FALSE;
  memcpy( data->lfmt, lfmt, sizeof(Fort_int)*9);
  data->buf = (uchar*)i_malloc(sizeof(uchar)*DIMX*DIMY);

  if( flags & GVIS_DISPLAY_NORGB)
    data -> reorderv = (NDIMV>1);
  else
    /* les images vectorielles sont rorganises (sauf RGB et RGBA) */
    data -> reorderv = ( NDIMV == 2 || NDIMV > 4) ;

  inrgvis_rrbuf( data, buf, flags);

  /* autres inits */
  data -> x = data -> y = 0;
  data -> ix = data -> iy = 1;
  data -> iv = data -> iz = 1;

  return data;
}

/**
 * @brief Retourne le tableau de format d'une structure image GVIS.
 */
Fort_int* inrgvis_get_lfmt( inrgvis* data) {
  return ((struct inrgvis*)data)->lfmt;
}

/**
 * @brief Retourne le tampon d'une structure image GVIS.
 */
uchar* inrgvis_get_buffer( inrgvis* data) {
  return ((struct inrgvis*)data)->buf;
}
