/**
 * Affichage d'image Inrimage avec GTK.
 * (c) 2007 Dominique Brziat
 */

#include <inrimage/gvis.h>

char version[] = VERSION;
char command[] = 
  "[options-image] [image [image [...]]]\n"
  "       gvis -i image [options-image] [image [image [...]]]";
char details[] = 
  "Affiche une ou plusieurs images ou affiche depuis l'entree standard\n"
  "si aucune image n'est specifiee.\n"
  "Les options sont:\n"
  "  -i : afficher une image par fenetre. Dans ce mode, les options\n"
  "       images ne s'applique qu'a l'image precedente.\n" 
  "options-image:\n"
  "  -zl time : temps d'animation par plan : un plan par time milliseconde\n"
  "    -zloop : demarre l'animation par plan\n"
  "  -vl time : temps d'animation par composante : une composante par time ms\n"
  "    -vloop : demarre l'animation par composante\n"
  "   -a : affichage par un mode alternatif a celui d'xvis (voir doc)\n"
  "-nrgb : les images RGB(A) sont traitees commes des images vectorielle ou\n"
  "        multiplans (selon le codage des composantes RGB)\n"
  "  -nu : affiche uniquement l'image.\n"
;

void set_opt( gvisapi *gvis) {
  int arg1;

  if( igetopt1( "-zl", "%d", &arg1))
    gvis_set_zloop_time( gvis, arg1);

  if( igetopt0( "-zloop"))
    gvis_set_zloop_pause( gvis);

  if( igetopt1( "-vl", "%d", &arg1))
    gvis_set_vloop_time( gvis, arg1);
  if( igetopt0( "-vloop"))
    gvis_set_vloop_pause( gvis);
}

int main( int argc, char **argv) {
  char filename[256];
  float t;
  GtkWidget *window, *widget;  
  gvisapi *gvis;
  int mode;

  inr_init( argc, argv, version, command, details);
  if( igetopt0("--help")) {
    iusage_(command,details); 
    return 0;
  }
  /* version format GNU */
  if( igetopt0("--version")) {
    fprintf( stderr, "%s\n", VERSION);
    return 0;
  }

  gtk_disable_setlocale (); 
  gtk_init (&argc, &argv);

  /* options globales */
  if( igetopt1( "-t", "%f", &t))
      inrgvis_set_threshold( t);
  
  if( igetopt0( "-i")) {
    infileopt( filename);
    do {
      inrgvis* image;
    
      window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
      gtk_widget_show( window); 
      
      if( igetopt0( "-nu"))
	widget = gtk_gvis_new( 0);
      else
	widget = gtk_gvis_new( GVIS_WIDGET_TOOLBAR+GVIS_WIDGET_STATUSBAR);
      gtk_widget_show( widget);
    
      gtk_container_add (GTK_CONTAINER (window), widget);
      gvis = gtk_gvis_api( widget);
      gvis_keymap( gvis);
      
      mode = GVIS_DISPLAY_XVIS;
      if( igetopt0("-a")) mode &= ~GVIS_DISPLAY_XVIS;
      if( igetopt0("-nrgb")) mode |= GVIS_DISPLAY_NORGB;

      image = inrgvis_load( filename, 1-igetopt0( "-a") , NULL);
      if( image) {
	gvis_attach( gvis, image);
	gvis_window_adjust( gvis,GVIS_ADJUST_TITLE+GVIS_ADJUST_SIZE);
	set_opt( gvis);
      }
    } while( infileopt( filename));
  } else {
    // seconde version : une fentre unique (il faut un loop sur fichiers) 
    
    infileopt( filename);

    mode = GVIS_DISPLAY_XVIS;
    if( igetopt0("-a")) mode &= ~GVIS_DISPLAY_XVIS;
    if( igetopt0("-nrgb")) mode |= GVIS_DISPLAY_NORGB;

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_show( window); 

    if( igetopt0( "-nu"))
      widget = gtk_gvis_new( 0);
    else
      widget = gtk_gvis_new( GVIS_WIDGET_TOOLBAR+GVIS_WIDGET_STATUSBAR);
    gtk_widget_show( widget);

    gtk_container_add (GTK_CONTAINER (window), widget);
    gvis = gtk_gvis_api( widget);

    gvis_keymap( gvis);

    do {
      static inrgvis* image = NULL;
      
      if( image == NULL) {
	image = inrgvis_load( filename, mode, NULL);
	if( image) {
	  gvis_attach( gvis, image);
	  gvis_window_adjust( gvis, GVIS_ADJUST_TITLE+GVIS_ADJUST_SIZE);
	  set_opt( gvis);
	}
      } else
	inrgvis_load( filename, mode, NULL);
    } while( infileopt( filename));
  }

  gtk_main ();
  
  return 0;
}
