/*
****************************************************************************
*									   *
*   Giraudon - INRIA sophia (15/11/84)  				   * 
*									   *
*   explication de l'algorithme :					   *
*   -----------------------------					   *
* Ce programme effectue un filtrage moyenne IDIM x KDIM sur une image      *
* d'entree imag flottante de maniere a ne faire qu'un minimum d'operations *
* Utilisation d'une moyenne glissante (scour)				   *
*									   *
* Utilisation d'un vecteur (scol) representant les moyennes partielles	   *
*	    par colonnes						   *
*									   *
* Utilisation d'un vecteur (isort) pour stocker les resultats	 	   *
*									   *
* Parametres d'entree :							   *
* --------------------							   *
* imag   =  vecteur ligne representant l'image				   *
* scol   =  vecteur de dimension mcolo representant les sommes partielles  *
* nlit   =  nombre de lignes a traiter					   *
* nlign  =  nombre de lignes lues (pour mettre des zeros dans isort)       *
* mcolo  =  nombre de colonnes de l'image				   *
* idim   =  taille en lignes du filtre					   *
* kdim   =  taille en colonnes du filtre				   *
* carre  =  idim * kdim							   *
*									   *
*									   *
* Parametres de sortie :						   *
* ---------------------							   *
* isort   =  vecteur ligne representant l'image des moyennes		   *
*									   *
****************************************************************************
*/

#include <stdio.h>
#include <inrimage/image.h>

bcfmoy_(imag,isort,scol,nlit,nlign,mcolo,idim,kdim,carre)
float *carre, scol[], isort[], imag[];
Fort_int *mcolo, *idim, *kdim, *nlit, *nlign ;
{
register float scour, *p_imag, *f_imag, *old_iso, s 
	 , *de_imag, *old_scol , r_carre ;
register int r_mcolo, r_kdim, k ; 
int i ;

	scour = 0.;
	r_mcolo = *mcolo ;
	r_carre = *carre ;
	old_iso = isort ;
	old_scol = scol ;
	r_kdim= *kdim ;
	isort = old_iso +=  (r_kdim / 2) ; /* on pointe sur le 1er pixel */
			                  /* resultat */

/****************************************************
 * Traitement special pour la 1er ligne de resultat *
 ****************************************************/

	/* d'abord calcul des moyennes partielles et courantes
	   sur la matrice idim x kdim				*/

for (k=0 ; k < r_kdim ; k++)
{
	s = 0. ;
	p_imag = imag + k - r_mcolo ;          /* pointeur debut de fichier */
	f_imag = p_imag + (*idim-1) * (r_mcolo); /* pointeur fin de fichier */
	while  ( p_imag <= f_imag )
	{
		s += *(p_imag += r_mcolo) ;  /* sommation par colonnes */
	}				     /* pour les idim lignes */

	scour += ( *(scol+k) = s ) ;        /* mise en somme partielle et */
}					  /* sommation dans moyenne courante*/
*(isort++) = scour / r_carre ;                 /* stockage resultat */
	
	/* ensuite on continue sur les colonnes suivantes */
for (k = r_kdim ; k < r_mcolo ; k++)
{
	s=0. ;
	p_imag = imag + k - r_mcolo ;         /* pointeur debut de fichier */
	f_imag = p_imag + (*idim-1)*(r_mcolo); /* pointeur fin de fichier */
	while  ( p_imag <= f_imag )
	{
		s += *(p_imag += r_mcolo) ;  /* sommation par colonnes */
	}				     /* pour les idim lignes */


/* mise en somme partielle et  sommation dans moyenne courante et
   soutraction de la somme partielle ancienne */

scour += ( *(scol+k) = s ) - *(scol + k - r_kdim) ;
*(isort++) = scour / r_carre ;                  /* stockage resultat */
}

/********************************************************
 * on va maintenant explorer toutes les lignes image    *
 ********************************************************/

for (i = 1 ; i < *nlit ; i++) 
{
	de_imag = f_imag = imag + (i + (*idim-1)) * r_mcolo ;
	p_imag = imag + (i - 1) * r_mcolo ;
 /* on a positionne les pointeurs courant et ancien de imag */
	isort += r_kdim - 1 ;                  
	scol = old_scol ;              /* on revient au debut de scol*/
	scour = 0. ;

   /* boucle sur les 3 premieres colonnes */
	while ( f_imag < (de_imag + r_kdim) ) 
	{		
		scour += ( *(scol++) += ( *(f_imag++) - *(p_imag++) ) ) ;
	}
	*(isort++) = scour / r_carre ; 

 /* maintenant on continue  sur les colonnes suivantes */
	de_imag +=  r_mcolo ;
	while (f_imag < de_imag) 
	{
		scour -= *(scol - r_kdim) ;
		scour += ( *(scol++) += ( *(f_imag++) - *(p_imag++) ) ) ;
		*(isort++) = scour / r_carre ; 
	}
}
}

/*
 ****************************************************************
 *								*
 *       Giraudon  -  INRIA sophia    (20/11/84)		*
 *								*
 * Ce programme permet de mettre a zero les colonnes d'un       *
 * vecteur image resultat d'un filtrage .			*
 *								*
 *	Parametres d'entree :					*
 *      ---------------------					*
 *  - sorti    :  buffer reel image resultat    		*
 *  - mcolo    :  nombre de colonnes de l'image                 *
 *  - nlign    :  nombre de lignes de l'image sorti		*
 *  - kdim2    :  taille de filtre en x divise par 2		*
 *								*
 ****************************************************************
 */

zerfil_(sorti,mcolo,nlign,kdim2)
float sorti[] ;
Fort_int *mcolo,*nlign,*kdim2 ;

{
	register int index, j, i , r_nlign, r_kdim2;
	{
	
	r_nlign = *nlign ;
	index =  *mcolo - ( r_kdim2 = *kdim2 ) ;

	for (i=0 ; i < r_nlign ; i++)
	{
		for (j = 0 ; j < r_kdim2 ; j++)
		{
			*(sorti + index ) = 0. ;
			*(sorti++) = 0. ;
		}
		sorti += index ;
	}
	}
}

