/* LOGBIN
	SUBROUTINE LOGBIN(A,NDIM,NF1,SF1,NF2,SF2,
     !NFO,SFO,LFMT,LFMT2,IOPT,IVAL)
	c_logbin(a,ndim,nf1,nf2,nfo,lfmt,lfmt2,iopt,ival)
***********************************************
*
* A=tableau de travail (resultat sur tableau image 1).
*          de dimension NDIM.
* NF1, NF2 : images input
* NFO : image output
* IOPT =     1   .iand. entre 2 images
*            2   .ior.  entre 2 images
*            3   .ieor. entre 2 images
*            4 .iand. avec la constante IVAL
*            5  .ior. avec la constante IVAL
*            6  .ieor. avec la constante IVAL
*            7  inversion des bit (ieor avec -1)
*
***********************************************
*/
#include <inrimage/image.h>

#define IAND2 0
#define IOR2 1
#define IEOR2 2
#define IAND1 3
#define IOR1 4
#define IEOR1 5
#define IINV 6
#define IOPMAX IINV

void logbin_(a,ndim,nf1,sf1,nf2,sf2,nfo,sfo,lfmt,lfmt2,iopt,ival)
unsigned int *a;
Fort_int lfmt[], lfmt2[], *iopt, *ival, *ndim;
struct image **nf1, **nf2, **nfo;
char *sf1, *sf2, *sfo;
{
	c_logbin(a,(int)*ndim,*nf1,*nf2,*nfo,lfmt,lfmt2,(int)*iopt,(int)*ival);
}

static void do_iand1(), do_ieor1(), do_ior1();
static void do_iand2(), do_ieor2(), do_ior2(), do_inv();
static void (*func[IOPMAX+1])()= {
	do_iand2,
	do_ior2,
	do_ieor2,
	do_iand1,
	do_ior1,
	do_ieor1,
	do_inv
	};
static int icas, idx, nblig, nblig2, lig1, lig2;
static Fort_int ibit2,ibit1,  npt;
c_logbin(a,ndim,nf1,nf2,nfo,lfmt,lfmt2,iopt,ival)
unsigned int *a;
Fort_int lfmt[], lfmt2[];
int ndim;
register int iopt;
register unsigned int ival;
struct image *nf1, *nf2, *nfo;
{
	register nb, nlong;
	Fort_int ibs1, ibs2;
	int ndimbit;
	unsigned int msk, *a2;
	idx= DIMX;
	lig1 = ligbit_(lfmt,&ibs1);
	ibit1 = (BSIZE < 0 && TYPE == FIXE) ?  -BSIZE : ibs1;
	ndimbit = 32 * ndim;
	iopt--;
	if(iopt < 0 || iopt > IOPMAX)
		imerror(9,"logbin: fonction inconnue\n");
	if(iopt >= IAND1) {  /* 1 seule image en entree */
		nblig=ndimbit /lig1;
		if(nblig <= 0) imerror(8,"pas assez de memoire (option -k)\n");
		if(nblig > DIMY) nblig = DIMY;
		msk = (unsigned int)0xffffffff >> (32 - ibs1);
		ival &= msk;
		switch(ibs1) {
		case 1:
			ival |= ival << 1;
		case 2:
			ival |= ival << 2;
		case 4:
			ival |= ival << 4;
		case 8:
			ival |= ival << 8;
		case 16:
			ival |= ival << 16;
		case 32:
			break;
		default:
			imerror(9,"bit size non traitee\n");
		}
		if(iopt == IINV) {  /* inversion de bits */
			ival = -1;
			icas = (BSIZE < 0 && TYPE == FIXE) ? 1 : 0;
		}
		nb = DIMY;
		while(nb > 0) {
			if(nblig > nb) nblig = nb;
			c_lect(nf1,nblig,a);
			(*func[iopt])(a,ival);
			c_ecr(nfo,nblig,a);
			nb -= nblig;
		}
		return 1 /* makes c99 happy */; 
	}
	/* 2 images en entree */
	lig2 = ligbit_(lfmt2,&ibs2);
	ibit2 = ibs2;
	if(lfmt2[I_BSIZE] < 0 && lfmt2[I_TYPE] == FIXE)
		ibit2 = -lfmt2[I_BSIZE];
	icas = 0;
	if(ibs1 != ibs2) {
		if(TYPE == FIXE)
			icas = 1;
		if(lfmt2[I_TYPE] == FIXE)
			icas = 2;
	}
	nblig = ((ibs1 * ndimbit) / (ibs1+ibs2)) / lig1;
	nblig2 = ((ibs2 * ndimbit) / (ibs1+ibs2)) / lig2;
	if(nblig > DIMY) nblig = DIMY;
	if(nblig2 > lfmt2[I_DIMY]) nblig2 = lfmt2[I_DIMY];
	if(nblig2 < nblig) nblig = nblig2;
	if(nblig <= 0) imerror(8,"pas assez de memoire (option -k)\n");
	a2 = a + (nblig * lig1 + 31)/32;
	nb = DIMY;
	while(nb > 0) {
		if(nblig > nb) nblig = nb;
		npt = nblig * idx;
		nlong = icas == 0 ? (npt * ibs1 + 31)/32 : (npt * ibit1 + 31)/32;
		c_lect(nf1,nblig,a);
		if(icas == 1)
			pckbt_(a,a,&npt,&ibit1);
		c_lect(nf2,nblig,a2);
		if(icas == 2)
			pckbt_(a2,a2,&npt,&ibit2);
		(*func[iopt])(a,a2,nlong);
		if(icas == 1)
			unpkbt_(a,a,&npt,&ibit1);
		c_ecr(nfo,nblig,a);
		nb -= nblig;
	}
		
}

static void do_iand1(a,ival)
register u_int4 *a;
register int ival;
{
	register nlong;
	nlong = (nblig * lig1 + 31) / 32;
	while(nlong-- > 0)
		*a++ &= ival;
}
static void do_ior1(a,ival)
register u_int4 *a;
register int ival;
{
	register nlong;
	nlong = (nblig * lig1 + 31) / 32;
	while(nlong-- > 0)
		*a++ |= ival;
}
static void do_ieor1(a,ival)
register u_int4 *a;
register int ival;
{
	register nlong;
	nlong = (nblig * lig1 + 31) / 32;
	while(nlong-- > 0)
		*a++ ^= ival;
}
static void do_inv(a,ival)
register u_int4 *a;
register int ival;
{
	register nlong;
	u_int4 *a0;
	int npt;
	a0 = a;
	npt = nblig * idx;
	nlong = (npt * ibit1 +31) / 32;
	if(icas)
		pckbt_(a,a,&npt,&ibit1);
	while(nlong-- > 0)
		*a++ ^= ival;
	if(icas)
		unpkbt_(a0,a0,&npt,&ibit1);
}

static void do_iand2(a,a2,nlong)
register u_int4 *a, *a2;
register int nlong;
{
	while(nlong-- > 0)
		*a++ &= *a2++;
}

static void do_ior2(a,a2,nlong)
register u_int4 *a, *a2;
register int nlong;
{
	while(nlong-- > 0)
		*a++ |= *a2++;
}

static void do_ieor2(a,a2,nlong)
register u_int4 *a, *a2;
register int nlong;
{
	while(nlong-- > 0)
		*a++ ^= *a2++;
}

