/*
  SUBROUTINE MEDFIL(INBUF,OUTBUF,DIMS,NFI,NFO,IDIM,KDIM))
    INBUF, OUTBUF : buffer  de taille DIMS(1) elements (long) pour donnees
  NFI,NFO : images in et out
  IDIM, KDIM : taille en X et Y de la fenetre
*/
  
/* filt_med
   filt_med(in,out,dx,dy,lglig,nblig,iy)
   unsigned char *in, *out : buffer d'entree et de sortie (pixels 8 bits)
   int dx, dy : taille du filtre
   int lglig, nblig : lgr ligne et nb lignes
   int iy : nb de ligne avant la ligne courante
le 1e point du tableau in est le 1e pixel a traiter (centre de la fenetre)
   Il doit exister iy lignes avant le debut du tableau et (dy - iy - 1) lignes
   apres les nblig lignes a traiter.
 
Pour le debut de l'image on fera dy/2 appels avec nblig=1
(voir exemple dans medfil_)

*/

#include <inrimage/image.h>


medfil_(inbuf,outbuf,dims,nfi,nfo,lfmt,idim,kdim)
unsigned char *inbuf, *outbuf;
struct image  **nfi, **nfo;
Fort_int *dims, *idim, *kdim;
Fort_int *lfmt;
{
	register unsigned char *pi, *po;
	int n;
	int ntot;
	int nbl_read, nbl_wr, nbl1;
	int buf_lg, off;
	register nbmove, yoff;
	register i;
	register dx, dy, dy2;
	int first;
	n = *dims * sizeof(t_aligned);
	buf_lg = n / DIMX;
	if(buf_lg > DIMY)
		buf_lg = DIMY;
	dx = *idim;
	dy = *kdim;
	if(buf_lg < dy)
		imerror(8,"not enough memory for %d lines\n",dy);
	dy2 = dy / 2;
	ntot = DIMY;
	first = 1;
	nbmove = (dy -1) * DIMX;
	yoff = dy2 * DIMX;
	nbl_read = buf_lg;
	off = 0;
	nbl_wr = nbl_read - dy2;
	nbl1 = nbl_wr - dy2;
	while(ntot > 0) {
		c_lect(*nfi,nbl_read,inbuf + off);
		po = outbuf;
		if(first) {
			first = 0;
			pi = inbuf;
			for(i=1;i<=dy2;i++) {
				filt_med(pi,po,dx,dy2 + i,DIMX,1,i-1);
				pi += DIMX;
				po += DIMX;
			}
		} else {
			pi = inbuf + yoff;
			nbl1 = nbl_wr;
		}
		filt_med(pi,po,dx,dy,DIMX,nbl1,dy2);
		c_ecr(*nfo,nbl_wr,outbuf);
		ntot -= nbl_read;
		pi = inbuf + off + (nbl_read * DIMX);
		if(ntot == 0) {/* fin image */
			po = outbuf;
			pi -= yoff;
			for(i=1;i<=dy2;i++) {
				filt_med(pi,po,dx,dy - i,DIMX,1,dy2);
				pi += DIMX;
				po += DIMX;
			}
			c_ecr(*nfo,dy2,outbuf);
			break;
		} else {
			memcpy(inbuf,pi-nbmove,nbmove);
		}
		off = nbmove;
		nbl_read = buf_lg - dy + 1;
		if(nbl_read > ntot)
			nbl_read = ntot;
		nbl_wr = nbl_read;
	}
}

static int tmpcpt[256], tmpcp0[16];
filt_med(in,out,dx,dy,lglig,nblig,iy)
register unsigned char *in, *out;
int dx, dy;  /* taille filtre */
int lglig, nblig;
int iy; /* nb de ligne avant la ligne courante */
{
	register unsigned char *pi;
	register dx2, j, i;
	register n, n2;
	int lglg;
	pi = in - (iy * lglig);
	dx2 = dx/2;
	while(nblig-- > 0) {
		j = dx - dx2;
		n = dy * j;
		lglg = lglig - j;
		for(i=0;i<256;i++)
			tmpcpt[i] = 0;
		for(i=0;i<16;i++)
			tmpcp0[i] = 0;
		median_init(pi,j-1,dy,(n - dy -1)/2,lglig);
		for(i=0;i<=dx2;i++) {
			n2 = (n - 1) / 2;
			*out++ = median(pi,j++,dy,n2,lglig,-1);
			n += dy;
		}
		n = dx * dy;
		lglg = lglig - dx;
		n2 = (n -1) / 2;
		pi++;
		for(;i<(lglig-dx2);i++)
			*out++ = median(pi++,dx,dy,n2,lglig,0);
		j = dx -1;
		for(;i<lglig;i++) {
			n -= dy;
			n2 = (n -1) / 2;
			*out++ = median(pi++,j--,dy,n2,lglig,1);
		}
		
		pi += dx2;
	}
}

median(pi,dx,dy,n2,inclig,flg)
register unsigned char *pi;
register dx, dy, inclig, n2;
int flg;
{
	register i, j, k;
	register v;
	register int *pv;
	if(flg == 0) {  /* points courants dans ligne */
		dx -= 1;
		for(j=0;j<dy;j++) {
			tmpcpt[(v = *(pi -1))] -= 1;
			tmpcp0[v/16] -= 1;
			tmpcpt[(v = *(pi + dx))] += 1;
			tmpcp0[v/16] += 1;
			pi += inclig;
		}
	} else if(flg < 0) {  /* debut de ligne : on compte seulement */
		dx -= 1;
		for(j=0;j<dy;j++) {
			tmpcpt[(v = *(pi + dx))] += 1;
			tmpcp0[v/16] += 1;
			pi += inclig;
		}
	} else {  /* fin de ligne : on enleve colonne prec. */
		pi--;
		for(j=0;j<dy;j++) {
			tmpcpt[(v = *pi)] -= 1;
			tmpcp0[v/16] -= 1;
			pi += inclig;
		}
	}
	j = n2;
	pv = tmpcp0;
	for(k=0;k<256;k += 16) {
		if(v = *pv++)
			if((j -= v) < 0)
				break;
	}
	
	pv = tmpcpt  + k;
	i = 16;
	j += v;
	while(i-- > 0) {
		if(v = *pv++)
			if((j -= v) < 0)
				return (15 - i + k);
	}
	return (k + 16);
}
median_init(pi,dx,dy,n2,inclig)
register unsigned char *pi;
register dx, dy, inclig, n2;
{
	register i, j;
	register v;
	register int *pv;
	inclig -= dx;
	for(j=0;j<dy;j++) {
		tmpcpt[(v = *pi++)] += 1;
		tmpcp0[v/16] += 1;
		for(i=1;i<dx;i++) {
			tmpcpt[(v = *pi++)] += 1;
			tmpcp0[v/16] += 1;
		}
		pi += inclig;
	}
}

#if 0
median(tabv,pi,dx,dy,n,n2,inclig)
unsigned char *tabv;
register unsigned char *pi;
register dx, dy, inclig;
register n, n2;
{
	register unsigned char *pv, *pv2;
	register i, j;
	register min, v, jmin;
	pv = tabv;
	while(dy-- > 0) {
		i = dx;
		while(i-- > 0)
			*pv++ = *pi++;
		pi += inclig;
	}
	pv = tabv;
	for(i=0;i<= n2; i++,pv++) {
		min = *pv;
		pv2 = pv+1;
		for(j=i+1;j<n;j++,pv2++) {
			if((v = *pv2) < min) {
				*pv = v;
				*pv2 = min;
				min = v;
			}
		}
	}
	return min;
}
#endif
