/* programmes de base d'acces aux headers images
desc_fmt(nf)  : remplit nf en fonction du header
fmt_desc(nf)  : remplit le header en fonction de nf.
*/

#include <stdio.h>
#include <stdlib.h>
#include <inrimage/inrfile.h>
#include <inrimage/inrlocal.h>

#define NB_VAR 16

/* le 1er bloc du header doit deja etre en memoire
retourne 0 si ok, -1 si pb de read (INR4), 1 si autre pb
*/
desc_fmt(nf)
struct image *nf;
{
	if(nf->hdr_data == (char *)0) return 1;  /* err si pas de header en memoire */
#ifndef NO_VISILOG
	if(nf->i_version == IV_VLOG) {  /* visilog */
		return vdesc_fmt(nf);
	}
#endif
#ifndef NO_INR4
	if(nf->i_version == IV_INR4)
		return i4desc_fmt(nf);
#endif
#ifndef NO_INR3
	/* image de type inrimage ancien */
	if(nf->i_version == IV_INR3 || nf->i_version == IV_OLD)
		return i3desc_fmt(nf);
#endif
	return 1;   /* version inconnue */
}

/* prepare un descripteur en fonction de ce qui est deja dans nf (lfmt, f_type,
i_version, desc_size, hdr_data)
complete nf (bits_lig, hors_tout, float_typ, flt_conv)
retourne 0, 1 si err
*/

fmt_desc(nf)
struct image *nf;
{
	register ist;
	while(1) {
#ifndef NO_VISILOG
		if(nf->i_version == IV_VLOG) { /* visilog */
			ist = fmt_vdesc(nf);
			break;
		}
		
#endif
#ifndef NO_INR3
		if(nf->i_version == IV_INR3 || nf->i_version == IV_OLD) { /* old Inrimage */
			ist = fmt_i3desc(nf);
			break;
		}
#endif
#ifndef NO_INR4
		if(nf->i_version == IV_INR4)  { /* new inrimage */
			ist =  fmt_i4desc(nf);
			break;
		}
#endif
		ist = 1;
		break;
	}
	nf->hdr_stat |= HST_TOWRITE;  /* a ecrire plus tard */
	return ist;  /* version inconnue */
}


/* alloc a bloc of line pointers for header 
returns the pointer (0 if error)
*/
char **hdrl_alloc(nf)
register struct image *nf;
{
	register char **h;
	register nb_var;
	h = nf->hdr_lines;
	nb_var = nf->hdrl_size + NB_VAR;
	if(h == (char **)0) {
		h = (char **) calloc(NB_VAR,sizeof(char *));
	} else {
		h = (char **) realloc(h,nb_var * sizeof(char *));
	}
	if(h == (char **) 0) return h;
	nf->hdr_lines = h;
	nf->hdrl_size = nb_var;
	return h;
}

/* initialise la liste des pointeurs de lignes */
bld_hdrlg(nf)
register struct image *nf;
{
	register ist;
	while(1) {
#ifndef NO_VISILOG
		if(nf->i_version == IV_VLOG) { /* visilog */
			ist = bld_hdrlv(nf);
			break;
		}
		
#endif
#ifndef NO_INR3
		if(nf->i_version == IV_INR3 || nf->i_version == IV_OLD) { /* old Inrimage */
			ist = bld_hdrl3(nf);
			break;
		}
#endif
#ifndef NO_INR4
		if(nf->i_version == IV_INR4)  { /* new inrimage */
			ist =  bld_hdrl4(nf);
			break;
		}
#endif
		ist = 1;
		break;
	}
	return ist;  /* version inconnue */
}
static char **header_lines;
static int nb_hlines;
/* on fabrique la liste des pointeurs vers les lignes non vides
   + 1pointeur par groupe de lignes vides 
   l'avant-dernier pointeur est egal au dernier si pas de place libre
   */
bld_hdrl(nf,p0)
register struct image *nf;
char *p0;
{
	register char *file_header, *p;
	register n, nb_var;
	register file_header_len;
	register flgnl;
	file_header = nf->hdr_data;
	p = p0;
	file_header_len = nf->desc_size;
	nf->first_hist = -1;
	nb_hlines = 0;
	flgnl = 1;
	if((header_lines = hdrl_alloc(nf)) == (char **)NULL) {
		perror("read_header");
		return(1);
	}
	nb_var = nf->hdrl_size;
	while ( p < file_header + file_header_len ) {
		if ( *p == '\n' || *p == 0) {
			/*!!!		*p = '\0'; on borne la chaine avec un nul */
			flgnl++;
			if(flgnl != 2) {
				p++;
				continue;
			}
		} else {
			if(!flgnl) {
				p++;
				continue;
			}
			flgnl = 0;
		}
		if ( nb_hlines >= nb_var ) {
			header_lines = hdrl_alloc(nf);
			if ( header_lines == NULL ) {
				perror("read_header");
				fprintf(stderr,"could'nt allocate %d variables\n",nb_var);
				return(1);
			}
			nb_var = nf->hdrl_size;
		}
		header_lines[nb_hlines++] = p;
		if(*p == '#') {
			register char *p1;
			p1 = p++;
			if((n = chkword(S_HISTORY,p1)) < 0)
				continue;
			nf->last_hist = nb_hlines -1;
			if(nf->first_hist < 0) nf->first_hist = nf->last_hist;
			p1 += n;
			if(*p1++ != '<') continue;
			nf->hnum2 = atoi(p1);
			if(nf->hnum1 <= 0) nf->hnum1 = nf->hnum2;
		} else
			p++;
	}
	if(nb_hlines < 2) {
		nb_hlines = 2;
		header_lines[0] = p0;
		header_lines[1] = file_header + file_header_len;
	}
	nf->hdr_nlines = nb_hlines;
	return 0;
}
