#include <inrimage/inrfile.h>

extern struct hdr_keys hdr_keys[];

/* find next matching line in header :
returns an index in hdr_lines (or -1)
key is the pattern to match, n0 is the 1st line (>= 0) where to begin 
never match last line, nor free line
don't match 1st line for Inrimage-4
*/
extern char **hdrl_alloc();

next_key(nf,key,n0)
register struct image *nf;
register char *key;
register n0;
{
	register char *p;
	register i;
	if(n0 >= nf->hdr_nlines -1) return -1;
	if(nf->i_version == IV_INR4) {
		if(n0 < 1)
			n0 = 1;
	} else {
		if(n0 < 0)
			n0 = 0;
	}
	for(i=n0;i<nf->hdr_nlines -1;i++) {
		p = nf->hdr_lines[i];
		if(p >= (nf->hdr_data + nf->desc_size))
			return -1;
		if(chkword(key,p) >= 0) {
			if(*p != 0 && *p != '\n')
				return i;
		}
	}
	return -1;
}

/* delete a line in the header
the line is given by its index in hdr_lines
returns num if ok, -1 if error
Can't del last line
Can't del 1st line for Inrimage-4
*/
del_hline(nf,num)
register struct image *nf;
register num;
{
	register char *p0, *p1, *pmax;
	register n, i;
	register nhl;
	if(num < 0 || num >= nf->hdr_nlines -1) return -1; 
	if(num < 1)
		if(nf->i_version == IV_INR4) return -1;
	p0 = nf->hdr_lines[num];
	p1 = nf->hdr_lines[num+1];
	n = (unsigned long)p1 - (unsigned long)p0;
	nhl = nf->hdr_nlines -1;
	pmax = nf->hdr_lines[nhl-1];
	nf->hdr_nlines--;
	/* suppress the pointer */
	for(i=num;i<nhl-1;i++)
		nf->hdr_lines[i] = nf->hdr_lines[i+1] - n;
	nf->hdr_lines[nhl-1] = nf->hdr_lines[nhl];
	/* and compress the data */
	while(p1 < pmax)
		*p0++ = *p1++;
	/* fill up */
	if(nf->i_version == IV_INR4)
		fillbuf(p0,(unsigned long)p1 - (unsigned long)p0,'\n');
	else
		fillbuf(p0,(unsigned long)p1 - (unsigned long)p0,0);
	
	nf->hdr_stat |= HST_TOWRITE;
	return num;
}

/* replaces the end of a line in the header
the line is given by its index in hdr_lines
returns num if ok, -1 if error
Can't replace last line, nor "empty" line
Can't replace 1st line for Inrimage-4
offset = 1st byte to replace
strg= new string (terminated by '\n' or '\0'
*/
repl_hline(nf,num,offset,strg)
register struct image *nf;
register num;
int offset;
char *strg;
{
	register char *p0, *p1, *pmax;
	register n, i;
	register nhl;
	int free, strl;
	nhl = nf->hdr_nlines -2;
	if(num < 0 || num >= nhl) return -1; 
	if(num < 1)
		if(nf->i_version == IV_INR4) return -1;
	p0 = nf->hdr_lines[num] + offset;
	p1 = nf->hdr_lines[num+1];
	n = (unsigned long)p1 - (unsigned long)p0;  /* old length */
	if(n < 0) return -1;
	strl = lgrline(strg);
	pmax = nf->hdr_lines[nhl];
	free = (unsigned long)nf->hdr_lines[nhl+1] - (unsigned long)pmax;
	if(free + n < strl +1) return -1;   /* not enough space */
	n -= (strl +1);  /* >0 = compress */
	if(n > 0) {   /* compress data */
		p0 = p1 - n;
		while(p1 < pmax)
			*p0++ = *p1++;
		/* fill up */
		if(nf->i_version == IV_INR4)
			fillbuf(p0,(unsigned long)p1 - (unsigned long)p0,'\n');
		else
			fillbuf(p0,(unsigned long)p1 - (unsigned long)p0,0);
	} else if(n < 0) {  /* expand */
		p0 = pmax - n -1;
		pmax--;
		while(pmax >= p1)
			*p0-- = *pmax--;
	}
	/* copy the data */
	p0 = nf->hdr_lines[num] + offset;
	while(strl-- > 0)
		*p0++ = *strg++;
	*p0++ = '\n';
	/* update the pointers */
	for(i=num+1;i<=nhl;i++)
		nf->hdr_lines[i] -= n;
	
	nf->hdr_stat |= HST_TOWRITE;
	return num;
}

/* add a string corresponding to a "key" in the header
we need the length of the 2 strings. A '\n' is appended to the line
The routine returns the index in table hdr_lines (of struct image nf) of the
new line.
if not enough space : return  -n where n is the number of missing chars
If truncate is 1, strg can be truncated by the program.
*/

add_hinfo(nf,key,keyl,strg,strl,truncate)
register struct image *nf;
register char *key, *strg;
register keyl, strl;
int truncate;
{
	register char *p;
	register n, num;
	if(*(strg + strl -1) == '\n') strl--;
	if(keyl < 0) keyl = 0;
	/* look for free space */
	if((num = nf->hdr_nlines-2) < 0) return -1;
	p = nf->hdr_lines[num];
	n = (unsigned long)nf->hdr_lines[nf->hdr_nlines-1] - (unsigned long)p;
	n -= keyl;
	if(n <= 0) return (n - strl -1);
	if(truncate <= 0 && n <= strl)
		return (n -strl -1);
	while(keyl-- > 0)
		*p++ = *key++;
	if(strl > n-1) strl = n -1;
	n -= strl - 1;
	while(strl-- > 0)
		*p++ = *strg++;
	*p++ = '\n';
	if(n <= 0) return num;
	if((n = nf->hdr_nlines) >= nf->hdrl_size)
		hdrl_alloc(nf);
	nf->hdr_lines[n] = nf->hdr_lines[n-1];
	nf->hdr_lines[n-1] = p;
	nf->hdr_nlines++;
	nf->hdr_stat |= HST_TOWRITE;
	return num;
}

/* check if key + buf is a history line 
returns offset in buf of 1st message char if yes, -1 if no 
*/
#define SZ (S_HISTORY + 10)
chk_hist(key,buf)
register char *key;
register char *buf;
{
	register char *p;
	register n;
	n = 0;
	init_hdrkeys();
	p = hdr_keys[IS_HISTORY].key;
	while(*p) {
		if(*key == 0) break;
		if(*key++ != *p++) return -1;
	}
	while(*p) {
		if(*buf++ != *p++) return -1;
		n++;
	}
	return n;
}
