/* user interface to image-header access : get, del, replace ... */
/*
	int idelhline(nf,key,num) 
	int ifreehdr(nf)
	int igethline(nf,key,num,buf,size,realsize)
	iwrcom_(anf,buf)
	c_iwrcom(nf,buf)
	int iputhline(nf,key,buf)
	int irephline(nf,key,num,buf)
*/
#include <inrimage/inrfile.h>
#include <string.h>

extern struct hdr_keys hdr_keys[];

/*
General info :
  a "key" is a null terminated string.
  except for igethline,
     if key does not begin with '#' it must end with '='
  Special work is done for history lines
/*
 int idelhline(nf,key,num) 
 struct image *nf; char *key; int num;
 start at line num (0=1st line) and delete next line begining by key
 if key matches a history line skip it and continue
returns num of next line 
-1 if no match
-2 if key is a reserved one

*/
/*
  int ifreehdr(nf)
  struct image *nf;
  returns nb of fre bytes in the header of nf
*/
/*
  int igethline(nf,key,num,buf,size,realsize)
  struct image *nf;char *key, *buf;int num, size, *realsize;
  starts at line num (>= 0), finds next line begining with key, and puts
  end of line (key excluded) in buffer buf of size "size" 
  as a null terminated string
 (ending  newline replaced by a null character)
  returns num of next line (num+1 of line found)
    -1 if no match
    puts real size of message in realsize
    message will be truncated in buf if realsize > size (last char 
    in buf will be null)
*/

/*
  subroutine iwrcom(nf,message)
  int c_iwrcom(nf,message)
  struct image *nf; char *message;
/*
  int iputhline(nf,key,buf)
  struct image *nf; char *buf;
  concatenates key and buf and puts the line buf at the end of the header
  the string in buf ends on null or '\n'
  key may be empty
  returns number of line created
  -1 if not enough space
  -2 if error or if key is a reserved one and is already present
  uses iwrhis_ if history line 
*/

/*
  int irephline(nf,key,num,buf)
  struct image *nf; char *key, *buf;int num;
  starts at line num and replaces remainding of line begining by key, by buf
  key must not be empty, but may be a reserved key
  returns num of next line if ok, -1 if no match, -2 if no space -3 if err
If key is not found, calls iputhline
*/


int idelhline(nf,key,num) 
register struct image *nf;
register char *key;
register int num;
{
	register n;
	n = strlen(key);
	if((*key == 0) || (*key != '#' && *(key +n -1) != '=')) return -3;
	if(tst_hdrkey(key) >= 0) return -2;
	while (1) {
		if((num = next_key(nf,key,num)) < 0) 
			return num;
		if(*key == '#')  /* check if history line */
			if(chk_hist("",nf->hdr_lines[num]) >= 0) {
				num++;
				continue;
			}
		break;
	}
	if((n = del_hline(nf,num)) < 0) return n;
	if(num < nf->last_hist)
		nf->last_hist--;
	if(num < nf->first_hist)
		nf->first_hist--;
	return n;
}

int ifreehdr(nf)
register struct image *nf;
{
	register n;
	if((n = nf->hdr_nlines -2) < 0) return 0;
	return ((unsigned long)nf->hdr_lines[n +1] - (unsigned long)nf->hdr_lines[n]);
}

int igethline(nf,key,num,buf,size,realsize)
register struct image *nf;
register char *key, *buf;
int num, size;
register *realsize;
{
	register n, i;
	register char *p;
	if((n = next_key(nf,key,num)) < 0) return n;
	p = nf->hdr_lines[n];
	if((i = chkword(key,p)) < 0)
		return -1;
	p += i;
	*realsize = move_line(p,buf,size);
	buf += *realsize -1;
	if(*realsize >= size) {
		*buf = 0;
	} else {
		if(*buf == '\n')
			*buf = 0;
		else {
			*(buf+1) = 0;
			*realsize += 1;
		}
	}
	return (n+1);
}

iwrcom_(anf,buf)
struct image **anf;
char *buf;
{
	c_iwrcom(*anf,buf);
}

c_iwrcom(nf,buf)
register struct image *nf;
register char *buf;
{
	return iputhline(nf,"#",buf);
}

int iputhline(nf,key,buf)
register struct image *nf;
register char *buf, *key;
{
	register n;
	n = strlen(key);
	if((*key == 0) || (*key != '#' && *(key +n -1) != '=')) return -2;
	if(*key != '#') {
		if(tst_hdrkey(key) >= 0)
			if(next_key(nf,key,0) >= 0)
				return -2;
	} else if((n = chk_hist(key,buf)) >= 0) {
		if(c_iwrhis(nf,buf+n) <= 0) return -2;
		return nf->last_hist;
	}
	if((n = add_hinfo(nf,key,strlen(key),buf,lgrline(buf),0)) < 0)
		return -1;
	return n;
}

int irephline(nf,key,num,buf)
register struct image *nf;
register char *key, *buf;
register int num;
{
	register n0, n;
	n0 = strlen(key);
	if((*key == 0) || (*key != '#' && *(key +n0 -1) != '=')) return -2;
	if((n = chk_hist(key,buf)) >= 0) return -3;
	while (1) {
		if((num = next_key(nf,key,num)) < 0) 
			return iputhline(nf,key,buf);
		if(*key == '#')  /* check if history line */
			if(chk_hist("",nf->hdr_lines[num]) >= 0) {
				num++;
				continue;
			}
		break;
	}
	if((n = repl_hline(nf,num,n0,buf)) < 0) return n;
	return (n +1);
}
