/* LIGBIT
	FUNCTION LIGBIT(IFMT,IBS)
	buffer_size(lfmt,nblig)
	buffer_pgsize(lfmt,nblig)
	buffer_align(buf)
	buffer_split(size,nb)
	buffer_pgsplit(size,nb)

	FUNCTION LIGBIT(IFMT,IBS)
  valeur=taille, en nb bits, de la ligne d'une image de format
    "reduit" IFMT.
    ifmt(1)=ndimx*ndimv, ifmt(2)=ndimy*ndimz
    ifmt(3)=byte size(>0) ou bit size(<0)
    ifmt(4)=type (1=float, 0=v fixe, -1=v.f. bit size packed)
  renvoie en outre IBS= nb bits reel par point (multiple de 8 si unpack
 ** genere err 9 si
  IFMT incorrect (e=3 si type incorrect, e=5 si type et size incompat.)

  buffer_size(lfmt,nblig)
  renvoie la taille "alignee" (c-a-d multiple de l'unite d'alignement,
  par ex utilise par lectpl)
  en octets d'un buffer contenant nblig ligne
  pour le format lfmt.

  buffer_pgsize(lfmt,nblig)
  renvoie la taille "subpage aligned" (c-a-d multiple de subpage size,
  en octets d'un buffer contenant nblig ligne
  pour le format lfmt.

  buffer_align(buf)
  renvoie le nb d'octets a ajouter a buf pour que son adresse soit alignee
  (multiple de long)

  buffer_split(size,nb)
  renvoie la taille de chacun des nb buffers se partageant size octets

  buffer_pgsplit(size,nb)
  renvoie la taille de chacun des nb buffers alignes sur subpage, se partageant size octets
*/
#include <inrimage/image.h>

Fort_int ligbit_(ifmt,ibs)
Fort_int *ifmt, *ibs;
{
	register n;
	n = (int)ifmt[I_BSIZE];
	if (n < 0) {
		n = -n;
		if(ifmt[I_TYPE] == FIXE) {
			n = 8 * ((n + 7) / 8);
		} else if(ifmt[I_TYPE] != PACKEE) 
			imerror(9,"ligbit: mauvais codage d'image\n");
	} else {
		if((ifmt[I_TYPE] != FIXE) && (ifmt[I_TYPE] != REELLE))
			imerror(9,"ligbit: mauvais codage d'image\n");
		if((ifmt[I_TYPE] == REELLE) && (n != sizeof(float)) && (n != sizeof(double)))
			imerror(9,"ligbit: mauvais codage flottant\n");
		n *= 8;
	}
	*ibs = n;
	return (Fort_int)(n * ifmt[I_DIMX]);
}


int buffer_size(lfmt,nlig)
Fort_int *lfmt;
int nlig;
{
	register k;
	Fort_int ibs;
	k = ligbit_(lfmt,&ibs) / 8;
	return ((((nlig * k) + sizeof(t_aligned) - 1) / sizeof(t_aligned)) * sizeof(t_aligned));

}

int buffer_pgsize(lfmt,nlig)
Fort_int *lfmt;
int nlig;
{
	register k;
	Fort_int ibs;
	k = ligbit_(lfmt,&ibs) / 8;
	return ((((nlig * k) + SUBPG_SIZE - 1) / SUBPG_SIZE) * SUBPG_SIZE);

}

int buffer_align(buf)
register char *buf;
{
	register unsigned long k;
	if(sizeof(unsigned long) < sizeof(char *))
		imerror(9,"buffer_align: cannot work on this system\n");
	k = (unsigned long)buf & (sizeof(t_aligned) - 1);
	return ( k > 0 ? sizeof(t_aligned) - k : 0);
}

int buffer_split(size,nb)
register int size, nb;
{
	register n;
	n = (size / nb) & ~(sizeof(t_aligned) - 1);
	return n;
}

int buffer_pgsplit(size,nb)
register int size, nb;
{
	register n;
	n = (size / nb) & ~(SUBPG_SIZE -1);
	return n;
}
