/*
C*OPTFMT
	SUBROUTINE OPTFMT(GFMT,IOPT)
	INTEGER GFMT(18), IOPT
   cherche les definitions de format dans la liste d'options
      options de taille si 'iopt & 2', de codage si 'iopt & 1'
      ( les 2 si iopt == 3)
      maille + offsets si iopt & 4
      bias + offset si iopt & 8 (images virg. fixe)
      retourne 1 si on les trouve, 0 sinon
*/
  
#include <inrimage/image.h>
#include <stdio.h>
#include <stdlib.h>

extern debug_;
#define abs(x) ( x < 0 ? -x : x)

optfmt_(gfmt,iopt)
Fort_int *iopt;
register struct nf_fmt *gfmt;
{
	register ist;
	if ( *iopt & VER_DIM ) { /* recherche des dimensions */
		 /* valeurs par defaut */
		gfmt->NDIMX =	gfmt->NDIMY =	gfmt->NDIMV = gfmt->NDIMZ = 1;
	}
	if ( *iopt & VER_OFF ) { /* recherche des offsets */
		 /* valeurs par defaut */
		gfmt->maille = 0;
		gfmt->X_OFFSET = gfmt->Y_OFFSET = gfmt->Z_OFFSET = 0;
	}

	if ( *iopt & VER_COD) { /* recherche du type et du codage */
		gfmt->BSIZE = 0; gfmt->TYPE = FIXE; gfmt->EXP = 0; /* valeurs par defaut */
	}
	if ( (*iopt & VER_XCOD) && gfmt->TYPE != REELLE) {
		/* recherche biais et scale */
		gfmt->bias = 0.;
		gfmt->scale = 0.;
	}

	if((ist = opt_modfmt(gfmt,(int)*iopt)) == 0) return ist; /* on n'a pas trouve de format */
	ver_gfmt(gfmt,(int)*iopt);  /* verif de coherence */
	return ist;
}

int c_optfmt( struct nf_fmt *gfmt, Fort_int iopt) {
  return optfmt_( gfmt, &iopt);
}

/* modifie le format en analysant les options de la commande
gfmt contient deja des valeurs par defaut
retourne 1 valeur >0 si on a trouve des options :
1 bit par champ de gfmt, bit de poids faible pour 1e champ, etc.
 */
#define F_SZ 4
#define F_TYP 8
#define F_X 0x10
#define F_Y 0x20
#define F_V 0x40
#define F_Z 0x80
#define F_EXP 0x100
#define F_X0 0x200
#define F_Y0 0x400
#define F_Z0 0x800
#define F_M 0x1000
#define F_BI 0x2000
#define F_SC 0x4000


static opt_dim(gfmt)
register struct nf_fmt *gfmt;
{
	register found= 0;
	int val;
	if(igetopt1("-x","%d",&val)) {
		if(val <= 0) val = 1;
		gfmt->NDIMX = (Fort_int)val;
		found |= F_X;
	}
	if(igetopt1("-y","%d",&val)) {
		if(val <= 0) val = 1;
		gfmt->NDIMY = (Fort_int)val;
		found |= F_Y;
	}
	if(igetopt1("-v","%d",&val)) {
		if(val <= 0) val = 1;
		gfmt->NDIMV = (Fort_int)val;
		found |= F_V;
	}
	if(igetopt1("-z","%d",&val)) {
		if(val <= 0) val = 1;
		gfmt->NDIMZ = (Fort_int)val;
		found |= F_Z;
	}
	gfmt->DIMX = gfmt->NDIMX * gfmt->NDIMV;
	gfmt->DIMY = gfmt->NDIMY * gfmt->NDIMZ;
	
	return found;
}
	
static opt_off(gfmt)
register struct nf_fmt *gfmt;
{
	register found= 0;
	int val;
	char buf[10];
	if(igetopt0x("-m=","%s",buf)) {
		if(buf[0] == 'h')
			gfmt->maille = I_HEXAG;
		else
			gfmt->maille = I_RECTA;
		found = F_M;
	}
	if(igetopt1("-x0","%d",&val)) {
		gfmt->X_OFFSET = (Fort_int)val;
		found |= F_X0;
	}
	if(igetopt1("-y0","%d",&val)) {
		gfmt->Y_OFFSET = (Fort_int)val;
		found |= F_Y0;
	}
	if(igetopt1("-z0","%d",&val)) {
		gfmt->Z_OFFSET = (Fort_int)val;
		found |= F_Z0;
	}
	return found;
}
static char *tabcod[]= {
	"-f","-p","-r" };
static int codval[]= {FIXE,PACKEE,REELLE};

static opt_type(gfmt,signe)
register struct nf_fmt *gfmt;
int signe;
{
	int val;
	register found = 0;
	register i;
	if((i = itoggleopt("-codtyp",tabcod,3)) >= 0) {
		found = F_TYP;
		gfmt->TYPE = codval[i];
	}
#if 0
	if(!found) {
		if ( igetopt0("-p") ) {
			found = F_TYP;
			gfmt->TYPE = PACKEE;
		}
	}
	if(!found) {
		if ( igetopt("-r","",0,"",0,"",0) ) {
			found = F_TYP;
			gfmt->TYPE = REELLE;
		} else if(igetopt("-r","%s",buff,"",0,"",0)) {
			found = F_TYP;
			gfmt->TYPE = REELLE;
		}
	}
#endif
	if ( igetopt1("-b","%d",&val) ) { {
		if(val <= 0) val = 1;
		gfmt->BSIZE = (Fort_int)val;
		found |= F_SZ;
	}
		gfmt->BSIZE = -gfmt->BSIZE;
	} else 
		if(igetopt1("-o","%d",&val)) {
			if(val <= 0) val = 1;
			gfmt->BSIZE = (Fort_int)val;
			found |= F_SZ;
		}
	
	if (gfmt->TYPE != REELLE &&  igetopt1("-e","%d",&val) ) {
		gfmt->EXP = (Fort_int)val;
		found |= F_EXP;
		gfmt->EXP += 200;
	}
	if ( igetopt0("-s") ) {
		found |= F_EXP;
		signe = -1;
	}
	
	/* calcul des valeurs par defaut */
	
	gfmt->EXP = ( gfmt->EXP == 0 ? 200 : gfmt->EXP );
	gfmt->EXP *= signe;
	
	if ( gfmt->BSIZE == 0 )
		switch ( gfmt->TYPE ) {
		case REELLE :	gfmt->BSIZE = sizeof(float); /* 4 (ou 8) octets */
			break;
			
		case FIXE :
			gfmt->BSIZE = 1; /* 1 octet */
			break;
			
		case PACKEE :
			gfmt->BSIZE = -1; /* 1 bit */
			break;
		}
	
	return found;
}

static opt_bsc(gfmt)
register struct nf_fmt *gfmt;
{
	register found = 0;
	if(igetopt1("-imbi","%f",&gfmt->bias))
		found |= F_BI;
	if(igetopt1("-imsc","%f",&gfmt->scale))
		found |= F_SC;
	return found;
}

opt_modfmt(gfmt,iopt)
register struct nf_fmt *gfmt;
register iopt;
{
	register signe;
	register found = 0;

	if ( iopt & VER_DIM ) { /* recherche des dimensions */
		found |= opt_dim(gfmt);
	}
	if ( iopt & VER_OFF ) { /* recherche des offsets */
		found |= opt_off(gfmt);
	}

	if ( iopt & VER_COD) { /* recherche du type et du codage */
		signe = gfmt->EXP < 0 ? -1 : 1;
		found |= opt_type(gfmt,signe);
	}
	if ( iopt & VER_XCOD && gfmt->TYPE != REELLE) {
		found |= opt_bsc(gfmt);
		if(found & F_SC && gfmt->scale != 0.) {  /* scale force exp a 1 */
			if(gfmt->EXP < 0)
				gfmt->EXP = -200;
			else
				gfmt->EXP = 200;
		}
	}
	return found;
}

/* verifie la coherence des parties du format generalise gfmt
demandees par iopt 
appelle imerror en cas de pb */
ver_gfmt(gfmt,iopt)
register struct nf_fmt *gfmt;
register iopt;
{
	if(iopt & VER_DIM) {
		/* coherence des dimensions */
		if ( gfmt->DIMX <=0 || gfmt->DIMY <=0 ) {
			imerror(7,"%s optfmt:dimensions erronnees\n",
				xargv[0]);
			exit(-1);
		}
	}
	if(!(iopt & VER_COD)) return 1 /* makes c99 happy */; 
	/* coherence du type et du codage */
	
	switch (gfmt->TYPE)  {
	case PACKEE :	/* virgule fixe packee */
		if ( gfmt->BSIZE > 0 ) {
			imerror(6,"%s optfmt:bit size erronnee\n",xargv[0]);
			exit(-1);
		}
	case FIXE :	/* virgule fixe non packee */
		if(gfmt->EXP == 0) break;
		if ( abs(gfmt->EXP) < 100 || abs(gfmt->EXP) > 300 ) {
			imerror(6,"%s optfmt:exposant erronne\n",xargv[0]);
			exit(-1);
		}
		break;
		
	case REELLE :	/* virgule flottante */
		
		if ( gfmt->BSIZE != sizeof(float) && gfmt->BSIZE != sizeof(double) ) {
			imerror(6,"%s optfmt:byte size erronnee\n",xargv[0]);
			exit(-1);
		}
		break;
	}

}
