#include <stdio.h>
#include <inrimage/image.h>
#include <inrimage/flt_type.h>

/* procedure de compactage 
   subroutine pckbt(in,out,nb,bsize)
   c_pckbt(in,out,nb,bsize)
   permet de passer d'un tableau de pixels stockes en format etendu
   ( les bits occupent un nb entier d'octets)
   a un tableau en format compact

   le tableau d'entree et de sortie peuvent etre identique

chaque octet de in est reparti entre octet courant et suivant de out
*/

pckbt_(in,out,nb,bsize)
register CHAR *in,*out;
Fort_int *nb;
register Fort_int *bsize;
{
	c_pckbt(in,out,(int)*nb,(int)*bsize);
}

c_pckbt(in,out,nb,bsize)
register CHAR *in,*out;
int nb;
register int bsize;
{
	unsigned char tmpb[4];
	register bpix;  /* nb bytes / pixel */
	register unsigned int val, v;
	register unsigned char msk;  /* high bits mask */
	register int pad;
	register ibits;   /* nb of bits in input byte */
	register obits; /* nb bits used in output byte */
	register bits1;  /* nb of input bits going to 1st byte */
	register i, j;
	if((bpix = (bsize + 7) / 8) <= 0)
		imerror(9,"pckbt_ : bad nb of bits\n");
	pad = bsize % 8;
	if(pad)
		pad = 8 - pad;
	v = 0;
	obits = 0;
	if(bpix == 1) {
		ibits = 8 - pad;
		for(i=0;i<nb;i++) {
			val = *in++ << pad;
			if((bits1 = 8 - obits) > ibits)
				bits1 = ibits; /* all input bits in 1st byte */
			msk = 0xff << (8 - bits1);
			v |= ((val & msk) >> obits);
			if((obits += bits1) >= 8) {
				*out++ = v;
				if(bits1 < ibits) {
					obits = ibits - bits1;
					v = (val & ~msk) << bits1;
				} else {
					obits = 0;
					v = 0;
				}
			}
		}
		if(obits)
			*out = v;
		return 1 /* makes c99 happy */; 
	}
	
	/* > 1 byte : put most significant bits first */
	if(ISMSBFIRST) {  /* the system has msb first */
		for(i=0;i<nb;i++) {
			val = *in++ << pad;
			j = 0;
			ibits = 8 - pad;
			while(1) {
				j++;
				if((bits1 = 8 - obits) > ibits)
					bits1 = ibits; /* all input bits in 1st byte */
				msk = 0xff << (8 - bits1);
				v |= ((val & msk) >> obits);
				if((obits += bits1) >= 8) {
					*out++ = v;
					obits -= 8;
					if(bits1 < ibits) {
						obits = ibits - bits1;
						v = (val & ~msk) << bits1;
					} else {
						obits = 0;
						v = 0;
					}
				}
				if(j >= bpix) break;
				val = *in++;
				ibits = 8;
			}
		}
		if(obits)
			*out = v;
		return 1 /* makes c99 happy */; 
	}
	/* cpu has LSB first : swap ; we must use a temporary buffer */
	for(i=0;i<nb;i++) {
		for(j=bpix-1;j>=0;j--)
			tmpb[j] = *in++;
		val = tmpb[0] << pad;  /* get MSB */
		ibits = 8 - pad;
		for(j=1;j<=bpix;j++) {
			if((bits1 = 8 - obits) > ibits)
				bits1 = ibits; /* all input bits in 1st byte */
			msk = 0xff << (8 - bits1);
			v |= ((val & msk) >> obits);
			if((obits += bits1) >= 8) {
				*out++ = v;
				obits -= 8;
				if(bits1 < ibits) {
					obits = ibits - bits1;
					v = (val & ~msk) << bits1;
				} else {
					obits = 0;
					v = 0;
				}
			}
			val = tmpb[j];
			ibits = 8;
		}
	}
	if(obits)
		*out = v;
}
