/*
 * tbvenp_(), tbpenv_() - 
 *      fonctions documentes dans Inrimage mais introuvables (tbvenp.3i), 
 *      le source a disparu ?
 *      tentative de rcriture from scratch (en C).
 * et ajout des fonctions c_tbvenp(), c_tbpenv(), c_tbvtp1() et c_tbp1tv().
 */

#include <inrimage/image.h>

extern int debug_;

/* a priori, une erreur sur iv + nbv - 1 */

/**
 * @param a tableau de vecteurs  lire.
 * @param b tableau scalaire  crire.
 * @param npts nombre de vecteur  lire dans a.
 * @param nbits taille, en bits, d'une composante d'un vecteur.
 * @param ndimv nombre de composante d'un vecteur.
 * @param iv indice de la premire composante de b  crire.
 * @param nbv nombre de composantes  crire.
 *
 * c_tbvenp() spare les composantes iv  iv + nbv -1 des npts
 * vecteurs de dimension ndimv contenus dans a, en les rpartissant dans
 * b. Le tableau b est dcoup en nbv tableaux lmentaires commenant
 * sur une frontire d'lment de type t_aligned. Chaque composante est
 * code sur nbits bits.  iv est compt  partir de 1.
 *
 * @sa c_tbvtp1(), c_tbpenv()
 */

int c_tbvenp( char *a, char *b, int npts, int nbits, int ndimv, int iv, int nbv) {
  int incr, z;
  
  /* calcul de la taille : les plans sont aligns sur 1 mot */
  incr = (((nbits * npts + 7) / 8 + sizeof(t_aligned) - 1) / sizeof(t_aligned)) * sizeof(t_aligned);
  if( debug_) fprintf(stderr, "tbvenp: incr=%d\n", incr);
  
  for( z=0; z<nbv; z++) {
    c_tbvtp1( a, b, iv+z, ndimv, npts, nbits);
    b += incr;
  }
  return 0;
}



/* ICI je modifie la signature car pour moi ca ne va pas
 * (non appelable depuis fortran) ASK TO JEAN-PAUL */

/**
 * @brief version de c_tbenvp() appelable pour FORTRAN.
 * @sa c_tbenvp()
 */

int tbvenp_( char *a, char *b, Fort_int *npts, Fort_int *nbits, Fort_int *ndimv, Fort_int *iv, Fort_int *nbv) {
  return c_tbvenp( a, b, *npts, *nbits, *ndimv, *iv, *nbv);  
}


/* autres fonctions a coder (et a mettre dans 
   des modules spars quand ce sera fini). */

/**
 * @param a tableau,  lire, de scalaire.
 * @param b tableau,  crire, de vecteurs.
 * @param npts nombre de valeurs  lire dans a.
 * @param nbits taille, en bits, des valeurs  lire.
 * @param ndimv nombre de composantes des vecteurs de b.
 * @param iv indice de la premiere composante de b  crire.
 * @param nbv nombre de composantes de b  crire.

 * tbpenv (tbpenv_) regroupe, dans b, les composantes iv  iv - nbv
 * -1, codes sur nbits bits, des npts vecteurs de dimension ndimv, 
 * partir de nbv tableaux rpartis dans a. a est dcoup en nbv tableaux
 * de taille multiple de sizeof(t_aligned) (4 ou 8 octets selon les
 * systmes).  iv est compt  partir de 1.
 */

int c_tbpenv( char *a, char *b, int npts, int nbits, int ndimv, int iv, int nbv) {
  int incr, z;
  
  /* calcul de la taille : les plans sont aligns sur 1 mot */
  incr = (((nbits * npts + 7) / 8 + sizeof(t_aligned) - 1) / sizeof(t_aligned)) * sizeof(t_aligned);
  if( debug_) fprintf(stderr, "tbvenp: incr=%d\n", incr);
  
  for( z=0; z<nbv; z++) {
    c_tbp1tv( a, b, iv+z, ndimv, npts, nbits);
    a += incr;    
  }
  return 0;
}


/* ICI je modifie la signature car pour moi ca ne va pas
 * (non appelable depuis fortran) ASK TO JEAN-PAUL */

/**
 * @brief version de c_tbpenv() appelable pour FORTRAN.
 * @sa c_tbpenv().
 */

int tbpenv_( char *a, char *b, Fort_int *npts, Fort_int *nbits, Fort_int *ndimv, Fort_int *iv, Fort_int *nbv) {
  return c_tbpenv( (char *)a, (char *)b, *npts, *nbits, *ndimv, *iv, *nbv);  
}



/**
 * @brief criture d'une composante dans un tampon vectoriel.
 *
 * @param a tampon  lire (tableau de scalaires).
 * @param b tampon  crire (tableau de vecteurs).
 * @param iv composante  crire dans b.
 * @param ndimv  nombre de composantes des vecteurs de b.
 * @param nbv nombre d'lements  lire dans a (et  crire dans b).
 * @param nbits taille en bit des valeurs de a (et b).
 *
 * c_tbp1tv() charge la composante iv des nbv vecteurs  ndimv composantes 
 * (nbits par valeur) contenus dans b,  partir des valeurs lues dans a. 
 * 
 * en gros, b[iv] <- a.
 */

int c_tbp1tv( char *a, char *b, int iv, int ndimv, int nbv, int nbits) {
  Fort_int *piv = &iv, *pndimv = &ndimv, *pnbv = &nbv, *pnbits = &nbits;
  tbp1tv_( a, b, piv, pndimv, pnbv, pnbits);
  return 0;
}

/**
 * @brief Lecture d'une composante d'un tampon vectoriel.
 *
 * @param a tampon  lire (tableau de vecteurs).
 * @param b tampon  crire (tableau de scalaire).
 * @param iv composante  lire dans a.
 * @param ndimv nombre de composantes des vecteurs de a.
 * @param nbv nombre d'lements dans a (et b).
 * @param nbits taille en bit des valeurs de a (et b).
 *
 * c_tbvtp1 (tbvtp1_) transfre dans b, la composante iv des nbv vecteurs de 
 * dimensions ndimv contenus dans a. Chaque lment tant cod sur nbits bits. 
 *
 * en rsum: b <- a[iv]
 */

int c_tbvtp1( char *a, char *b, int iv, int ndimv, int nbv, int nbits) {
  Fort_int *piv = &iv, *pndimv = &ndimv, *pnbv = &nbv, *pnbits = &nbits;
  tbvtp1_( a, b, piv, pndimv, pnbv, pnbits);
  return 0;
}

