/* transformation d'un tableau en image 8 bits pour visualisation  *
/*
	SUBROUTINE CNVVIS(A,B,NLIG,LFMT,IEXPO,IREST,IFOND,IPAS)
	c_cnvvis(a,b,nlig,lfmt,iexpo,irest,ifond,ipas)
  transforme un tableau A en B 8 bits pour visualisation
   NLIG=nb lignes source
  LFMT= format de l'image
  IREST nb de pixels (=> NDIMV * IREST valeurs) de valeur IFOND (0-255) a rajouter au
  bout de chaque ligne 
  IEXPO : exposant pour le resultat (2**n) . de -128 a + 128
 IPAS= pas d'echantillonage sur lignes et colonnes (1 sur ipas, >=1)
   si IPAS<0 on duplique points et lignes -IPAS fois
  Le transfert commence par le debut si pas > 0, par la fin si pas < 0. 
  A doit etre convenablement place par rapport a B
l'echantillonnage tient compte de NDIMV mais pas de NDIMZ
(il faut appeler le programme pour chaque plan)
Les composantes vectorielles sont sparees par lignes : ndimx comp1, ndimx comp2, ...
*/

/* ATTENTION : passage par cnvtbg, donc si le format est packe
		les tailles doivent etre calculees pour le format unpacked */


#include <stdio.h>
#include <inrimage/image.h>


/* types : 1, 2, 4 octets , signe ou Non Signe */
#define T_FLOAT4 0
#define T_S1 0x11
#define T_NS1 1
#define T_S2 0x12
#define T_NS2 2
#define T_S4 0x14
#define T_NS4 4
#define T_FLOAT8 5
#define T_SIGNED 0x10

extern double pow();
static int cnvgen(), cnvg1b(), echantillonne(), duplique(), xcopie();

static int nbyte_val; /* nb byte par valeur et par pixel */
static int nbits /* nb bits mantisse */, i_bits, /* nb bits efffectif */iexp, i_type;
static double  v_coef, v_coefn;
static double outmax, outmaxn;

c_cnvvisg(in,out,nb_lignes,lfmt,iv0,nbv,iexpo,reste,fond,pas)
CHAR *in,*out;
int nb_lignes,reste,fond,pas,iexpo, iv0, nbv;
Fort_int lfmt[9];
{
	register nbpt;
	register lgout, nbl_eff;
	register i, j;
	register double i_exp, V;
	if ( pas == 0 ) 
		imerror(9,"cnvvis, pas d'echantillonage nul !\n");
	if(iexpo < -128 || iexpo > 128)
		imerror(9,"cnvvis: exposant de sortie hors limites (-128...128)\n");

#if 0
	if(TYPE == PACKEE && BSIZE == -1)
		return cnv_1bit(in,out,nb_lignes,lfmt,reste,fond,pas);
#endif
	
	if(TYPE == PACKEE) {
		nbpt = nb_lignes * DIMX;
		c_unpkbt(in,in,nbpt,-BSIZE);
	}

	nbyte_val = BSIZE > 0 ? BSIZE : (7 - BSIZE)/8;
	i_bits = nbits = BSIZE < 0 ? -BSIZE : 8 * BSIZE;
	if(nbyte_val == 3)
		imerror(9,"cnvvis: codage 3 octets non acceptee\n");
	iexp = 0;
	if(TYPE == REELLE) {
		i_type = T_FLOAT4;
		if(BSIZE == 8)
			i_type = T_FLOAT8;
	} else {
		i_type = nbyte_val;
		if(EXP < 0) {
			i_type |= T_SIGNED;
			iexp = -200 - EXP;
			nbits--;
		} else
			iexp = EXP - 200;
	}
	outmax = 255. / pow((double)2.,(double)iexpo);
	outmaxn = 256. / pow((double)2.,(double)iexpo);
	if(TYPE != REELLE) {
		V = pow((double)2.,(double)i_bits);
		i_exp = pow((double)2.,(double)iexp);
		v_coef = outmax * (i_exp / (V - 1.));
		v_coefn = outmaxn * (i_exp / V);
	}
	reste *= NDIMV;
	if(pas > 0) { /* echantillonnage */
		if((pas == 1) && (BSIZE == 1) && (NDIMV == 1) && (iexp == 0) && (iexpo == 0))
			xcopie(in,out,nb_lignes,DIMX,reste);
		else {
			lgout = nbv * (1 + ((NDIMX - 1) / pas)) + reste;
			nbl_eff = 1 + ((nb_lignes -1) / pas);
			echantillonne(in,out,nb_lignes,lfmt,iv0,nbv,nbyte_val,reste,pas);
		}
	} else {  /* on duplique */
		pas = -pas;
		lgout = nbv * pas * NDIMX + reste;
		nbl_eff = nb_lignes * pas;
		duplique(in,out,nb_lignes,lfmt,iv0,nbv,nbyte_val,reste,pas);
	}
	/* mettre le fond */
	if(reste > 0) {
		for(i=0;i<nbl_eff;i++) {
			out += (lgout - reste);
			for(j=0;j<reste;j++) {
				*out++ = fond;
			}
		}
	}
}

static int xcopie(in,out,nbl,nbx,reste)
register CHAR *in, *out;
register nbl, nbx, reste;
{
	register i;
	if((in == out) && reste == 0) return 1 /* makes c99 happy */; 
	while(nbl-- > 0) {
		for(i=0;i<nbx;i++) {
			*out++ = *in++;
			out += reste;
		}
	}
		
}


/* reste = nb byte fin de ligne out */
static int echantillonne(in,out,nb_lignes,lfmt,iv0,nbv,nbyte_val,reste,pas)
CHAR *in;
CHAR *out;
int nb_lignes;
Fort_int lfmt[9];
int pas, reste, nbyte_val, iv0, nbv;
{
	register nxo;
	int nyo;
	register incxi, incyi;
	register i;
	nxo = 1 + (NDIMX -1) / pas;
	nyo = 1 + (nb_lignes -1) / pas;
	incxi = nbyte_val * NDIMV * (pas -1);
	incyi = pas * DIMX * nbyte_val;
	cnvgen(in,out,nyo,nxo,(int)NDIMV,iv0,nbv,incxi,0,incyi,reste,nxo);
}

/* reste = nb byte fin de ligne out */
static int duplique(in,out,nb_lignes,lfmt,iv0,nbv,nbyte_val,reste,pas)
register CHAR *in;
CHAR *out;
int nb_lignes;
Fort_int lfmt[9];
int nbyte_val, iv0, nbv;
int pas, reste;
{
	CHAR *po, *p2;
	int i, j;
	int incxo, incyi, incyo;
	int lgligo, lglig1;
	lglig1 = pas * NDIMX * nbv;
	lgligo = lglig1 + reste;
	incyi = -(DIMX * nbyte_val);
	incxo = 1 - pas;
	incyo = -(reste * pas + (pas -1) * (NDIMX * nbv) * pas);
	out += nb_lignes * pas * lgligo - reste - 1;
	cnvgen(in + (nb_lignes * DIMX - 1) * nbyte_val,out,
	       nb_lignes,NDIMX,NDIMV,iv0,nbv,0,incxo * nbv,incyi,incyo,pas*NDIMX);
	incyi = (pas - 1) * NDIMX * nbv;
	incyo = lgligo;
	incxo = -incxo;
	while(nb_lignes-- > 0) {
		i = NDIMX * nbv;
		p2 = po = out;
		while(i-- > 0) {	
			j = incxo;
			p2--;
			while(j-- > 0)
				*p2-- = *po--;
			po = p2;
		}
		/* dupliquer les lignes */
		i = pas -1;
		while(i-- > 0) {
			po -= reste;
			for(j=0;j<lglig1;j++) {
				*po-- = *out--;
			}
			out -= reste;
		}
		out = po;
	}
}

/* si incyi < 0 on commence par la fin */
static int cnvgen(in,out,nbl,nbx,ndimv,iv0,nbv,incxi,incxo,incyi,incyo,nbxo)
CHAR *in;
register CHAR *out;
register nbl;
int nbx;
int incyi, incyo, nbxo, ndimv, iv0, nbv;
register incxi, incxo;
{
	register CHAR *pi;
	CHAR *po, *po1, *tmpbuf, *ptmpbuf;
	register double V;
	register j, i;
	register val, i_sval, i_sign;
	register vmax;
	int incr_start, incr_end;
	register float omax;
	if((omax = outmax) > 255.)
		omax = 255.;
	vmax = (int)omax;
	i_sign = 1 << nbits;
	i_sval = 0xffffffff << nbits;
	ptmpbuf = NULL;
	if(ndimv > 1) {
	  tmpbuf = ptmpbuf = i_malloc(nbxo*nbv);
	  if(incyi < 0) tmpbuf += (nbxo*nbv - 1);
	} else  po = out;
	if(incyi < 0) {
	  incr_start = (iv0 + nbv - ndimv) * nbyte_val;
	  incr_end = - (iv0 * nbyte_val);
	} else {
	  incr_end = (ndimv - iv0 -nbv) * nbyte_val;
	  incr_start = iv0 * nbyte_val;
	}
	while(nbl-- > 0) {
		pi = in;
		i = nbx;
		if(ndimv > 1) po = tmpbuf;
		while(i-- > 0) {
		  j = nbv;
		  pi += incr_start;
			while(j-- > 0) {
				switch(i_type) {
				case T_FLOAT4:
					V = (double) *((float *)pi) * outmax;
					break;
				case T_FLOAT8:
					V = *(double *)pi * outmax;
					break;
				case T_NS1:
					V = ((double) *(unsigned char *)pi) * v_coef;
					break;
				case T_S1:
					if((val = *(char *)pi) & i_sign)
						V = ((double)(val | i_sval)) * v_coefn;
					else
						V = (double)val * v_coef;
					V += 128.;  /* ramener en positif */
					break;
				case T_NS2:
					V = ((double) *(unsigned short *)pi) * v_coef;
					break;
				case T_S2:
					if((val = *(short *)pi) & i_sign) 
						V = ((double) (val | i_sval)) * v_coefn;
					else
						V = ((double) val) * v_coef;
					V += 128.;  /* ramener en positif */
					break;
				case T_NS4:
					V = ((double) *(u_int4 *)pi) * v_coef;
					break;
				case T_S4:
					if((val = *(int4 *)pi) & i_sign)
						V = ((double)(val | i_sval)) *  v_coefn;
					else
						V = (double)val * v_coef;
					V += 128.;  /* ramener en positif */
					break;
					
				}
				if((V += .5) < 0.)
					*po = 0;
				else {
#if 0
					if(V > 255.)
						*po = 0xff;
#else
					if(V > omax)
						*po = vmax;
#endif
					else
						*po = (unsigned char) V;
				}
				if(incyi < 0)  {
					pi -= nbyte_val;
					po--;
				} else {
					pi += nbyte_val;
					po++;
				}
			}
			po += incxo;
			pi += incxi + incr_end;
		}
		if(ndimv > 1) {
		  po1 = tmpbuf;
		  j = nbv;
		  while(j-- > 0) {
		    i = nbxo;
		    po = po1;
		    if(incyi < 0) {
		      while(i-- > 0) {
			*out-- = *po;
			po -= nbv;
		      }
		      po1--;
		    } else {
		      while(i-- > 0) {
			*out++ = *po;
			po += nbv;
		      }
		      po1++;
		    }
		  }
		  out += incyo;
		} else 
		  po += incyo;
		in += incyi;
	}
	i_Free(&ptmpbuf);
}

