#!/bin/bash
# (c) 2017 D.Bereziat

source test-tools

inittest "cims" 1.0

label scal "simple 2D scalar image"
myexec 'cims -x 3 -y 3 -f | tpr -l 3 -c > out.txt' <<EOF
1 1 64
2 2 128
3 3 255
EOF
compare out.txt <<EOF
64 0 0
0 128 0
0 0 255
EOF

label err_coord "missing coordinate"
myexec 'cims -x 3 -y 3 -f 2> err.txt >/dev/null' <<EOF
1 1 64
2 2 128
3
EOF
compare err.txt <<EOF
Bad format at line 3 (expected: x y values)
EOF

label warn_coord "coordinate out of domain"
myexec "cims -x 3 -y 3 -f 2> err.txt >/dev/null" <<EOF
1 1 64
2 2 128
3 4 255
EOF
compare err.txt <<EOF
Warning line 3: coord (3,4) outside the domain, values ignored
EOF

label scal_seq "simple scalar sequence"
myexec "cims -x 3 -y 3 -f -z 3 | tpr -l 3 -cz > out.txt" <<EOF
1 1 1 64
2 2 2 128
3 3 3 255
EOF
compare out.txt <<EOF
plan 1
64 0 0
0 0 0
0 0 0
plan 2
0 0 0
0 128 0
0 0 0
plan 3
0 0 0
0 0 0
0 0 255
EOF

label vect "vector image"
myexec "cims -x 3 -y 3 -f -v 3 | tpr -l 9 -cz >out.txt" <<EOF
1 1 128 128 128
2 2 255 0 255 
3 3 255 0 0
EOF
compare out.txt <<EOF
128 128 128 0 0 0 0 0 0
0 0 0 255 0 255 0 0 0
0 0 0 0 0 0 255 0 0
EOF

label vect_err "vector image with missing values"
myexec "cims -x 3 -y 3 -f -v 3 2>err.txt" <<EOF
1 1 128 128 128
2 2 255 0  
3 3 255 0 0
EOF
compare err.txt <<EOF
read value error: missing a value at line 2
EOF

label ind0 "coordinates start from 0"
myexec "cims -x 3 -y 3 -f -z 3 -b0 | tpr -l 3 -cz > out.txt" <<EOF
0 0 0 64
1 1 1 128
2 2 2 255
EOF
compare out.txt <<EOF
plan 1
64 0 0
0 0 0
0 0 0
plan 2
0 0 0
0 128 0
0 0 0
plan 3
0 0 0
0 0 0
0 0 255
EOF

label mod "modify existing image" 
myexec "cims -x 3 -y 3 -f > test$current_test.inr" <<EOF
1 1 64
EOF
#tpr -l 3 -c test1.inr
myexec "cims -x 3 -y 3 -f test$current_test.inr" <<EOF
2 2 128
EOF
myexec "tpr -l 3 -c test$current_test.inr >out.txt"
compare out.txt <<EOF
64 0 0
0 128 0
0 0 0
EOF
rm test$current_test.inr


# test format & conversion (ref to cim)
label B1 "format 1-byte"
for a in {0..255}; do echo $((a+1)) 1 $a; done | cims -x 256 | tpr -c -l 256 > out.txt
compare out.txt <<< $(seq 0 255 | cim -x 256 | tpr -c -l 256)

label B1float "format 1-byte with float input"
for a in {0..255}; do echo $((a+1)) 1 $(echo "$a/255" | bc -l); done | cims -x 256 -f=%f | tpr -c -l 256 > out.txt
# /!\ cim semble buggée avec -f=%f pour une image 1 octet 
compare out.txt <<< $(seq 0 255 | cim -x 256 | tpr -c -l 256)

label S1 "format 1-byte signed"
for a in {0..255}; do echo $((a+1)) 1 $a; done | cims -x 256 -s | tpr -c -l 256 > out.txt
compare out.txt  <<< $(seq 0 255 | cim -x 256 -s | tpr -c -l 256)

label B2 "format 2-byte"
myexec 'i=1; for a in 0.{0..9} 1.0; do echo $((i++)) 1 $a; done | cims -x 11 -o 2 | tpr -c -l 11 > out.txt'
case $(uname) in Darwin) recho "FIXME: /!\ cim plante sur OSX";; esac
compare out.txt <<< $(for a in 0.{0..9} 1.0; do echo $a; done | cim -x 11 -o 2 | tpr -c -l 11)

label S2 "format 2-byte signed " # ko sur osx
let i=1
for a in 0.{0..9} 1.0; do echo $((i++)) 1 -$a; done | cims -x 11 -o 2 -s | tpr -c -l 11 > out.txt
case $(uname) in Darwin) recho "FIXME: /!\ cim plante sur OSX";; esac
compare out.txt <<< $(for a in 0.{0..9} 1.0; do echo -$a; done | cim -x 11 -o 2 -s | tpr -c -l 11)

label B4 "format 4-byte"
let i=1
for a in 0.{0..9} 1.0; do echo $((i++)) 1 $a; done | cims -x 11 -o 4 | tpr -c -l 11 > out.txt
compare out.txt <<< $(for a in 0.{0..9} 1.0; do echo $a; done | cim -x 11 -o 4 | tpr -c -l 11)

label S4 "format 4-byte signed "
let i=1
for a in 0.{0..9} 1.0; do echo $((i++)) 1 -$a; done | cims -x 11 -o 4 -s | tpr -c -l 11 > out.txt
compare out.txt <<< $(for a in 0.{0..9} 1.0; do echo -$a; done | cim -x 11 -o 4 -s | tpr -c -l 11)

for b in {1..7}; do
    label b$b "format $b-bit"
    for a in {0..255}; do echo $((a+1)) 1 $a; done | cims -x 256 -b $b | tpr -c -l 256 > out.txt
    compare out.txt <<< $(seq 0 255 | cim -x 256 -b $b | tpr -c -l 256)

    label p$b "format $b-bit packed"
    for a in {0..255}; do echo $((a+1)) 1 $a; done | cims -x 256 -p $b | tpr -c -l 256 > out.txt
    compare out.txt <<< $(seq 0 255 | cim -x 256 -p $b | tpr -c -l 256)
done

label float "format float"
let i=1
for a in $(seq -10 10); do echo $((i++)) 1 $a; done | cims -x 21 -r | tpr -c -l 21 > out.txt
compare out.txt <<< $(for a in $(seq -10 10); do echo $a; done | cim -x 21 -r | tpr -c -l 21)



